@@ -33,17 +33,16 @@ Document::Document() {
 	dbcsCodePage = 0;
 	stylingBits = 5;
 	stylingBitsMask = 0x1F;
-	stylingPos = 0;
 	stylingMask = 0;
-	for (int ch = 0; ch < 256; ch++) {
-		wordchars[ch] = isalnum(ch) || ch == '_';
-	}
+	SetWordChars(0);
 	endStyled = 0;
 	enteredCount = 0;
 	enteredReadOnlyCount = 0;
 	tabInChars = 8;
 	indentInChars = 0;
 	useTabs = true;
+	tabIndents = true;
+	backspaceUnindents = false;
 	watchers = 0;
 	lenWatchers = 0;
 
@@ -150,8 +149,8 @@ int Document::VCHomePosition(int position) {
 int Document::SetLevel(int line, int level) {
 	int prev = cb.SetLevel(line, level);
 	if (prev != level) {
-		DocModification mh(SC_MOD_CHANGEFOLD | SC_MOD_CHANGEMARKER, 
-			LineStart(line), 0, 0, 0);
+		DocModification mh(SC_MOD_CHANGEFOLD | SC_MOD_CHANGEMARKER,
+		                   LineStart(line), 0, 0, 0);
 		mh.line = line;
 		mh.foldLevelNow = level;
 		mh.foldLevelPrev = prev;
@@ -181,7 +180,7 @@ int Document::GetLastChild(int lineParent, int level) {
 	if (lineMaxSubord > lineParent) {
 		if (level > (GetLevel(lineMaxSubord + 1) & SC_FOLDLEVELNUMBERMASK)) {
 			// Have chewed up some whitespace that belongs to a parent so seek back
-			if ((lineMaxSubord > lineParent) && (GetLevel(lineMaxSubord) & SC_FOLDLEVELWHITEFLAG)) {
+			if (GetLevel(lineMaxSubord) & SC_FOLDLEVELWHITEFLAG) {
 				lineMaxSubord--;
 			}
 		}
@@ -218,7 +217,6 @@ bool Document::IsCrLf(int pos) {
 	return (cb.CharAt(pos) == '\r') && (cb.CharAt(pos + 1) == '\n');
 }
 
-#if PLAT_WIN
 bool Document::IsDBCS(int pos) {
 	if (dbcsCodePage) {
 		if (SC_CP_UTF8 == dbcsCodePage) {
@@ -231,7 +229,7 @@ bool Document::IsDBCS(int pos) {
 			while (startLine > 0 && cb.CharAt(startLine) != '\r' && cb.CharAt(startLine) != '\n')
 				startLine--;
 			while (startLine <= pos) {
-				if (IsDBCSLeadByteEx(dbcsCodePage, cb.CharAt(startLine))) {
+				if (Platform::IsDBCSLeadByte(dbcsCodePage, cb.CharAt(startLine))) {
 					startLine++;
 					if (startLine >= pos)
 						return true;
@@ -242,13 +240,6 @@ bool Document::IsDBCS(int pos) {
 	}
 	return false;
 }
-#else
-// PLAT_GTK or PLAT_WX
-// TODO: support DBCS under GTK+ and WX
-bool Document::IsDBCS(int) {
-	return false;
-}
-#endif
 
 int Document::LenChar(int pos) {
 	if (IsCrLf(pos)) {
@@ -301,7 +292,6 @@ int Document::MovePositionOutsideChar(int pos, int moveDir, bool checkLineEnd) {
 
 	// Not between CR and LF
 
-#if PLAT_WIN
 	if (dbcsCodePage) {
 		if (SC_CP_UTF8 == dbcsCodePage) {
 			unsigned char ch = static_cast<unsigned char>(cb.CharAt(pos));
@@ -323,14 +313,14 @@ int Document::MovePositionOutsideChar(int pos, int moveDir, bool checkLineEnd) {
 			while (startLine < pos) {
 				if (atLeadByte)
 					atLeadByte = false;
-				else if (IsDBCSLeadByteEx(dbcsCodePage, cb.CharAt(startLine)))
+				else if (Platform::IsDBCSLeadByte(dbcsCodePage, cb.CharAt(startLine)))
 					atLeadByte = true;
 				else
 					atLeadByte = false;
 				startLine++;
-				//Platform::DebugPrintf("DBCS %s\n", atlead ? "D" : "-");
 			}
 
+
 			if (atLeadByte) {
 				// Position is between a lead byte and a trail byte
 				if (moveDir > 0)
@@ -340,7 +330,6 @@ int Document::MovePositionOutsideChar(int pos, int moveDir, bool checkLineEnd) {
 			}
 		}
 	}
-#endif
 
 	return pos;
 }
@@ -355,8 +344,10 @@ void Document::ModifiedAt(int pos) {
 
 // Unlike Undo, Redo, and InsertStyledString, the pos argument is a cell number not a char number
 void Document::DeleteChars(int pos, int len) {
+	if (len == 0)
+		return;
 	if ((pos + len) > Length())
-		return ;
+		return;
 	if (cb.IsReadOnly() && enteredReadOnlyCount == 0) {
 		enteredReadOnlyCount++;
 		NotifyModifyAttempt();
@@ -375,7 +366,10 @@ void Document::DeleteChars(int pos, int len) {
 			const char *text = cb.DeleteChars(pos * 2, len * 2);
 			if (startSavePoint && cb.IsCollectingUndo())
 				NotifySavePoint(!startSavePoint);
-			ModifiedAt(pos);
+			if ((pos < Length()) || (pos == 0))
+				ModifiedAt(pos);
+			else
+				ModifiedAt(pos-1);
 			NotifyModified(
 			    DocModification(
 			        SC_MOD_DELETETEXT | SC_PERFORMED_USER,
@@ -693,72 +687,85 @@ void Document::ConvertLineEnds(int eolModeSet) {
 	EndUndoAction();
 }
 
-bool Document::IsWordChar(unsigned char ch) {
+Document::charClassification Document::WordCharClass(unsigned char ch) {
 	if ((SC_CP_UTF8 == dbcsCodePage) && (ch > 0x80))
-		return true;
-	return wordchars[ch];
+		return ccWord;
+	return charClass[ch];
 }
 
+/**
+ * Used by commmands that want to select whole words.
+ * Finds the start of word at pos when delta < 0 or the end of the word when delta >= 0.
+ */
 int Document::ExtendWordSelect(int pos, int delta) {
 	if (delta < 0) {
-		while (pos > 0 && IsWordChar(cb.CharAt(pos - 1)))
+		charClassification ccStart = WordCharClass(cb.CharAt(pos-1));
+		while (pos > 0 && (WordCharClass(cb.CharAt(pos - 1)) == ccStart))
 			pos--;
 	} else {
-		while (pos < (Length()) && IsWordChar(cb.CharAt(pos)))
-			pos++;
-	}
-	return pos;
-}
-
-int Document::NextWordStart(int pos, int delta) {
-	if (delta < 0) {
-		while (pos > 0 && (cb.CharAt(pos - 1) == ' ' || cb.CharAt(pos - 1) == '\t'))
-			pos--;
-		if (isspacechar(cb.CharAt(pos - 1))) {	// Back up to previous line
-			while (pos > 0 && isspacechar(cb.CharAt(pos - 1)))
-				pos--;
-		} else {
-			bool startAtWordChar = IsWordChar(cb.CharAt(pos - 1));
-			while (pos > 0 && !isspacechar(cb.CharAt(pos - 1)) && (startAtWordChar == IsWordChar(cb.CharAt(pos - 1))))
-				pos--;
-		}
-	} else {
-		bool startAtWordChar = IsWordChar(cb.CharAt(pos));
-		while (pos < (Length()) && isspacechar(cb.CharAt(pos)))
-			pos++;
-		while (pos < (Length()) && !isspacechar(cb.CharAt(pos)) && (startAtWordChar == IsWordChar(cb.CharAt(pos))))
-			pos++;
-		while (pos < (Length()) && (cb.CharAt(pos) == ' ' || cb.CharAt(pos) == '\t'))
+		charClassification ccStart = WordCharClass(cb.CharAt(pos));
+		while (pos < (Length()) && (WordCharClass(cb.CharAt(pos)) == ccStart))
 			pos++;
 	}
 	return pos;
 }
 
 /**
- * Check that the character before the given position
- * is not a word character.
+ * Find the start of the next word in either a forward (delta >= 0) or backwards direction 
+ * (delta < 0).
+ * This is looking for a transition between character classes although there is also some
+ * additional movement to transit white space.
+ * Used by cursor movement by word commands.
+ */
+int Document::NextWordStart(int pos, int delta) {
+	if (delta < 0) {
+		while (pos > 0 && (WordCharClass(cb.CharAt(pos - 1)) == ccSpace))
+			pos--;
+		if (pos > 0) {
+			charClassification ccStart = WordCharClass(cb.CharAt(pos-1));
+			while (pos > 0 && (WordCharClass(cb.CharAt(pos - 1)) == ccStart)) {
+				pos--;
+			}
+		}
+	} else {
+		charClassification ccStart = WordCharClass(cb.CharAt(pos));
+		while (pos < (Length()) && (WordCharClass(cb.CharAt(pos)) == ccStart))
+			pos++;
+		while (pos < (Length()) && (WordCharClass(cb.CharAt(pos)) == ccSpace))
+			pos++;
+	}
+	return pos;
+}
+
+/**
+ * Check that the character at the given position is a word or punctuation character and that
+ * the previous character is of a different character class.
  */
 bool Document::IsWordStartAt(int pos) {
 	if (pos > 0) {
-		return !IsWordChar(CharAt(pos - 1));
+		charClassification ccPos = WordCharClass(CharAt(pos));
+		return (ccPos == ccWord || ccPos == ccPunctuation) &&
+			(ccPos != WordCharClass(CharAt(pos - 1)));
 	}
 	return true;
 }
 
 /**
- * Check that the character after the given position
- * is not a word character.
+ * Check that the character at the given position is a word or punctuation character and that
+ * the next character is of a different character class.
  */
 bool Document::IsWordEndAt(int pos) {
 	if (pos < Length() - 1) {
-		return !IsWordChar(CharAt(pos));
+		charClassification ccPrev = WordCharClass(CharAt(pos-1));
+		return (ccPrev == ccWord || ccPrev == ccPunctuation) &&
+			(ccPrev != WordCharClass(CharAt(pos)));
 	}
 	return true;
 }
 
 /**
- * Check that the given range is delimited by
- * non word characters.
+ * Check that the given range is has transitions between character classes at both 
+ * ends and where the characters on the inside are word or punctuation characters.
  */
 bool Document::IsWordAt(int start, int end) {
 	return IsWordStartAt(start) && IsWordEndAt(end);
@@ -826,7 +833,7 @@ long Document::FindText(int minPos, int maxPos, const char *s,
 		startPos = MovePositionOutsideChar(startPos, 1, false);
 		endPos = MovePositionOutsideChar(endPos, 1, false);
 
-		const char *errmsg = pre->Compile(s, caseSensitive);
+		const char *errmsg = pre->Compile(s, *length, caseSensitive);
 		if (errmsg) {
 			return -1;
 		}
@@ -836,11 +843,26 @@ long Document::FindText(int minPos, int maxPos, const char *s,
 		//     Replace: $(\1-\2)
 		int lineRangeStart = LineFromPosition(startPos);
 		int lineRangeEnd = LineFromPosition(endPos);
+		if ((startPos >= LineEnd(lineRangeStart)) && (lineRangeStart < lineRangeEnd)) {
+			// the start position is at end of line or between line end characters.
+			lineRangeStart++;
+			startPos = LineStart(lineRangeStart);
+		}
 		int pos = -1;
 		int lenRet = 0;
-		char searchEnd = '\0';
-		if (*s)
-			searchEnd = s[strlen(s) - 1];
+		char searchEnd = s[*length - 1];
+		if (*length == 1) {
+			// These produce empty selections so nudge them on if needed
+			if (s[0] == '^') {
+				if (startPos == LineStart(lineRangeStart))
+					startPos++;
+			} else if (s[0] == '$') {
+				if ((startPos == LineEnd(lineRangeStart)) && (lineRangeStart < lineRangeEnd))
+					startPos = LineStart(lineRangeStart + 1);
+			}
+			lineRangeStart = LineFromPosition(startPos);
+			lineRangeEnd = LineFromPosition(endPos);
+		}
 		for (int line = lineRangeStart; line <= lineRangeEnd; line++) {
 			int startOfLine = LineStart(line);
 			int endOfLine = LineEnd(line);
@@ -855,7 +877,7 @@ long Document::FindText(int minPos, int maxPos, const char *s,
 				endOfLine = endPos;
 			}
 			DocumentIndexer di(this, endOfLine);
-			int success = pre->Execute(di, startOfLine);
+			int success = pre->Execute(di, startOfLine, endOfLine);
 			if (success) {
 				pos = pre->bopat[0];
 				lenRet = pre->eopat[0] - pre->bopat[0];
@@ -875,7 +897,9 @@ long Document::FindText(int minPos, int maxPos, const char *s,
 		int endPos = MovePositionOutsideChar(maxPos, increment, false);
 
 		// Compute actual search ranges needed
-		int lengthFind = strlen(s);
+		int lengthFind = *length;
+		if (lengthFind == -1)
+			lengthFind = strlen(s);
 		int endSearch = endPos;
 		if (startPos <= endPos) {
 			endSearch = endPos - lengthFind + 1;
@@ -929,7 +953,7 @@ long Document::FindText(int minPos, int maxPos, const char *s,
 	return -1;
 }
 
-const char *Document::SubstituteByPosition(const char *text) {
+const char *Document::SubstituteByPosition(const char *text, int *length) {
 	if (!pre)
 		return 0;
 	delete []substituted;
@@ -938,11 +962,11 @@ const char *Document::SubstituteByPosition(const char *text) {
 	if (!pre->GrabMatches(di))
 		return 0;
 	unsigned int lenResult = 0;
-	for (const char *t = text; *t; t++) {
-		if ((*t == '\\') && (*(t + 1) >= '1' && *(t + 1) <= '9')) {
-			unsigned int patNum = *(t + 1) - '0';
+	for (int i = 0; i < *length; i++) {
+		if ((text[i] == '\\') && (text[i + 1] >= '1' && text[i + 1] <= '9')) {
+			unsigned int patNum = text[i + 1] - '0';
 			lenResult += pre->eopat[patNum] - pre->bopat[patNum];
-			t++;
+			i++;
 		} else {
 			lenResult++;
 		}
@@ -951,18 +975,20 @@ const char *Document::SubstituteByPosition(const char *text) {
 	if (!substituted)
 		return 0;
 	char *o = substituted;
-	for (const char *s = text; *s; s++) {
-		if ((*s == '\\') && (*(s + 1) >= '1' && *(s + 1) <= '9')) {
-			unsigned int patNum = *(s + 1) - '0';
+	for (int j = 0; j < *length; j++) {
+		if ((text[j] == '\\') && (text[j + 1] >= '1' && text[j + 1] <= '9')) {
+			unsigned int patNum = text[j + 1] - '0';
 			unsigned int len = pre->eopat[patNum] - pre->bopat[patNum];
-			strcpy(o, pre->pat[patNum]);
+			if (pre->pat[patNum])	// Will be null if try for a match that did not occur
+				memcpy(o, pre->pat[patNum], len);
 			o += len;
-			s++;
+			j++;
 		} else {
-			*o++ = *s;
+			*o++ = text[j];
 		}
 	}
 	*o = '\0';
+	*length = lenResult;
 	return substituted;
 }
 
@@ -992,16 +1018,22 @@ void Document::ChangeCase(Range r, bool makeUpperCase) {
 void Document::SetWordChars(unsigned char *chars) {
 	int ch;
 	for (ch = 0; ch < 256; ch++) {
-		wordchars[ch] = false;
+		if (ch == '\r' || ch == '\n')
+			charClass[ch] = ccNewLine;
+		else if (ch < 0x20 || ch == ' ')
+			charClass[ch] = ccSpace;
+		else
+			charClass[ch] = ccPunctuation;
 	}
 	if (chars) {
 		while (*chars) {
-			wordchars[*chars] = true;
+			charClass[*chars] = ccWord;
 			chars++;
 		}
 	} else {
 		for (ch = 0; ch < 256; ch++) {
-			wordchars[ch] = isalnum(ch) || ch == '_';
+			if (ch > 0x80 || isalnum(ch) || ch == '_') 
+				charClass[ch] = ccWord;
 		}
 	}
 }
@@ -1016,21 +1048,20 @@ void Document::SetStylingBits(int bits) {
 }
 
 void Document::StartStyling(int position, char mask) {
-	stylingPos = position;
 	stylingMask = mask;
+	endStyled = position;
 }
 
 void Document::SetStyleFor(int length, char style) {
 	if (enteredCount == 0) {
 		enteredCount++;
 		int prevEndStyled = endStyled;
-		if (cb.SetStyleFor(stylingPos, length, style, stylingMask)) {
+		if (cb.SetStyleFor(endStyled, length, style, stylingMask)) {
 			DocModification mh(SC_MOD_CHANGESTYLE | SC_PERFORMED_USER,
 			                   prevEndStyled, length);
 			NotifyModified(mh);
 		}
-		stylingPos += length;
-		endStyled = stylingPos;
+		endStyled += length;
 		enteredCount--;
 	}
 }
@@ -1040,12 +1071,11 @@ void Document::SetStyles(int length, char *styles) {
 		enteredCount++;
 		int prevEndStyled = endStyled;
 		bool didChange = false;
-		for (int iPos = 0; iPos < length; iPos++, stylingPos++) {
-			if (cb.SetStyleAt(stylingPos, styles[iPos], stylingMask)) {
+		for (int iPos = 0; iPos < length; iPos++, endStyled++) {
+			if (cb.SetStyleAt(endStyled, styles[iPos], stylingMask)) {
 				didChange = true;
 			}
 		}
-		endStyled = stylingPos;
 		if (didChange) {
 			DocModification mh(SC_MOD_CHANGESTYLE | SC_PERFORMED_USER,
 			                   prevEndStyled, endStyled - prevEndStyled);
@@ -1125,7 +1155,7 @@ void Document::NotifyModified(DocModification mh) {
 }
 
 bool Document::IsWordPartSeparator(char ch) {
-	return ispunct(ch) && IsWordChar(ch);
+	return ispunct(ch) && (WordCharClass(ch) == ccWord);
 }
 
 int Document::WordPartLeft(int pos) {
