@@ -1,7 +1,7 @@
 // Scintilla source code edit control
 /** @file LexHTML.cxx
  ** Lexer for HTML.
- **/
+ **/ 
 // Copyright 1998-2001 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
@@ -15,6 +15,7 @@
 
 #include "PropSet.h"
 #include "Accessor.h"
+#include "StyleContext.h"
 #include "KeyWords.h"
 #include "Scintilla.h"
 #include "SciLexer.h"
@@ -23,16 +24,24 @@
 #define SCE_HA_VBS (SCE_HBA_START - SCE_HB_START)
 #define SCE_HA_PYTHON (SCE_HPA_START - SCE_HP_START)
 
-enum { eScriptNone = 0, eScriptJS, eScriptVBS, eScriptPython, eScriptPHP, eScriptXML };
+enum { eScriptNone = 0, eScriptJS, eScriptVBS, eScriptPython, eScriptPHP, eScriptXML, eScriptSGML, eScriptSGMLblock };
 enum { eHtml = 0, eNonHtmlScript, eNonHtmlPreProc, eNonHtmlScriptPreProc };
 
+inline bool IsAWordChar(const int ch) {
+	return (ch < 0x80) && (isalnum(ch) || ch == '.' || ch == '_');
+}
+
+inline bool IsAWordStart(const int ch) {
+	return (ch < 0x80) && (isalnum(ch) || ch == '_');
+}
+
 static int segIsScriptingIndicator(Accessor &styler, unsigned int start, unsigned int end, int prevValue) {
 	char s[30 + 1];
-	s[0] = '\0';
-	for (unsigned int i = 0; i < end - start + 1 && i < 30; i++) {
+	unsigned int i = 0;
+	for (; i < end - start + 1 && i < 30; i++) {
 		s[i] = static_cast<char>(tolower(styler[start + i]));
-		s[i + 1] = '\0';
 	}
+	s[i] = '\0';
 	//Platform::DebugPrintf("Scripting indicator [%s]\n", s);
 	if (strstr(s, "src"))	// External script
 		return eScriptNone;
@@ -55,11 +64,11 @@ static int segIsScriptingIndicator(Accessor &styler, unsigned int start, unsigne
 static int PrintScriptingIndicatorOffset(Accessor &styler, unsigned int start, unsigned int end) {
 	int iResult = 0;
 	char s[30 + 1];
-	s[0] = '\0';
-	for (unsigned int i = 0; i < end - start + 1 && i < 30; i++) {
+	unsigned int i = 0;
+	for (; i < end - start + 1 && i < 30; i++) {
 		s[i] = static_cast<char>(tolower(styler[start + i]));
-		s[i + 1] = '\0';
 	}
+	s[i] = '\0';
 	if (0 == strncmp(s, "php", 3)) {
 		iResult = 3;
 	}
@@ -67,24 +76,22 @@ static int PrintScriptingIndicatorOffset(Accessor &styler, unsigned int start, u
 	return iResult;
 }
 
-//static int ScriptOfState(int state,int defaultScript)
 static int ScriptOfState(int state) {
-	int scriptLanguage;
-
 	if ((state >= SCE_HP_START) && (state <= SCE_HP_IDENTIFIER)) {
-		scriptLanguage = eScriptPython;
+		return eScriptPython;
 	} else if ((state >= SCE_HB_START) && (state <= SCE_HB_STRINGEOL)) {
-		scriptLanguage = eScriptVBS;
+		return eScriptVBS;
 	} else if ((state >= SCE_HJ_START) && (state <= SCE_HJ_REGEX)) {
-		scriptLanguage = eScriptJS;
+		return eScriptJS;
 	} else if ((state >= SCE_HPHP_DEFAULT) && (state <= SCE_HPHP_COMMENTLINE)) {
-		scriptLanguage = eScriptPHP;
+		return eScriptPHP;
+	} else if ((state >= SCE_H_SGML_DEFAULT) && (state <= SCE_H_SGML_BLOCK_DEFAULT)) {
+		return eScriptSGML;
+	} else if (state == SCE_H_SGML_BLOCK_DEFAULT) {
+		return eScriptSGMLblock;
 	} else {
-		//		scriptLanguage = defaultScript;
-		scriptLanguage = eScriptNone;
+		return eScriptNone;
 	}
-
-	return scriptLanguage;
 }
 
 static int statePrintForState(int state, int inScriptType) {
@@ -175,11 +182,11 @@ static void classifyAttribHTML(unsigned int start, unsigned int end, WordList &k
 		chAttr = SCE_H_NUMBER;
 	} else {
 		char s[30 + 1];
-		s[0] = '\0';
-		for (unsigned int i = 0; i < end - start + 1 && i < 30; i++) {
+		unsigned int i = 0;
+		for (; i < end - start + 1 && i < 30; i++) {
 			s[i] = static_cast<char>(tolower(styler[start + i]));
-			s[i + 1] = '\0';
 		}
+		s[i] = '\0';
 		if (keywords.InList(s))
 			chAttr = SCE_H_ATTRIBUTE;
 	}
@@ -196,31 +203,28 @@ static int classifyTagHTML(unsigned int start, unsigned int end,
 	unsigned int i = 0;
 	for (unsigned int cPos = start; cPos <= end && i < 30; cPos++) {
 		char ch = styler[cPos];
-		if (ch != '<')
+		if ((ch != '<') && (ch != '/'))
 			s[i++] = static_cast<char>(tolower(ch));
 	}
 	s[i] = '\0';
+	bool isScript = false;
 	char chAttr = SCE_H_TAGUNKNOWN;
-	if (s[0] == '!' && s[1] == '-' && s[2] == '-') {	//Comment
-		chAttr = SCE_H_COMMENT;
-	} else if (strcmp(s, "![cdata[") == 0) {	// In lower case because already converted
-		chAttr = SCE_H_CDATA;
+	if (s[0] == '!') {
+		chAttr = SCE_H_SGML_DEFAULT;
 	} else if (s[0] == '/') {	// Closing tag
 		if (keywords.InList(s + 1))
 			chAttr = SCE_H_TAG;
 	} else {
 		if (keywords.InList(s)) {
 			chAttr = SCE_H_TAG;
-		}
-		if (0 == strcmp(s, "script")) {
-			chAttr = SCE_H_SCRIPT;
+			isScript = 0 == strcmp(s, "script");
 		}
 	}
 	if ((chAttr == SCE_H_TAGUNKNOWN) && !keywords)
 		// No keywords -> all are known
 		chAttr = SCE_H_TAG;
 	styler.ColourTo(end, chAttr);
-	return chAttr;
+	return isScript ? SCE_H_SCRIPT : chAttr;
 }
 
 static void classifyWordHTJS(unsigned int start, unsigned int end,
@@ -231,10 +235,11 @@ static void classifyWordHTJS(unsigned int start, unsigned int end,
 		chAttr = SCE_HJ_NUMBER;
 	else {
 		char s[30 + 1];
-		for (unsigned int i = 0; i < end - start + 1 && i < 30; i++) {
+		unsigned int i = 0;
+		for (; i < end - start + 1 && i < 30; i++) {
 			s[i] = styler[start + i];
-			s[i + 1] = '\0';
 		}
+		s[i] = '\0';
 		if (keywords.InList(s))
 			chAttr = SCE_HJ_KEYWORD;
 	}
@@ -248,10 +253,11 @@ static int classifyWordHTVB(unsigned int start, unsigned int end, WordList &keyw
 		chAttr = SCE_HB_NUMBER;
 	else {
 		char s[30 + 1];
-		for (unsigned int i = 0; i < end - start + 1 && i < 30; i++) {
+		unsigned int i = 0;
+		for (; i < end - start + 1 && i < 30; i++) {
 			s[i] = static_cast<char>(tolower(styler[start + i]));
-			s[i + 1] = '\0';
 		}
+		s[i] = '\0';
 		if (keywords.InList(s)) {
 			chAttr = SCE_HB_WORD;
 			if (strcmp(s, "rem") == 0)
@@ -268,10 +274,11 @@ static int classifyWordHTVB(unsigned int start, unsigned int end, WordList &keyw
 static void classifyWordHTPy(unsigned int start, unsigned int end, WordList &keywords, Accessor &styler, char *prevWord, int inScriptType) {
 	bool wordIsNumber = isdigit(styler[start]);
 	char s[30 + 1];
-	for (unsigned int i = 0; i < end - start + 1 && i < 30; i++) {
+	unsigned int i = 0;
+	for (; i < end - start + 1 && i < 30; i++) {
 		s[i] = styler[start + i];
-		s[i + 1] = '\0';
 	}
+	s[i] = '\0';
 	char chAttr = SCE_HP_IDENTIFIER;
 	if (0 == strcmp(prevWord, "class"))
 		chAttr = SCE_HP_CLASSNAME;
@@ -294,16 +301,37 @@ static void classifyWordHTPHP(unsigned int start, unsigned int end, WordList &ke
 		chAttr = SCE_HPHP_NUMBER;
 	else {
 		char s[30 + 1];
-		for (unsigned int i = 0; i < end - start + 1 && i < 30; i++) {
+		unsigned int i = 0;
+		for (; i < end - start + 1 && i < 30; i++) {
 			s[i] = styler[start + i];
-			s[i + 1] = '\0';
 		}
+		s[i] = '\0';
 		if (keywords.InList(s))
 			chAttr = SCE_HPHP_WORD;
 	}
 	styler.ColourTo(end, chAttr);
 }
 
+static bool isWordHSGML(unsigned int start, unsigned int end, WordList &keywords, Accessor &styler) {
+	char s[30 + 1];
+	unsigned int i = 0;
+	for (; i < end - start + 1 && i < 30; i++) {
+		s[i] = styler[start + i];
+	}
+	s[i] = '\0';
+	return keywords.InList(s);
+}
+
+static bool isWordCdata(unsigned int start, unsigned int end, Accessor &styler) {
+	char s[30 + 1];
+	unsigned int i = 0;
+	for (; i < end - start + 1 && i < 30; i++) {
+		s[i] = styler[start + i];
+	}
+	s[i] = '\0';
+	return (0 == strcmp(s, "[CDATA["));
+}
+
 // Return the first state to reach when entering a scripting language
 static int StateForScript(int scriptLanguage) {
 	int Result;
@@ -320,6 +348,9 @@ static int StateForScript(int scriptLanguage) {
 	case eScriptXML:
 		Result = SCE_H_TAGUNKNOWN;
 		break;
+	case eScriptSGML:
+		Result = SCE_H_SGML_DEFAULT;
+		break;
 	default :
 		Result = SCE_HJ_START;
 		break;
@@ -331,6 +362,10 @@ inline bool ishtmlwordchar(char ch) {
 	return isalnum(ch) || ch == '.' || ch == '-' || ch == '_' || ch == ':' || ch == '!' || ch == '#';
 }
 
+inline bool issgmlwordchar(char ch) {
+	return isalnum(ch) || ch == '.' || ch == '_' || ch == ':' || ch == '!' || ch == '#' || ch == '[';
+}
+
 static bool InTagState(int state) {
 	return state == SCE_H_TAG || state == SCE_H_TAGUNKNOWN ||
 	       state == SCE_H_SCRIPT ||
@@ -339,6 +374,10 @@ static bool InTagState(int state) {
 	       state == SCE_H_DOUBLESTRING || state == SCE_H_SINGLESTRING;
 }
 
+static bool IsCommentState(const int state) {
+	return state == SCE_H_COMMENT || state == SCE_H_SGML_COMMENT;
+}
+
 static bool isLineEnd(char ch) {
 	return ch == '\r' || ch == '\n';
 }
@@ -347,17 +386,24 @@ static bool isOKBeforeRE(char ch) {
 	return (ch == '(') || (ch == '=') || (ch == ',');
 }
 
+static bool isPHPStringState(int state) {
+	return
+	    (state == SCE_HPHP_HSTRING) ||
+	    (state == SCE_HPHP_SIMPLESTRING) ||
+	    (state == SCE_HPHP_HSTRING_VARIABLE);
+}
+
 static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initStyle, WordList *keywordlists[],
                                   Accessor &styler) {
-
 	WordList &keywords = *keywordlists[0];
 	WordList &keywords2 = *keywordlists[1];
 	WordList &keywords3 = *keywordlists[2];
 	WordList &keywords4 = *keywordlists[3];
 	WordList &keywords5 = *keywordlists[4];
+	WordList &keywords6 = *keywordlists[5]; // SGML (DTD) keywords
 
 	// Lexer for HTML requires more lexical states (7 bits worth) than most lexers
-	styler.StartAt(startPos, 127);
+	styler.StartAt(startPos, STYLE_MAX);
 	char prevWord[200];
 	prevWord[0] = '\0';
 	int StateToPrint = initStyle;
@@ -365,52 +411,53 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 
 	// If inside a tag, it may be a script tag, so reread from the start to ensure any language tags are seen
 	if (InTagState(state)) {
-		while ((startPos > 1) && (InTagState(styler.StyleAt(startPos - 1)))) {
+		while ((startPos > 0) && (InTagState(styler.StyleAt(startPos - 1)))) {
 			startPos--;
-            length++;
+			length++;
 		}
 		state = SCE_H_DEFAULT;
 	}
-	styler.StartAt(startPos, 127);
+	styler.StartAt(startPos, STYLE_MAX);
 
-	int lineState = eScriptVBS;
 	int lineCurrent = styler.GetLine(startPos);
-	if (lineCurrent > 0)
+	int lineState;
+	if (lineCurrent > 0) {
 		lineState = styler.GetLineState(lineCurrent);
-	int inScriptType  = (lineState >> 0) & 0x03; // 2 bits of scripting type
-	bool tagOpened    = (lineState >> 2) & 0x01; // 1 bit to know if we are in an opened tag
-	bool tagClosing   = (lineState >> 3) & 0x01; // 1 bit to know if we are in a closing tag
-	int defaultScript = (lineState >> 4) & 0x0F; // 4 bits of script name
-	int beforePreProc = (lineState >> 8) & 0xFF; // 8 bits of state
+	} else {
+		// Default client and ASP scripting language is JavaScript
+		lineState = eScriptJS << 8;
+		lineState |= styler.GetPropertyInt("asp.default.language", eScriptJS) << 4;
+	}
+	int inScriptType = (lineState >> 0) & 0x03; // 2 bits of scripting type
+	bool tagOpened = (lineState >> 2) & 0x01; // 1 bit to know if we are in an opened tag
+	bool tagClosing = (lineState >> 3) & 0x01; // 1 bit to know if we are in a closing tag
+	int aspScript = (lineState >> 4) & 0x0F; // 4 bits of script name
+	int clientScript = (lineState >> 8) & 0x0F; // 4 bits of script name
+	int beforePreProc = (lineState >> 12) & 0xFF; // 8 bits of state
 
-	//	int scriptLanguage = ScriptOfState(state,defaultScript);
 	int scriptLanguage = ScriptOfState(state);
 
-	bool fold = styler.GetPropertyInt("fold");
-	bool foldHTML = styler.GetPropertyInt("fold.html",0);
-	bool foldCompact = styler.GetPropertyInt("fold.compact",1);
-
-	fold = foldHTML && fold;
+	const bool foldHTML = styler.GetPropertyInt("fold.html", 0);
+	const bool fold = foldHTML && styler.GetPropertyInt("fold");
+	const bool foldCompact = styler.GetPropertyInt("fold.compact", 1);
 
 	int levelPrev = styler.LevelAt(lineCurrent) & SC_FOLDLEVELNUMBERMASK;
 	int levelCurrent = levelPrev;
-	int visibleChars;
-
-	visibleChars = 0;
+	int visibleChars = 0;
 
 	char chPrev = ' ';
 	char ch = ' ';
 	char chPrevNonWhite = ' ';
 	styler.StartSegment(startPos);
-	int lengthDoc = startPos + length;
+	const int lengthDoc = startPos + length;
 	for (int i = startPos; i < lengthDoc; i++) {
-		char chPrev2 = chPrev;
+		const char chPrev2 = chPrev;
 		chPrev = ch;
 		if (ch != ' ' && ch != '\t')
 			chPrevNonWhite = ch;
 		ch = styler[i];
 		char chNext = styler.SafeGetCharAt(i + 1);
-		char chNext2 = styler.SafeGetCharAt(i + 2);
+		const char chNext2 = styler.SafeGetCharAt(i + 2);
 
 		// Handle DBCS codepages
 		if (styler.IsLeadByte(ch)) {
@@ -445,18 +492,21 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 					} else if ((ch == '\n') && !((chNext == '\r') && (chNext2 == '\n')) && (chNext != '\n')) {
 						// check if the number of tabs is lower than the level
 						int Findlevel = (levelCurrent & ~SC_FOLDLEVELBASE) * 8;
-						for (int j = 0;Findlevel > 0;j++) {
+						for (int j = 0; Findlevel > 0; j++) {
 							char chTmp = styler.SafeGetCharAt(i + j + 1);
 							if (chTmp == '\t') {
 								Findlevel -= 8;
-							}	else if (chTmp == ' ') {
+							} else if (chTmp == ' ') {
 								Findlevel--;
-							}	else break;
+							} else {
+								break;
+							}
 						}
 
 						if (Findlevel > 0) {
 							levelCurrent -= Findlevel / 8;
-							if (Findlevel % 8) levelCurrent--;
+							if (Findlevel % 8)
+								levelCurrent--;
 						}
 					}
 				}
@@ -481,11 +531,12 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			}
 			lineCurrent++;
 			styler.SetLineState(lineCurrent,
-			                    ((inScriptType  & 0x03) << 0) |
-								((tagOpened     & 0x01) << 2) |
-								((tagClosing    & 0x01) << 3) |
-			                    ((defaultScript & 0x0F) << 4) |
-			                    ((beforePreProc & 0xFF) << 8));
+			                    ((inScriptType & 0x03) << 0) |
+			                    ((tagOpened & 0x01) << 2) |
+			                    ((tagClosing & 0x01) << 3) |
+			                    ((aspScript & 0x0F) << 4) |
+			                    ((clientScript & 0x0F) << 8) |
+			                    ((beforePreProc & 0xFF) << 12));
 		}
 
 		// generic end of script processing
@@ -497,9 +548,9 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			case SCE_H_SINGLESTRING:
 			case SCE_HJ_COMMENT:
 			case SCE_HJ_COMMENTDOC:
-			// SCE_HJ_COMMENTLINE removed as this is a common thing done to hide
-			// the end of script marker from some JS interpreters.
-			//case SCE_HJ_COMMENTLINE:
+				// SCE_HJ_COMMENTLINE removed as this is a common thing done to hide
+				// the end of script marker from some JS interpreters.
+				//case SCE_HJ_COMMENTLINE:
 			case SCE_HJ_DOUBLESTRING:
 			case SCE_HJ_SINGLESTRING:
 			case SCE_HB_STRING:
@@ -508,25 +559,31 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			case SCE_HP_TRIPLEDOUBLE:
 				break;
 			default :
-				// maybe we should check here if it's a tag and if it's SCRIPT
+				// closing tag of the script (it's a closing HTML tag anyway)
 				styler.ColourTo(i - 1, StateToPrint);
 				state = SCE_H_TAGUNKNOWN;
 				inScriptType = eHtml;
 				scriptLanguage = eScriptNone;
+				clientScript = eScriptJS;
 				i += 2;
-				// unfold closing script
-				levelCurrent--;
+				visibleChars += 2;
+				tagClosing = true;
 				continue;
 			}
 		}
 
 		/////////////////////////////////////
 		// handle the start of PHP pre-processor = Non-HTML
-		else if ((state != SCE_H_ASPAT) && (ch == '<') && (chNext == '?')) {
+		else if ((state != SCE_H_ASPAT) &&
+		         !isPHPStringState(state) &&
+		         (state != SCE_HPHP_COMMENT) &&
+		         (ch == '<') &&
+		         (chNext == '?')) {
 			styler.ColourTo(i - 1, StateToPrint);
 			beforePreProc = state;
 			scriptLanguage = segIsScriptingIndicator(styler, styler.GetStartSegment() + 2, i + 10, eScriptPHP);
 			i++;
+			visibleChars++;
 			i += PrintScriptingIndicatorOffset(styler, styler.GetStartSegment() + 2, i + 10);
 			if (scriptLanguage == eScriptXML)
 				styler.ColourTo(i, SCE_H_XMLSTART);
@@ -539,6 +596,8 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 				inScriptType = eNonHtmlPreProc;
 			// fold whole script
 			levelCurrent++;
+			if (scriptLanguage == eScriptXML)
+				levelCurrent--; // no folding of the XML first tag (all XML-like tags in this case)
 			// should be better
 			ch = styler.SafeGetCharAt(i);
 			continue;
@@ -555,16 +614,25 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 
 			if (chNext2 == '@') {
 				i += 2; // place as if it was the second next char treated
+				visibleChars += 2;
 				state = SCE_H_ASPAT;
+			} else if ((chNext2 == '-') && (styler.SafeGetCharAt(i + 3) == '-')) {
+				styler.ColourTo(i + 3, SCE_H_ASP);
+				state = SCE_H_XCCOMMENT;
+				scriptLanguage = eScriptVBS;
+				continue;
 			} else {
 				if (chNext2 == '=') {
 					i += 2; // place as if it was the second next char treated
+					visibleChars += 2;
 				} else {
 					i++; // place as if it was the next char treated
+					visibleChars++;
 				}
 
-				state = StateForScript(defaultScript);
+				state = StateForScript(aspScript);
 			}
+			scriptLanguage = eScriptVBS;
 			styler.ColourTo(i, SCE_H_ASP);
 			// fold whole script
 			levelCurrent++;
@@ -573,14 +641,42 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			continue;
 		}
 
+		/////////////////////////////////////
+		// handle the start of SGML language (DTD)
+		else if (((scriptLanguage == eScriptNone) || (scriptLanguage == eScriptXML)) &&
+		         (chPrev == '<') &&
+		         (ch == '!') &&
+		         (StateToPrint != SCE_H_CDATA) && (!IsCommentState(StateToPrint))) {
+			beforePreProc = state;
+			styler.ColourTo(i - 2, StateToPrint);
+			if ((chNext == '-') && (chNext2 == '-')) {
+				state = SCE_H_COMMENT; // wait for a pending command
+			}
+			else if (isWordCdata(i + 1, i + 7, styler)) {
+				state = SCE_H_CDATA;
+			} else {
+				styler.ColourTo(i, SCE_H_SGML_DEFAULT); // <! is default
+				scriptLanguage = eScriptSGML;
+				state = SCE_H_SGML_COMMAND; // wait for a pending command
+			}
+			// fold whole tag (-- when closing the tag)
+
+			levelCurrent++;
+			continue;
+		}
+
 		// handle the end of a pre-processor = Non-HTML
-		else if (((inScriptType == eNonHtmlPreProc)
-			   || (inScriptType == eNonHtmlScriptPreProc))
-			  && (((scriptLanguage == eScriptPHP) && (ch == '?'))
-			   || (!isStringState(state) && !isCommentASPState(state) && (ch == '%')))
-			  && (chNext == '>')) {
+		else if ((
+		             ((inScriptType == eNonHtmlPreProc)
+		              || (inScriptType == eNonHtmlScriptPreProc)) && (
+		                 ((scriptLanguage == eScriptPHP) && (ch == '?') && !isPHPStringState(state) && (state != SCE_HPHP_COMMENT)) ||
+		                 ((scriptLanguage != eScriptNone) && !isStringState(state) &&
+		                  (ch == '%'))
+		             ) && (chNext == '>')) ||
+		         ((scriptLanguage == eScriptSGML) && (ch == '>') && (state != SCE_H_SGML_COMMENT))) {
 			if (state == SCE_H_ASPAT) {
-				defaultScript = segIsScriptingIndicator(styler, styler.GetStartSegment(), i - 1, defaultScript);
+				aspScript = segIsScriptingIndicator(styler,
+				                                    styler.GetStartSegment(), i - 1, aspScript);
 			}
 			// Bounce out of any ASP mode
 			switch (state) {
@@ -596,15 +692,23 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			case SCE_HPHP_WORD:
 				classifyWordHTPHP(styler.GetStartSegment(), i - 1, keywords5, styler);
 				break;
+			case SCE_H_XCCOMMENT:
+				styler.ColourTo(i - 1, state);
+				break;
 			default :
 				styler.ColourTo(i - 1, StateToPrint);
 				break;
 			}
-			i++;
+			if (scriptLanguage != eScriptSGML) {
+				i++;
+				visibleChars++;
+			}
 			if (ch == '%')
 				styler.ColourTo(i, SCE_H_ASP);
 			else if (scriptLanguage == eScriptXML)
 				styler.ColourTo(i, SCE_H_XMLEND);
+			else if (scriptLanguage == eScriptSGML)
+				styler.ColourTo(i, SCE_H_SGML_DEFAULT);
 			else
 				styler.ColourTo(i, SCE_H_QUESTION);
 			state = beforePreProc;
@@ -624,39 +728,156 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			if (ch == '<') {
 				// in HTML, fold on tag open and unfold on tag close
 				tagOpened = true;
-				if (chNext == '/') {
-					tagClosing = true;
-				} else {
-					tagClosing = false;
-				}
-
+				tagClosing = (chNext == '/');
 				styler.ColourTo(i - 1, StateToPrint);
-				if (chNext == '!' && chNext2 == '-' && styler.SafeGetCharAt(i + 3) == '-') {
-					// should be better
-					i += 3;
-					levelCurrent++;
-					state = SCE_H_COMMENT;
-				} else
+				if (chNext != '!')
 					state = SCE_H_TAGUNKNOWN;
 			} else if (ch == '&') {
 				styler.ColourTo(i - 1, SCE_H_DEFAULT);
 				state = SCE_H_ENTITY;
 			}
 			break;
-		case SCE_H_COMMENT:
-			if ((ch == '>') && (chPrev == '-') && (chPrev2 == '-')) {
-				// unfold HTML comment
-				levelCurrent--;
+		case SCE_H_SGML_DEFAULT:
+			if (scriptLanguage == eScriptSGMLblock)
+				StateToPrint = SCE_H_SGML_BLOCK_DEFAULT;
+
+			if (ch == '\"') {
+				styler.ColourTo(i - 1, StateToPrint);
+				state = SCE_H_SGML_DOUBLESTRING;
+			} else if (ch == '\'') {
+				styler.ColourTo(i - 1, StateToPrint);
+				state = SCE_H_SGML_SIMPLESTRING;
+			} else if ((ch == '-') && (chPrev == '-')) {
+				styler.ColourTo(i - 2, StateToPrint);
+				state = SCE_H_SGML_COMMENT;
+			} else if (isalpha(ch) && (chPrev == '%')) {
+				styler.ColourTo(i - 2, StateToPrint);
+				state = SCE_H_SGML_ENTITY;
+			} else if (ch == '#') {
+				styler.ColourTo(i - 1, StateToPrint);
+				state = SCE_H_SGML_SPECIAL;
+			} else if (ch == '[') {
+				styler.ColourTo(i - 1, StateToPrint);
+				scriptLanguage = eScriptSGMLblock;
+			} else if (ch == ']') {
+				if (scriptLanguage == eScriptSGMLblock) {
+					styler.ColourTo(i, StateToPrint);
+					scriptLanguage = eScriptSGML;
+				} else {
+					styler.ColourTo(i - 1, StateToPrint);
+					styler.ColourTo(i, SCE_H_SGML_ERROR);
+				}
+			} else if (scriptLanguage == eScriptSGMLblock) {
+				if ((ch == '!') && (chPrev == '<')) {
+					styler.ColourTo(i - 2, StateToPrint);
+					styler.ColourTo(i, SCE_H_SGML_DEFAULT);
+					state = SCE_H_SGML_COMMAND;
+				} else if (ch == '>') {
+					styler.ColourTo(i - 1, StateToPrint);
+					styler.ColourTo(i, SCE_H_SGML_DEFAULT);
+				}
+			}
+			break;
+		case SCE_H_SGML_COMMAND:
+			if ((ch == '-') && (chPrev == '-')) {
+				styler.ColourTo(i - 2, StateToPrint);
+				state = SCE_H_SGML_COMMENT;
+			} else if (!issgmlwordchar(ch)) {
+				if (isWordHSGML(styler.GetStartSegment(), i - 1, keywords6, styler)) {
+					styler.ColourTo(i - 1, StateToPrint);
+					state = SCE_H_SGML_1ST_PARAM;
+				} else {
+					state = SCE_H_SGML_ERROR;
+				}
+			}
+			break;
+		case SCE_H_SGML_1ST_PARAM:
+			// wait for the beginning of the word
+			if ((ch == '-') && (chPrev == '-')) {
+				if (scriptLanguage == eScriptSGMLblock) {
+					styler.ColourTo(i - 2, SCE_H_SGML_BLOCK_DEFAULT);
+				} else {
+					styler.ColourTo(i - 2, SCE_H_SGML_DEFAULT);
+				}
+				state = SCE_H_SGML_1ST_PARAM_COMMENT;
+			} else if (issgmlwordchar(ch)) {
+				if (scriptLanguage == eScriptSGMLblock) {
+					styler.ColourTo(i - 1, SCE_H_SGML_BLOCK_DEFAULT);
+				} else {
+					styler.ColourTo(i - 1, SCE_H_SGML_DEFAULT);
+				}
+				// find the length of the word
+				int size = 1;
+				while (ishtmlwordchar(styler.SafeGetCharAt(i + size)))
+					size++;
+				styler.ColourTo(i + size - 1, StateToPrint);
+				i += size - 1;
+				visibleChars += size - 1;
+				ch = styler.SafeGetCharAt(i);
+				state = SCE_H_SGML_DEFAULT;
+				continue;
+			}
+			break;
+		case SCE_H_SGML_ERROR:
+			if ((ch == '-') && (chPrev == '-')) {
+				styler.ColourTo(i - 2, StateToPrint);
+				state = SCE_H_SGML_COMMENT;
+			}
+		case SCE_H_SGML_DOUBLESTRING:
+			if (ch == '\"') {
 				styler.ColourTo(i, StateToPrint);
-				state = SCE_H_DEFAULT;
-				tagOpened = false;
+				state = SCE_H_SGML_DEFAULT;
+			}
+			break;
+		case SCE_H_SGML_SIMPLESTRING:
+			if (ch == '\'') {
+				styler.ColourTo(i, StateToPrint);
+				state = SCE_H_SGML_DEFAULT;
+			}
+			break;
+		case SCE_H_SGML_COMMENT:
+			if ((ch == '-') && (chPrev == '-')) {
+				styler.ColourTo(i, StateToPrint);
+				state = SCE_H_SGML_DEFAULT;
 			}
 			break;
 		case SCE_H_CDATA:
-			if ((ch == '>') && (chPrev == ']') && (chPrev2 == ']')) {
+			if ((chPrev2 == ']') && (chPrev == ']') && (ch == '>')) {
 				styler.ColourTo(i, StateToPrint);
 				state = SCE_H_DEFAULT;
-				tagOpened = false;
+				levelCurrent--;
+			}
+			break;
+		case SCE_H_COMMENT:
+			if ((chPrev2 == '-') && (chPrev == '-') && (ch == '>')) {
+				styler.ColourTo(i, StateToPrint);
+				state = SCE_H_DEFAULT;
+				levelCurrent--;
+			}
+			break;
+		case SCE_H_SGML_1ST_PARAM_COMMENT:
+			if ((ch == '-') && (chPrev == '-')) {
+				styler.ColourTo(i, SCE_H_SGML_COMMENT);
+				state = SCE_H_SGML_1ST_PARAM;
+			}
+			break;
+		case SCE_H_SGML_SPECIAL:
+			if (!isupper(ch)) {
+				styler.ColourTo(i - 1, StateToPrint);
+				if (isalnum(ch)) {
+					state = SCE_H_SGML_ERROR;
+				} else {
+					state = SCE_H_SGML_DEFAULT;
+				}
+			}
+			break;
+		case SCE_H_SGML_ENTITY:
+			if (ch == ';') {
+				styler.ColourTo(i, StateToPrint);
+				state = SCE_H_SGML_DEFAULT;
+			} else if (!isalnum(ch) && ch != '-' && ch != '.') {
+				styler.ColourTo(i, SCE_H_SGML_ERROR);
+				state = SCE_H_SGML_DEFAULT;
 			}
 			break;
 		case SCE_H_ENTITY:
@@ -673,9 +894,14 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			if (!ishtmlwordchar(ch) && !((ch == '/') && (chPrev == '<')) && ch != '[') {
 				int eClass = classifyTagHTML(styler.GetStartSegment(), i - 1, keywords, styler);
 				if (eClass == SCE_H_SCRIPT) {
-					inScriptType = eNonHtmlScript;
-					scriptLanguage = defaultScript;
-					eClass = SCE_H_TAG;
+					if (!tagClosing) {
+						inScriptType = eNonHtmlScript;
+						scriptLanguage = clientScript;
+						eClass = SCE_H_TAG;
+					} else {
+						scriptLanguage = eScriptNone;
+						eClass = SCE_H_TAG;
+					}
 				}
 				if (ch == '>') {
 					styler.ColourTo(i, eClass);
@@ -685,26 +911,27 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 						state = SCE_H_DEFAULT;
 					}
 					tagOpened = false;
-					if (tagClosing)
+					if (tagClosing) {
 						levelCurrent--;
-					else
-						levelCurrent++;
-					tagClosing = false;
-			    } else if (ch == '/' && chNext == '>') {
-					if (eClass == SCE_H_TAGUNKNOWN) {
-					    styler.ColourTo(i + 1, SCE_H_TAGUNKNOWN);
 					} else {
-					    styler.ColourTo(i - 1, StateToPrint);
-					    styler.ColourTo(i + 1, SCE_H_TAGEND);
+						levelCurrent++;
 					}
-				    i++;
-				    ch = chNext;
-				    state = SCE_H_DEFAULT;
+					tagClosing = false;
+				} else if (ch == '/' && chNext == '>') {
+					if (eClass == SCE_H_TAGUNKNOWN) {
+						styler.ColourTo(i + 1, SCE_H_TAGUNKNOWN);
+					} else {
+						styler.ColourTo(i - 1, StateToPrint);
+						styler.ColourTo(i + 1, SCE_H_TAGEND);
+					}
+					i++;
+					ch = chNext;
+					state = SCE_H_DEFAULT;
 					tagOpened = false;
 				} else {
 					if (eClass != SCE_H_TAGUNKNOWN) {
-						if (eClass == SCE_H_CDATA) {
-							state = SCE_H_CDATA;
+						if (eClass == SCE_H_SGML_DEFAULT) {
+							state = SCE_H_SGML_DEFAULT;
 						} else {
 							state = SCE_H_OTHER;
 						}
@@ -715,10 +942,11 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 		case SCE_H_ATTRIBUTE:
 			if (!ishtmlwordchar(ch) && ch != '/' && ch != '-') {
 				if (inScriptType == eNonHtmlScript) {
-                    int scriptLanguagePrev = scriptLanguage;
-					scriptLanguage = segIsScriptingIndicator(styler, styler.GetStartSegment(), i - 1, scriptLanguage);
-                    if ((scriptLanguagePrev != scriptLanguage) && (scriptLanguage == eScriptNone))
-                        inScriptType = eHtml;
+					int scriptLanguagePrev = scriptLanguage;
+					clientScript = segIsScriptingIndicator(styler, styler.GetStartSegment(), i - 1, scriptLanguage);
+					scriptLanguage = clientScript;
+					if ((scriptLanguagePrev != scriptLanguage) && (scriptLanguage == eScriptNone))
+						inScriptType = eHtml;
 				}
 				classifyAttribHTML(styler.GetStartSegment(), i - 1, keywords, styler);
 				if (ch == '>') {
@@ -924,8 +1152,6 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			} else if (ch == '\"') {
 				styler.ColourTo(i, statePrintForState(SCE_HJ_DOUBLESTRING, inScriptType));
 				state = SCE_HJ_DEFAULT;
-				i++;
-				ch = chNext;
 			} else if ((inScriptType == eNonHtmlScript) && (ch == '-') && (chNext == '-') && (chNext2 == '>')) {
 				styler.ColourTo(i - 1, StateToPrint);
 				state = SCE_HJ_COMMENTLINE;
@@ -943,8 +1169,6 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			} else if (ch == '\'') {
 				styler.ColourTo(i, statePrintForState(SCE_HJ_SINGLESTRING, inScriptType));
 				state = SCE_HJ_DEFAULT;
-				i++;
-				ch = chNext;
 			} else if ((inScriptType == eNonHtmlScript) && (ch == '-') && (chNext == '-') && (chNext2 == '>')) {
 				styler.ColourTo(i - 1, StateToPrint);
 				state = SCE_HJ_COMMENTLINE;
@@ -993,7 +1217,7 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 				state = SCE_HB_COMMENTLINE;
 			} else if (isoperator(ch)) {
 				styler.ColourTo(i - 1, StateToPrint);
-				styler.ColourTo(i, SCE_HB_DEFAULT);
+				styler.ColourTo(i, statePrintForState(SCE_HB_DEFAULT, inScriptType));
 				state = SCE_HB_DEFAULT;
 			} else if ((ch == ' ') || (ch == '\t')) {
 				if (state == SCE_HB_START) {
@@ -1021,8 +1245,6 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			if (ch == '\"') {
 				styler.ColourTo(i, StateToPrint);
 				state = SCE_HB_DEFAULT;
-				i++;
-				ch = chNext;
 			} else if (ch == '\r' || ch == '\n') {
 				styler.ColourTo(i - 1, StateToPrint);
 				state = SCE_HB_STRINGEOL;
@@ -1179,6 +1401,8 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 					state = SCE_HPHP_SIMPLESTRING;
 				} else if (ch == '$') {
 					state = SCE_HPHP_VARIABLE;
+				} else if (isoperator(ch)) {
+					state = SCE_HPHP_OPERATOR;
 				} else {
 					state = SCE_HPHP_DEFAULT;
 				}
@@ -1187,13 +1411,19 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 		case SCE_HPHP_NUMBER:
 			if (!isdigit(ch)) {
 				styler.ColourTo(i - 1, SCE_HPHP_NUMBER);
-				state = SCE_HPHP_DEFAULT;
+				if (isoperator(ch))
+					state = SCE_HPHP_OPERATOR;
+				else
+					state = SCE_HPHP_DEFAULT;
 			}
 			break;
 		case SCE_HPHP_VARIABLE:
 			if (!iswordstart(ch)) {
 				styler.ColourTo(i - 1, SCE_HPHP_VARIABLE);
-				state = SCE_HPHP_DEFAULT;
+				if (isoperator(ch))
+					state = SCE_HPHP_OPERATOR;
+				else
+					state = SCE_HPHP_DEFAULT;
 			}
 			break;
 		case SCE_HPHP_COMMENT:
@@ -1213,7 +1443,7 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 				// skip the next char
 				i++;
 			} else if (ch == '$') {
-				styler.ColourTo(i-1, StateToPrint);
+				styler.ColourTo(i - 1, StateToPrint);
 				state = SCE_HPHP_HSTRING_VARIABLE;
 			} else if (ch == '\"') {
 				styler.ColourTo(i, StateToPrint);
@@ -1231,11 +1461,12 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 			break;
 		case SCE_HPHP_HSTRING_VARIABLE:
 			if (!iswordstart(ch)) {
-				styler.ColourTo(i-1, StateToPrint);
+				styler.ColourTo(i - 1, StateToPrint);
 				i--; // strange but it works
 				state = SCE_HPHP_HSTRING;
 			}
 			break;
+		case SCE_HPHP_OPERATOR:
 		case SCE_HPHP_DEFAULT:
 			styler.ColourTo(i - 1, StateToPrint);
 			if (isdigit(ch)) {
@@ -1256,13 +1487,21 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 				state = SCE_HPHP_SIMPLESTRING;
 			} else if (ch == '$') {
 				state = SCE_HPHP_VARIABLE;
+			} else if (isoperator(ch)) {
+				state = SCE_HPHP_OPERATOR;
+			} else if ((state == SCE_HPHP_OPERATOR) && (isspacechar(ch))) {
+				state = SCE_HPHP_DEFAULT;
 			}
 			break;
 			///////////// end - PHP state handling
 		}
 
+		// Some of the above terminated their lexeme but since the same character starts
+		// the same class again, only reenter if non empty segment.
+
+		bool nonEmptySegment = i >= static_cast<int>(styler.GetStartSegment());
 		if (state == SCE_HB_DEFAULT) {    // One of the above succeeded
-			if (ch == '\"') {
+			if ((ch == '\"') && (nonEmptySegment)) {
 				state = SCE_HB_STRING;
 			} else if (ch == '\'') {
 				state = SCE_HB_COMMENTLINE;
@@ -1272,7 +1511,7 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 				styler.ColourTo(i, SCE_HB_DEFAULT);
 			}
 		} else if (state == SCE_HBA_DEFAULT) {    // One of the above succeeded
-			if (ch == '\"') {
+			if ((ch == '\"') && (nonEmptySegment)) {
 				state = SCE_HBA_STRING;
 			} else if (ch == '\'') {
 				state = SCE_HBA_COMMENTLINE;
@@ -1289,15 +1528,14 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 					state = SCE_HJ_COMMENT;
 			} else if (ch == '/' && chNext == '/') {
 				state = SCE_HJ_COMMENTLINE;
-			} else if (ch == '\"') {
+			} else if ((ch == '\"') && (nonEmptySegment)) {
 				state = SCE_HJ_DOUBLESTRING;
-			} else if (ch == '\'') {
+			} else if ((ch == '\'') && (nonEmptySegment)) {
 				state = SCE_HJ_SINGLESTRING;
 			} else if (iswordstart(ch)) {
 				state = SCE_HJ_WORD;
 			} else if (isoperator(ch)) {
 				styler.ColourTo(i, statePrintForState(SCE_HJ_SYMBOLS, inScriptType));
-				//styler.ColourTo(i, SCE_HJ_SYMBOLS);
 			}
 		}
 	}
@@ -1312,6 +1550,193 @@ static void ColouriseHyperTextDoc(unsigned int startPos, int length, int initSty
 	}
 }
 
+static bool isASPScript(int state) {
+	return 
+		(state >= SCE_HJA_START && state <= SCE_HJA_REGEX) ||
+		(state >= SCE_HBA_START && state <= SCE_HBA_STRINGEOL) ||
+		(state >= SCE_HPA_DEFAULT && state <= SCE_HPA_IDENTIFIER);
+}
+
+static void ColouriseHBAPiece(StyleContext &sc, WordList *keywordlists[]) {
+	WordList &keywordsVBS = *keywordlists[2];
+	if (sc.state == SCE_HBA_WORD) {
+		if (!IsAWordChar(sc.ch)) {
+			char s[100];
+			sc.GetCurrentLowered(s, sizeof(s));
+			if (keywordsVBS.InList(s)) {
+				if (strcmp(s, "rem") == 0) {
+					sc.ChangeState(SCE_HBA_COMMENTLINE);
+					if (sc.atLineEnd) {
+						sc.SetState(SCE_HBA_DEFAULT);
+					}
+				} else {
+					sc.SetState(SCE_HBA_DEFAULT);
+				}
+			} else {
+				sc.ChangeState(SCE_HBA_IDENTIFIER);
+				sc.SetState(SCE_HBA_DEFAULT);
+			}
+		}
+	} else if (sc.state == SCE_HBA_NUMBER) {
+		if (!IsAWordChar(sc.ch)) {
+			sc.SetState(SCE_HBA_DEFAULT);
+		}
+	} else if (sc.state == SCE_HBA_STRING) {
+		if (sc.ch == '\"') {
+			sc.ForwardSetState(SCE_HBA_DEFAULT);
+		} else if (sc.ch == '\r' || sc.ch == '\n') {
+			sc.ChangeState(SCE_HBA_STRINGEOL);
+			sc.ForwardSetState(SCE_HBA_DEFAULT);
+		}
+	} else if (sc.state == SCE_HBA_COMMENTLINE) {
+		if (sc.ch == '\r' || sc.ch == '\n') {
+			sc.SetState(SCE_HBA_DEFAULT);
+		}
+	}
+
+	if (sc.state == SCE_HBA_DEFAULT) {
+		if (IsADigit(sc.ch) || (sc.ch == '.' && IsADigit(sc.chNext))) {
+			sc.SetState(SCE_HBA_NUMBER);
+		} else if (IsAWordStart(sc.ch)) {
+			sc.SetState(SCE_HBA_WORD);
+		} else if (sc.ch == '\'') {
+			sc.SetState(SCE_HBA_COMMENTLINE);
+		} else if (sc.ch == '\"') {
+			sc.SetState(SCE_HBA_STRING);
+		}
+	}
+}
+
+static void ColouriseASPPiece(StyleContext &sc, WordList *keywordlists[]) {
+	// Possibly exit current state to either SCE_H_DEFAULT or SCE_HBA_DEFAULT
+	if ((sc.state == SCE_H_ASPAT || isASPScript(sc.state)) && sc.Match('%', '>')) {
+		sc.SetState(SCE_H_ASP);
+		sc.Forward();
+		sc.ForwardSetState(SCE_H_DEFAULT);
+	} 
+			 
+	// Handle some ASP script
+	if (sc.state >= SCE_HBA_START && sc.state <= SCE_HBA_STRINGEOL) {
+		ColouriseHBAPiece(sc, keywordlists);
+	}
+	
+	// Enter new sc.state
+	if (sc.state == SCE_H_DEFAULT) {
+		if (sc.Match('<', '%')) {
+			sc.SetState(SCE_H_ASP);
+			sc.Forward();
+			sc.Forward();
+			if (sc.ch == '@') {
+				sc.ForwardSetState(SCE_H_ASPAT);
+			} else {
+				if (sc.ch == '=') {
+					sc.Forward();
+				}
+				sc.SetState(SCE_HBA_DEFAULT);
+			}
+		}
+	}
+}
+
+static void ColouriseASPDoc(unsigned int startPos, int length, int initStyle, WordList *keywordlists[],
+                                  Accessor &styler) {
+	// Lexer for HTML requires more lexical states (7 bits worth) than most lexers
+	StyleContext sc(startPos, length, initStyle, styler, 0x7f);
+	for (; sc.More(); sc.Forward()) {
+		ColouriseASPPiece(sc, keywordlists);
+	}
+	sc.Complete();
+}
+
+static void ColourisePHPPiece(StyleContext &sc, WordList *[]) {
+	// Possibly exit current state to either SCE_H_DEFAULT or SCE_HBA_DEFAULT
+	if (sc.state >= SCE_HPHP_DEFAULT && sc.state <= SCE_HPHP_OPERATOR) {
+		if (!isPHPStringState(sc.state) && 
+			(sc.state != SCE_HPHP_COMMENT) &&
+			(sc.Match('?', '>'))) {
+			sc.SetState(SCE_H_QUESTION);
+			sc.Forward();
+			sc.ForwardSetState(SCE_H_DEFAULT);
+		}
+	} 
+			 
+	// Handle some PHP script
+	if (sc.state == SCE_HPHP_WORD) {
+		if (!IsAWordStart(sc.ch)) {
+			sc.SetState(SCE_HPHP_DEFAULT);
+		}
+	} else if (sc.state == SCE_HPHP_COMMENTLINE) {
+		if (sc.ch == '\r' || sc.ch == '\n') {
+			sc.SetState(SCE_HPHP_DEFAULT);
+		}
+	} else if (sc.state == SCE_HPHP_COMMENT) {
+		if (sc.Match('*', '/')) {
+			sc.Forward();
+			sc.Forward();
+			sc.SetState(SCE_HPHP_DEFAULT);
+		}
+	} else if (sc.state == SCE_HPHP_HSTRING) {
+		if (sc.ch == '\"') {
+			sc.ForwardSetState(SCE_HPHP_DEFAULT);
+		}
+	} else if (sc.state == SCE_HPHP_SIMPLESTRING) {
+		if (sc.ch == '\'') {
+			sc.ForwardSetState(SCE_HPHP_DEFAULT);
+		}
+	} else if (sc.state == SCE_HPHP_VARIABLE) {
+		if (!IsAWordStart(sc.ch)) {
+			sc.SetState(SCE_HPHP_DEFAULT);
+		}
+	} else if (sc.state == SCE_HPHP_OPERATOR) {
+		sc.SetState(SCE_HPHP_DEFAULT);
+	}
+	
+	// Enter new sc.state
+	if (sc.state == SCE_H_DEFAULT) {
+		if (sc.Match("<?php")) {
+			sc.SetState(SCE_H_QUESTION);
+			sc.Forward();
+			sc.Forward();
+			sc.Forward();
+			sc.Forward();
+			sc.Forward();
+			sc.SetState(SCE_HPHP_DEFAULT);
+		}
+	}
+	if (sc.state == SCE_HPHP_DEFAULT) {
+		if (IsAWordStart(sc.ch)) {
+			sc.SetState(SCE_HPHP_WORD);
+		} else if (sc.ch == '#') {
+			sc.SetState(SCE_HPHP_COMMENTLINE);
+		} else if (sc.Match("<!--")) {
+			sc.SetState(SCE_HPHP_COMMENTLINE);
+		} else if (sc.Match('/', '/')) {
+			sc.SetState(SCE_HPHP_COMMENTLINE);
+		} else if (sc.Match('/', '*')) {
+			sc.SetState(SCE_HPHP_COMMENT);
+		} else if (sc.ch == '\"') {
+			sc.SetState(SCE_HPHP_HSTRING);
+		} else if (sc.ch == '\'') {
+			sc.SetState(SCE_HPHP_SIMPLESTRING);
+		} else if (sc.ch == '$') {
+			sc.SetState(SCE_HPHP_VARIABLE);
+		} else if (isoperator(static_cast<char>(sc.ch))) {
+			sc.SetState(SCE_HPHP_OPERATOR);
+		}
+	}
+}
+
+static void ColourisePHPDoc(unsigned int startPos, int length, int initStyle, WordList *keywordlists[],
+                                  Accessor &styler) {
+	// Lexer for HTML requires more lexical states (7 bits worth) than most lexers
+	StyleContext sc(startPos, length, initStyle, styler, 0x7f);
+	for (; sc.More(); sc.Forward()) {
+		ColourisePHPPiece(sc, keywordlists);
+	}
+	sc.Complete();
+}
+
 LexerModule lmHTML(SCLEX_HTML, ColouriseHyperTextDoc, "hypertext");
 LexerModule lmXML(SCLEX_XML, ColouriseHyperTextDoc, "xml");
-
+LexerModule lmASP(SCLEX_ASP, ColouriseASPDoc, "asp");
+LexerModule lmPHP(SCLEX_PHP, ColourisePHPDoc, "php");
