@@ -8,6 +8,7 @@
 #include <stdlib.h>
 
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "Platform.h"
 
@@ -20,60 +21,29 @@
 #define FAST_WAY
 #endif
 
+#ifdef _MSC_VER
+// Ignore unreferenced local functions in GTK+ headers
+#pragma warning(disable: 4505)
+#endif
+
+static GdkFont *PFont(Font &f)  { 
+	return reinterpret_cast<GdkFont *>(f.GetID()); 
+}
+
+static GtkWidget *PWidget(WindowID id) { 
+	return reinterpret_cast<GtkWidget *>(id); 
+}
+
+static GtkWidget *PWidget(Window &w) { 
+	return PWidget(w.GetID()); 
+}
+
 Point Point::FromLong(long lpoint) {
 	return Point(
 	           Platform::LowShortFromLong(lpoint),
 	           Platform::HighShortFromLong(lpoint));
 }
 
-static GdkColor ColourfromRGB(unsigned int red, unsigned int green, unsigned int blue) {
-	GdkColor co;
-	co.red = red * (65535 / 255);
-	co.green = green * (65535 / 255);
-	co.blue = blue * (65535 / 255);
-	// the pixel value indicates the index in the colourmap of the colour.
-	// it is simply a combination of the RGB values we set earlier
-	co.pixel = (gulong)(red * 65536 + green * 256 + blue);
-	return co;
-}
-
-Colour::Colour(long lcol) {
-	unsigned int red = lcol & 0xff;
-	unsigned int green = (lcol >> 8) & 0xff;
-	unsigned int blue = lcol >> 16;
-	co = ColourfromRGB(red, green, blue);
-}
-
-Colour::Colour(unsigned int red, unsigned int green, unsigned int blue) {
-	co = ColourfromRGB(red, green, blue);
-}
-
-bool Colour::operator==(const Colour &other) const {
-	return
-	    co.red == other.co.red &&
-	    co.green == other.co.green &&
-	    co.blue == other.co.blue;
-}
-
-unsigned int Colour::GetRed() {
-	return co.red;
-}
-
-unsigned int Colour::GetGreen() {
-	return co.green;
-}
-
-unsigned int Colour::GetBlue() {
-	return co.blue;
-}
-
-long Colour::AsLong() const {
-	unsigned int red = co.red * 255 / 65535;
-	unsigned int green = co.green * 255 / 65535;
-	unsigned int blue = co.blue * 255 / 65535;
-	return (red + green*256 + blue*65536);
-}
-
 Palette::Palette() {
 	used = 0;
 	allowRealization = false;
@@ -87,7 +57,7 @@ Palette::~Palette() {
 
 void Palette::Release() {
 	used = 0;
-	delete []allocatedPalette;
+	delete [](reinterpret_cast<GdkColor *>(allocatedPalette));
 	allocatedPalette = 0;
 	allocatedLen = 0;
 }
@@ -104,7 +74,7 @@ void Palette::WantFind(ColourPair &cp, bool want) {
 
 		if (used < numEntries) {
 			entries[used].desired = cp.desired;
-			entries[used].allocated = cp.desired;
+			entries[used].allocated.Set(cp.desired.AsLong());
 			used++;
 		}
 	} else {
@@ -114,31 +84,36 @@ void Palette::WantFind(ColourPair &cp, bool want) {
 				return ;
 			}
 		}
-		cp.allocated = cp.desired;
+		cp.allocated.Set(cp.desired.AsLong());
 	}
 }
 
 void Palette::Allocate(Window &w) {
 	if (allocatedPalette) {
-		gdk_colormap_free_colors(gtk_widget_get_colormap(w.GetID()),
-		                         allocatedPalette, allocatedLen);
-		delete []allocatedPalette;
+		gdk_colormap_free_colors(gtk_widget_get_colormap(PWidget(w)),
+		        reinterpret_cast<GdkColor *>(allocatedPalette), 
+			allocatedLen);
+		delete [](reinterpret_cast<GdkColor *>(allocatedPalette));
 		allocatedPalette = 0;
 		allocatedLen = 0;
 	}
-	allocatedPalette = new GdkColor[used];
+	GdkColor *paletteNew = new GdkColor[used];
+	allocatedPalette = paletteNew;
 	gboolean *successPalette = new gboolean[used];
-	if (allocatedPalette) {
+	if (paletteNew) {
 		allocatedLen = used;
 		int iPal = 0;
 		for (iPal = 0; iPal < used; iPal++) {
-			allocatedPalette[iPal] = entries[iPal].desired.co;
+			paletteNew[iPal].red = entries[iPal].desired.GetRed() * (65535 / 255);
+			paletteNew[iPal].green = entries[iPal].desired.GetGreen() * (65535 / 255);
+			paletteNew[iPal].blue = entries[iPal].desired.GetBlue() * (65535 / 255);
+			paletteNew[iPal].pixel = entries[iPal].desired.AsLong();
 		}
-		gdk_colormap_alloc_colors(gtk_widget_get_colormap(w.GetID()),
-		                          allocatedPalette, allocatedLen, FALSE, TRUE,
+		gdk_colormap_alloc_colors(gtk_widget_get_colormap(PWidget(w)),
+		                          paletteNew, allocatedLen, FALSE, TRUE,
 		                          successPalette);
 		for (iPal = 0; iPal < used; iPal++) {
-			entries[iPal].allocated.co = allocatedPalette[iPal];
+			entries[iPal].allocated.Set(paletteNew[iPal].pixel);
 		}
 	}
 	delete []successPalette;
@@ -151,45 +126,45 @@ Font::~Font() {}
 static const char *CharacterSetName(int characterSet) {
 	switch (characterSet) {
 	case SC_CHARSET_ANSI:
-		return "iso8859";
+		return "iso8859-*";
 	case SC_CHARSET_DEFAULT:
-		return "iso8859";
+		return "iso8859-*";
 	case SC_CHARSET_BALTIC:
-		return "*";
+		return "*-*";
 	case SC_CHARSET_CHINESEBIG5:
-		return "*";
+		return "*-*";
 	case SC_CHARSET_EASTEUROPE:
-		return "iso8859-2";
+		return "*-2";
 	case SC_CHARSET_GB2312:
-		return "gb2312.1980";
+		return "gb2312.1980-*";
 	case SC_CHARSET_GREEK:
-		return "adobe";
+		return "*-7";
 	case SC_CHARSET_HANGUL:
-		return "ksc5601.1987";
+		return "ksc5601.1987-*";
 	case SC_CHARSET_MAC:
-		return "*";
+		return "*-*";
 	case SC_CHARSET_OEM:
-		return "*";
+		return "*-*";
 	case SC_CHARSET_RUSSIAN:
-		return "*";
+		return "*-r";
 	case SC_CHARSET_SHIFTJIS:
-		return "jisx0208.1983";
+		return "jisx0208.1983-*";
 	case SC_CHARSET_SYMBOL:
-		return "*";
+		return "*-*";
 	case SC_CHARSET_TURKISH:
-		return "*";
+		return "*-*";
 	case SC_CHARSET_JOHAB:
-		return "*";
+		return "*-*";
 	case SC_CHARSET_HEBREW:
-		return "*";
+		return "*-8";
 	case SC_CHARSET_ARABIC:
-		return "*";
+		return "*-6";
 	case SC_CHARSET_VIETNAMESE:
-		return "*";
+		return "*-*";
 	case SC_CHARSET_THAI:
-		return "*";
+		return "*-*";
 	default:
-		return "*";
+		return "*-*";
 	}
 }
 
@@ -221,7 +196,6 @@ void Font::Create(const char *faceName, int characterSet,
 	strcat(fontspec, sizePts);
 	strcat(fontspec, "-*-*-*-*-");
 	strcat(fontspec, CharacterSetName(characterSet));
-	strcat(fontspec, "-*");
 	id = gdk_font_load(fontspec);
 	if (!id) {
 		// Font not available so substitute a reasonable code font
@@ -232,18 +206,69 @@ void Font::Create(const char *faceName, int characterSet,
 
 void Font::Release() {
 	if (id)
-		gdk_font_unref(id);
+		gdk_font_unref(PFont(*this));
 	id = 0;
 }
 
-Surface::Surface() : unicodeMode(false), drawable(0), gc(0), ppixmap(0),
+class SurfaceImpl : public Surface {
+	bool unicodeMode;
+	GdkDrawable *drawable;
+	GdkGC *gc;
+	GdkPixmap *ppixmap;
+	int x;
+	int y;
+	bool inited;
+	bool createdGC;
+public:
+	SurfaceImpl();
+	virtual ~SurfaceImpl();
+
+	void Init();
+	void Init(SurfaceID sid);
+	void InitPixMap(int width, int height, Surface *surface_);
+
+	void Release();
+	bool Initialised();
+	void PenColour(ColourAllocated fore);
+	int LogPixelsY();
+	int DeviceHeightFont(int points);
+	void MoveTo(int x_, int y_);
+	void LineTo(int x_, int y_);
+	void Polygon(Point *pts, int npts, ColourAllocated fore, ColourAllocated back);
+	void RectangleDraw(PRectangle rc, ColourAllocated fore, ColourAllocated back);
+	void FillRectangle(PRectangle rc, ColourAllocated back);
+	void FillRectangle(PRectangle rc, Surface &surfacePattern);
+	void RoundedRectangle(PRectangle rc, ColourAllocated fore, ColourAllocated back);
+	void Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back);
+	void Copy(PRectangle rc, Point from, Surface &surfaceSource);
+
+	void DrawTextNoClip(PRectangle rc, Font &font_, int ybase, const char *s, int len, ColourAllocated fore, ColourAllocated back);
+	void DrawTextClipped(PRectangle rc, Font &font_, int ybase, const char *s, int len, ColourAllocated fore, ColourAllocated back);
+	void MeasureWidths(Font &font_, const char *s, int len, int *positions);
+	int WidthText(Font &font_, const char *s, int len);
+	int WidthChar(Font &font_, char ch);
+	int Ascent(Font &font_);
+	int Descent(Font &font_);
+	int InternalLeading(Font &font_);
+	int ExternalLeading(Font &font_);
+	int Height(Font &font_);
+	int AverageCharWidth(Font &font_);
+
+	int SetPalette(Palette *pal, bool inBackGround);
+	void SetClip(PRectangle rc);
+	void FlushCachedState();
+
+	void SetUnicodeMode(bool unicodeMode_);
+};
+
+SurfaceImpl::SurfaceImpl() : unicodeMode(false), drawable(0), gc(0), ppixmap(0),
 x(0), y(0), inited(false), createdGC(false) {}
 
-Surface::~Surface() {
+SurfaceImpl::~SurfaceImpl() {
 	Release();
 }
 
-void Surface::Release() {
+void SurfaceImpl::Release() {
 	drawable = 0;
 	if (createdGC) {
 		createdGC = false;
@@ -259,53 +284,61 @@ void Surface::Release() {
 	createdGC = false;
 }
 
-bool Surface::Initialised() {
+bool SurfaceImpl::Initialised() {
 	return inited;
 }
 
-void Surface::Init() {
+void SurfaceImpl::Init() {
 	Release();
 	inited = true;
 }
 
-void Surface::Init(GdkDrawable *drawable_) {
+void SurfaceImpl::Init(SurfaceID sid) {
+	GdkDrawable *drawable_ = reinterpret_cast<GdkDrawable *>(sid);
 	Release();
 	drawable = drawable_;
 	gc = gdk_gc_new(drawable_);
+	//gdk_gc_set_line_attributes(gc, 1, 
+	//	GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_BEVEL);
 	createdGC = true;
 	inited = true;
 }
 
-void Surface::InitPixMap(int width, int height, Surface *surface_) {
+void SurfaceImpl::InitPixMap(int width, int height, Surface *surface_) {
 	Release();
 	if (height > 0 && width > 0)
-		ppixmap = gdk_pixmap_new(surface_->drawable, width, height, -1);
+		ppixmap = gdk_pixmap_new(static_cast<SurfaceImpl *>(surface_)->drawable, width, height, -1);
 	drawable = ppixmap;
-	gc = gdk_gc_new(surface_->drawable);
+	gc = gdk_gc_new(static_cast<SurfaceImpl *>(surface_)->drawable);
+	//gdk_gc_set_line_attributes(gc, 1, 
+	//	GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_BEVEL);
 	createdGC = true;
 	inited = true;
 }
 
-void Surface::PenColour(Colour fore) {
-	if (gc)
-		gdk_gc_set_foreground(gc, &fore.co);
+void SurfaceImpl::PenColour(ColourAllocated fore) {
+	if (gc) {
+		GdkColor co;
+		co.pixel = fore.AsLong();
+		gdk_gc_set_foreground(gc, &co);
+	}
 }
 
-int Surface::LogPixelsY() {
+int SurfaceImpl::LogPixelsY() {
 	return 72;
 }
 
-int Surface::DeviceHeightFont(int points) {
+int SurfaceImpl::DeviceHeightFont(int points) {
 	int logPix = LogPixelsY();
 	return (points * logPix + logPix / 2) / 72;
 }
 
-void Surface::MoveTo(int x_, int y_) {
+void SurfaceImpl::MoveTo(int x_, int y_) {
 	x = x_;
 	y = y_;
 }
 
-void Surface::LineTo(int x_, int y_) {
+void SurfaceImpl::LineTo(int x_, int y_) {
 	gdk_draw_line(drawable, gc,
 	              x, y,
 	              x_, y_);
@@ -313,10 +346,8 @@ void Surface::LineTo(int x_, int y_) {
 	y = y_;
 }
 
-void Surface::Polygon(Point *pts, int npts, Colour fore,
-                      Colour back) {
-	// Nasty casts works because Point is exactly same as GdkPoint
-	// Oh no it doesn't...
+void SurfaceImpl::Polygon(Point *pts, int npts, ColourAllocated fore,
+                      ColourAllocated back) {
 	GdkPoint gpts[20];
 	if (npts < static_cast<int>((sizeof(gpts) / sizeof(gpts[0])))) {
 		for (int i = 0;i < npts;i++) {
@@ -324,41 +355,39 @@ void Surface::Polygon(Point *pts, int npts, Colour fore,
 			gpts[i].y = pts[i].y;
 		}
 		PenColour(back);
-		//gdk_draw_polygon(drawable, gc, 1,
-		//                 reinterpret_cast<GdkPoint *>(pts), npts);
 		gdk_draw_polygon(drawable, gc, 1, gpts, npts);
 		PenColour(fore);
 		gdk_draw_polygon(drawable, gc, 0, gpts, npts);
 	}
 }
 
-void Surface::RectangleDraw(PRectangle rc, Colour fore, Colour back) {
+void SurfaceImpl::RectangleDraw(PRectangle rc, ColourAllocated fore, ColourAllocated back) {
 	if (gc && drawable) {
 		PenColour(back);
 		gdk_draw_rectangle(drawable, gc, 1,
-		                   rc.left, rc.top,
-		                   rc.right - rc.left + 1, rc.bottom - rc.top + 1);
+		                   rc.left + 1, rc.top + 1,
+		                   rc.right - rc.left - 2, rc.bottom - rc.top - 2);
+
 		PenColour(fore);
+		// The subtraction of 1 off the width and height here shouldn't be needed but 
+		// otherwise a different rectangle is drawn than would be done if the fill parameter == 1
 		gdk_draw_rectangle(drawable, gc, 0,
 		                   rc.left, rc.top,
-		                   rc.right - rc.left + 1, rc.bottom - rc.top + 1);
+		                   rc.right - rc.left - 1, rc.bottom - rc.top - 1);
 	}
 }
 
-void Surface::FillRectangle(PRectangle rc, Colour back) {
-	// GTK+ rectangles include their lower and right edges
-	rc.bottom--;
-	rc.right--;
+void SurfaceImpl::FillRectangle(PRectangle rc, ColourAllocated back) {
 	PenColour(back);
 	if (drawable) {
 		gdk_draw_rectangle(drawable, gc, 1,
 		                   rc.left, rc.top,
-		                   rc.right - rc.left + 1, rc.bottom - rc.top + 1);
+		                   rc.right - rc.left, rc.bottom - rc.top);
 	}
 }
 
-void Surface::FillRectangle(PRectangle rc, Surface &surfacePattern) {
-	if (surfacePattern.drawable) {
+void SurfaceImpl::FillRectangle(PRectangle rc, Surface &surfacePattern) {
+	if (static_cast<SurfaceImpl &>(surfacePattern).drawable) {
 		// Tile pattern over rectangle
 		// Currently assumes 8x8 pattern
 		int widthPat = 8;
@@ -369,7 +398,7 @@ void Surface::FillRectangle(PRectangle rc, Surface &surfacePattern) {
 				int heighty = (yTile + heightPat > rc.bottom) ? rc.bottom - yTile : heightPat;
 				gdk_draw_pixmap(drawable,
 				                gc,
-				                surfacePattern.drawable,
+				                static_cast<SurfaceImpl &>(surfacePattern).drawable,
 				                0, 0,
 				                xTile, yTile,
 				                widthx, heighty);
@@ -378,11 +407,11 @@ void Surface::FillRectangle(PRectangle rc, Surface &surfacePattern) {
 	} else {
 		// Something is wrong so try to show anyway
 		// Shows up black because colour not allocated
-		FillRectangle(rc, Colour(0xff, 0, 0));
+		FillRectangle(rc, ColourAllocated(0));
 	}
 }
 
-void Surface::RoundedRectangle(PRectangle rc, Colour fore, Colour back) {
+void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourAllocated fore, ColourAllocated back) {
 	if (((rc.right - rc.left) > 4) && ((rc.bottom - rc.top) > 4)) {
 		// Approximate a round rect with some cut off corners
 		Point pts[] = {
@@ -401,52 +430,54 @@ void Surface::RoundedRectangle(PRectangle rc, Colour fore, Colour back) {
 	}
 }
 
-void Surface::Ellipse(PRectangle rc, Colour fore, Colour back) {
+void SurfaceImpl::Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back) {
 	PenColour(back);
 	gdk_draw_arc(drawable, gc, 1,
-	             rc.left, rc.top,
-	             rc.right - rc.left, rc.bottom - rc.top,
+	             rc.left + 1, rc.top + 1,
+	             rc.right - rc.left - 2, rc.bottom - rc.top - 2,
 	             0, 32767);
+
+	// The subtraction of 1 here is similar to the case for RectangleDraw
 	PenColour(fore);
 	gdk_draw_arc(drawable, gc, 0,
 	             rc.left, rc.top,
-	             rc.right - rc.left, rc.bottom - rc.top,
+	             rc.right - rc.left - 1, rc.bottom - rc.top - 1,
 	             0, 32767);
 }
 
-void Surface::Copy(PRectangle rc, Point from, Surface &surfaceSource) {
-	if (surfaceSource.drawable) {
+void SurfaceImpl::Copy(PRectangle rc, Point from, Surface &surfaceSource) {
+	if (static_cast<SurfaceImpl &>(surfaceSource).drawable) {
 		gdk_draw_pixmap(drawable,
 		                gc,
-		                surfaceSource.drawable,
+		                static_cast<SurfaceImpl &>(surfaceSource).drawable,
 		                from.x, from.y,
 		                rc.left, rc.top,
 		                rc.right - rc.left, rc.bottom - rc.top);
 	}
 }
 
-void Surface::DrawText(PRectangle rc, Font &font_, int ybase, const char *s, int len,
-                       Colour fore, Colour back) {
+void SurfaceImpl::DrawTextNoClip(PRectangle rc, Font &font_, int ybase, const char *s, int len,
+                       ColourAllocated fore, ColourAllocated back) {
 	FillRectangle(rc, back);
 	PenColour(fore);
 	if (gc && drawable)
-		gdk_draw_text(drawable, font_.id, gc, rc.left, ybase, s, len);
+		gdk_draw_text(drawable, PFont(font_), gc, rc.left, ybase, s, len);
 }
 
-// On GTK+, exactly same as DrawText
-void Surface::DrawTextClipped(PRectangle rc, Font &font_, int ybase, const char *s, int len,
-                              Colour fore, Colour back) {
+// On GTK+, exactly same as DrawTextNoClip
+void SurfaceImpl::DrawTextClipped(PRectangle rc, Font &font_, int ybase, const char *s, int len,
+                              ColourAllocated fore, ColourAllocated back) {
 	FillRectangle(rc, back);
 	PenColour(fore);
 	if (gc && drawable)
-		gdk_draw_text(drawable, font_.id, gc, rc.left, ybase, s, len);
+		gdk_draw_text(drawable, PFont(font_), gc, rc.left, ybase, s, len);
 }
 
-void Surface::MeasureWidths(Font &font_, const char *s, int len, int *positions) {
+void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, int *positions) {
 	int totalWidth = 0;
 	for (int i = 0;i < len;i++) {
-		if (font_.id) {
-			int width = gdk_char_width(font_.id, s[i]);
+		if (font_.GetID()) {
+			int width = gdk_char_width(PFont(font_), s[i]);
 			totalWidth += width;
 		} else {
 			totalWidth++;
@@ -455,16 +486,16 @@ void Surface::MeasureWidths(Font &font_, const char *s, int len, int *positions)
 	}
 }
 
-int Surface::WidthText(Font &font_, const char *s, int len) {
-	if (font_.id)
-		return gdk_text_width(font_.id, s, len);
+int SurfaceImpl::WidthText(Font &font_, const char *s, int len) {
+	if (font_.GetID())
+		return gdk_text_width(PFont(font_), s, len);
 	else
 		return 1;
 }
 
-int Surface::WidthChar(Font &font_, char ch) {
-	if (font_.id)
-		return gdk_char_width(font_.id, ch);
+int SurfaceImpl::WidthChar(Font &font_, char ch) {
+	if (font_.GetID())
+		return gdk_char_width(PFont(font_), ch);
 	else
 		return 1;
 }
@@ -484,11 +515,11 @@ const char largeSizeString[] = " `~!@#$%^&*()-_=+\\|[]{};:\"\'<,>.?/1234567890"
 const char sizeString[] = "`~!@#$%^&*()-_=+\\|[]{};:\"\'<,>.?/1234567890"
                           "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
-int Surface::Ascent(Font &font_) {
-	if (!font_.id)
+int SurfaceImpl::Ascent(Font &font_) {
+	if (!font_.GetID())
 		return 1;
 #ifdef FAST_WAY
-	return font_.id->ascent;
+	return PFont(font_)->ascent;
 #else
 	gint lbearing;
 	gint rbearing;
@@ -496,17 +527,17 @@ int Surface::Ascent(Font &font_) {
 	gint ascent;
 	gint descent;
 
-	gdk_string_extents(font_.id, sizeString,
+	gdk_string_extents(PFont(font_), sizeString,
 	                   &lbearing, &rbearing, &width, &ascent, &descent);
 	return ascent;
 #endif
 }
 
-int Surface::Descent(Font &font_) {
-	if (!font_.id)
+int SurfaceImpl::Descent(Font &font_) {
+	if (!font_.GetID())
 		return 1;
 #ifdef FAST_WAY
-	return font_.id->descent;
+	return PFont(font_)->descent;
 #else
 	gint lbearing;
 	gint rbearing;
@@ -514,43 +545,51 @@ int Surface::Descent(Font &font_) {
 	gint ascent;
 	gint descent;
 
-	gdk_string_extents(font_.id, sizeString,
+	gdk_string_extents(PFont(font_), sizeString,
 	                   &lbearing, &rbearing, &width, &ascent, &descent);
 	return descent;
 #endif
 }
 
-int Surface::InternalLeading(Font &) {
+int SurfaceImpl::InternalLeading(Font &) {
 	return 0;
 }
 
-int Surface::ExternalLeading(Font &) {
+int SurfaceImpl::ExternalLeading(Font &) {
 	return 0;
 }
 
-int Surface::Height(Font &font_) {
+int SurfaceImpl::Height(Font &font_) {
 	return Ascent(font_) + Descent(font_);
 }
 
-int Surface::AverageCharWidth(Font &font_) {
-	if (font_.id)
-		return gdk_char_width(font_.id, 'n');
+int SurfaceImpl::AverageCharWidth(Font &font_) {
+	if (font_.GetID())
+		return gdk_char_width(PFont(font_), 'n');
 	else
 		return 1;
 }
 
-int Surface::SetPalette(Palette *, bool) {
+int SurfaceImpl::SetPalette(Palette *, bool) {
 	// Handled in palette allocation for GTK so this does nothing
 	return 0;
 }
 
-void Surface::SetClip(PRectangle rc) {
+void SurfaceImpl::SetClip(PRectangle rc) {
 	GdkRectangle area = {rc.left, rc.top,
 	                     rc.right - rc.left, rc.bottom - rc.top};
 	gdk_gc_set_clip_rectangle(gc, &area);
 }
 
-void Surface::FlushCachedState() {}
+void SurfaceImpl::FlushCachedState() {}
+
+void SurfaceImpl::SetUnicodeMode(bool unicodeMode_) {
+	unicodeMode=unicodeMode_;
+}
+
+Surface *Surface::Allocate() {
+	return new SurfaceImpl;
+}
 
 Window::~Window() {}
 
@@ -568,11 +607,11 @@ PRectangle Window::GetPosition() {
 	// Before any size allocated pretend its 1000 wide so not scrolled
 	PRectangle rc(0, 0, 1000, 1000);
 	if (id) {
-		rc.left = id->allocation.x;
-		rc.top = id->allocation.y;
-		if (id->allocation.width > 20) {
-			rc.right = rc.left + id->allocation.width;
-			rc.bottom = rc.top + id->allocation.height;
+		rc.left = PWidget(id)->allocation.x;
+		rc.top = PWidget(id)->allocation.y;
+		if (PWidget(id)->allocation.width > 20) {
+			rc.right = rc.left + PWidget(id)->allocation.width;
+			rc.bottom = rc.top + PWidget(id)->allocation.height;
 		}
 	}
 	return rc;
@@ -586,7 +625,7 @@ void Window::SetPosition(PRectangle rc) {
 	alloc.y = rc.top;
 	alloc.width = rc.Width();
 	alloc.height = rc.Height();
-	gtk_widget_size_allocate(id, &alloc);
+	gtk_widget_size_allocate(PWidget(id), &alloc);
 #else
 	gtk_widget_set_uposition(id, rc.left, rc.top);
 	gtk_widget_set_usize(id, rc.right - rc.left, rc.bottom - rc.top);
@@ -596,9 +635,9 @@ void Window::SetPosition(PRectangle rc) {
 void Window::SetPositionRelative(PRectangle rc, Window relativeTo) {
 	int ox = 0;
 	int oy = 0;
-	gdk_window_get_origin(relativeTo.id->window, &ox, &oy);
+	gdk_window_get_origin(PWidget(relativeTo.id)->window, &ox, &oy);
 
-	gtk_widget_set_uposition(id, rc.left + ox, rc.top + oy);
+	gtk_widget_set_uposition(PWidget(id), rc.left + ox, rc.top + oy);
 #if 0
 	GtkAllocation alloc;
 	alloc.x = rc.left + ox;
@@ -607,7 +646,7 @@ void Window::SetPositionRelative(PRectangle rc, Window relativeTo) {
 	alloc.height = rc.bottom - rc.top;
 	gtk_widget_size_allocate(id, &alloc);
 #endif
-	gtk_widget_set_usize(id, rc.right - rc.left, rc.bottom - rc.top);
+	gtk_widget_set_usize(PWidget(id), rc.right - rc.left, rc.bottom - rc.top);
 }
 
 PRectangle Window::GetClientPosition() {
@@ -617,18 +656,18 @@ PRectangle Window::GetClientPosition() {
 
 void Window::Show(bool show) {
 	if (show)
-		gtk_widget_show(id);
+		gtk_widget_show(PWidget(id));
 }
 
 void Window::InvalidateAll() {
 	if (id) {
-		gtk_widget_queue_draw(id);
+		gtk_widget_queue_draw(PWidget(id));
 	}
 }
 
 void Window::InvalidateRectangle(PRectangle rc) {
 	if (id) {
-		gtk_widget_queue_draw_area(id,
+		gtk_widget_queue_draw_area(PWidget(id),
 		                           rc.left, rc.top,
 		                           rc.right - rc.left, rc.bottom - rc.top);
 	}
@@ -639,34 +678,38 @@ void Window::SetFont(Font &) {
 }
 
 void Window::SetCursor(Cursor curs) {
-	if(!id->window) return; // Varify if the widow exists.
+	GdkCursor *gdkCurs;
 	switch (curs) {
 	case cursorText:
-		gdk_window_set_cursor(id->window, gdk_cursor_new(GDK_XTERM));
+		gdkCurs = gdk_cursor_new(GDK_XTERM);
 		break;
 	case cursorArrow:
-		gdk_window_set_cursor(id->window, gdk_cursor_new(GDK_ARROW));
+		gdkCurs = gdk_cursor_new(GDK_ARROW);
 		break;
 	case cursorUp:
-		gdk_window_set_cursor(id->window, gdk_cursor_new(GDK_CENTER_PTR));
+		gdkCurs = gdk_cursor_new(GDK_CENTER_PTR);
 		break;
 	case cursorWait:
-		gdk_window_set_cursor(id->window, gdk_cursor_new(GDK_WATCH));
+		gdkCurs = gdk_cursor_new(GDK_WATCH);
 		break;
 	case cursorReverseArrow:
-		gdk_window_set_cursor(id->window, gdk_cursor_new(GDK_TOP_LEFT_ARROW));
+		gdkCurs = gdk_cursor_new(GDK_TOP_LEFT_ARROW);
 		break;
 	default:
-		gdk_window_set_cursor(id->window, gdk_cursor_new(GDK_ARROW));
+		gdkCurs = gdk_cursor_new(GDK_ARROW);
 		break;
 	}
+	
+	gdk_window_set_cursor(PWidget(id)->window, gdkCurs);
+	gdk_cursor_destroy(gdkCurs);
 }
 
 void Window::SetTitle(const char *s) {
 	gtk_window_set_title(GTK_WINDOW(id), s);
 }
 
-ListBox::ListBox() : list(0), current(0), desiredVisibleRows(5), maxItemCharacters(0) {}
+ListBox::ListBox() : list(0), current(0), desiredVisibleRows(5), maxItemCharacters(0),
+	doubleClickAction(NULL), doubleClickActionData(NULL) {}
 
 ListBox::~ListBox() {}
 
@@ -676,10 +719,20 @@ static void SelectionAC(GtkWidget *, gint row, gint,
 	*pi = row;
 }
 
+static gboolean ButtonPress(GtkWidget *, GdkEventButton* ev, gpointer p) {
+	ListBox* lb = reinterpret_cast<ListBox*>(p);
+	if (ev->type == GDK_2BUTTON_PRESS && lb->doubleClickAction != NULL) {
+		lb->doubleClickAction(lb->doubleClickActionData);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 void ListBox::Create(Window &, int) {
 	id = gtk_window_new(GTK_WINDOW_POPUP);
 
-	GtkWidget* frame = gtk_frame_new(NULL);
+	GtkWidget *frame = gtk_frame_new(NULL);
 	gtk_widget_show (frame);
 	gtk_container_add(GTK_CONTAINER(GetID()), frame);
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
@@ -689,31 +742,33 @@ void ListBox::Create(Window &, int) {
 	gtk_container_set_border_width(GTK_CONTAINER(scroller), 0);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
 	                               GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	gtk_container_add(GTK_CONTAINER(frame), scroller);
-	gtk_widget_show(scroller);
+	gtk_container_add(GTK_CONTAINER(frame), PWidget(scroller));
+	gtk_widget_show(PWidget(scroller));
 
 	list = gtk_clist_new(1);
-	gtk_widget_show(list);
-	gtk_container_add(GTK_CONTAINER(scroller), list);
-	gtk_clist_set_column_auto_resize(GTK_CLIST(list), 0, TRUE);
-	gtk_clist_set_selection_mode(GTK_CLIST(list), GTK_SELECTION_BROWSE);
-	gtk_signal_connect(GTK_OBJECT(list), "select_row",
+	gtk_widget_show(PWidget(list));
+	gtk_container_add(GTK_CONTAINER(PWidget(scroller)), PWidget(list));
+	gtk_clist_set_column_auto_resize(GTK_CLIST(PWidget(list)), 0, TRUE);
+	gtk_clist_set_selection_mode(GTK_CLIST(PWidget(list)), GTK_SELECTION_BROWSE);
+	gtk_signal_connect(GTK_OBJECT(PWidget(list)), "select_row",
 	                   GTK_SIGNAL_FUNC(SelectionAC), &current);
-	gtk_clist_set_shadow_type(GTK_CLIST(list), GTK_SHADOW_NONE);
+	gtk_signal_connect(GTK_OBJECT(PWidget(list)), "button_press_event",
+	                   GTK_SIGNAL_FUNC(ButtonPress), this);
+	gtk_clist_set_shadow_type(GTK_CLIST(PWidget(list)), GTK_SHADOW_NONE);
 
-	gtk_widget_realize(id);
+	gtk_widget_realize(PWidget(id));
 }
 
 void ListBox::SetFont(Font &scint_font) {
 	GtkStyle* style;
 
-	style = gtk_widget_get_style(GTK_WIDGET(list));
-	if (!gdk_font_equal(style->font, scint_font.GetID())) {
+	style = gtk_widget_get_style(GTK_WIDGET(PWidget(list)));
+	if (!gdk_font_equal(style->font, PFont(scint_font))) {
 		style = gtk_style_copy(style);
 		gdk_font_unref(style->font);
-		style->font = scint_font.GetID();
+		style->font = PFont(scint_font);
 		gdk_font_ref(style->font);
-		gtk_widget_set_style(GTK_WIDGET(list), style);
+		gtk_widget_set_style(GTK_WIDGET(PWidget(list)), style);
 		gtk_style_unref(style);
 	}
 }
@@ -740,9 +795,9 @@ PRectangle ListBox::GetDesiredRect() {
 		// First calculate height of the clist for our desired visible row count otherwise it tries to expand to the total # of rows
 		height = (rows * GTK_CLIST(list)->row_height
 		          + rows + 1
-		          + 2 * (list->style->klass->ythickness
-		                 + GTK_CONTAINER(list)->border_width));
-		gtk_widget_set_usize(GTK_WIDGET(list), -1, height);
+		          + 2 * (PWidget(list)->style->klass->ythickness
+		                 + GTK_CONTAINER(PWidget(list))->border_width));
+		gtk_widget_set_usize(GTK_WIDGET(PWidget(list)), -1, height);
 
 		// Get the size of the scroller because we set usize on the window
 		gtk_widget_size_request(GTK_WIDGET(scroller), &req);
@@ -828,15 +883,15 @@ void Menu::Destroy() {
 }
 
 void Menu::Show(Point pt, Window &) {
-	gtk_item_factory_popup(id, pt.x - 4, pt.y, 3, 0);
+	gtk_item_factory_popup(reinterpret_cast<GtkItemFactory *>(id), pt.x - 4, pt.y, 3, 0);
 }
 
-Colour Platform::Chrome() {
-	return Colour(0xe0, 0xe0, 0xe0);
+ColourDesired Platform::Chrome() {
+	return ColourDesired(0xe0, 0xe0, 0xe0);
 }
 
-Colour Platform::ChromeHighlight() {
-	return Colour(0xff, 0xff, 0xff);
+ColourDesired Platform::ChromeHighlight() {
+	return ColourDesired(0xff, 0xff, 0xff);
 }
 
 const char *Platform::DefaultFont() {
@@ -873,6 +928,10 @@ long Platform::SendScintilla(
 	return scintilla_send_message(SCINTILLA(w), msg, wParam, lParam);
 }
 
+bool Platform::IsDBCSLeadByte(int /*codePage*/, char /*ch*/) {
+	return false;
+}
+
 // These are utility functions not really tied to a platform
 
 int Platform::Minimum(int a, int b) {
