@@ -9,8 +9,15 @@
 #include <ctype.h>
 #include <time.h>
 
+#include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
+
 #include "Platform.h"
 
+#if PLAT_GTK_WIN32
+#include "Windows.h"
+#endif
+
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 #ifdef SCI_LEXER
@@ -39,6 +46,8 @@
 #ifdef _MSC_VER
 // Constant conditional expressions are because of GTK+ headers
 #pragma warning(disable: 4127)
+// Ignore unreferenced local functions in GTK+ headers
+#pragma warning(disable: 4505)
 #endif
 
 class ScintillaGTK : public ScintillaBase {
@@ -49,21 +58,34 @@ class ScintillaGTK : public ScintillaBase {
 	GtkObject *adjustmenth;
 	int scrollBarWidth;
 	int scrollBarHeight;
-	char *pasteBuffer;
-	bool pasteBufferIsRectangular;
+	
+	// Because clipboard access is asynchronous, copyText is created by Copy
+	SelectionText copyText;
+	
+	SelectionText primary;
+	
 	GdkEventButton evbtn;
 	bool capturedMouse;
 	bool dragWasDropped;
-	char *primarySelectionCopy;
 
 	GtkWidgetClass *parentClass;
 
 	static GdkAtom clipboard_atom;
+ 
+#if PLAT_GTK_WIN32
+	CLIPFORMAT cfColumnSelect;
+#endif
 
 	// Input context used for supporting internationalized key entry
 	GdkIC *ic;
 	GdkICAttr *ic_attr;
 
+	// Wheel mouse support
+	unsigned int linesPerScroll;
+	GTimeVal lastWheelMouseTime;
+	gint lastWheelMouseDirection;       
+	gint wheelMouseIntensity;
+
 	// Private so ScintillaGTK objects can not be copied
 ScintillaGTK(const ScintillaGTK &) : ScintillaBase() {}
 	ScintillaGTK &operator=(const ScintillaGTK &) { return * this; }
@@ -71,7 +93,7 @@ ScintillaGTK(const ScintillaGTK &) : ScintillaBase() {}
 public:
 	ScintillaGTK(_ScintillaObject *sci_);
 	virtual ~ScintillaGTK();
-	static void ClassInit(GtkWidgetClass *widget_class);
+	static void ClassInit(GtkObjectClass* object_class, GtkWidgetClass *widget_class);
 
 private:
 	virtual void Initialise();
@@ -106,7 +128,7 @@ private:
 	virtual void ClaimSelection();
 	void ReceivedSelection(GtkSelectionData *selection_data);
 	void ReceivedDrop(GtkSelectionData *selection_data);
-	void GetSelection(GtkSelectionData *selection_data, guint info, char *text, bool isRectangular);
+	void GetSelection(GtkSelectionData *selection_data, guint info, SelectionText *selected);
 	void UnclaimSelection(GdkEventSelection *selection_event);
 	void Resize(int width, int height);
 
@@ -130,12 +152,16 @@ private:
 
 	static void ScrollSignal(GtkAdjustment *adj, ScintillaGTK *sciThis);
 	static void ScrollHSignal(GtkAdjustment *adj, ScintillaGTK *sciThis);
+	gint PressThis(GdkEventButton *event);
 	static gint Press(GtkWidget *widget, GdkEventButton *event);
 	static gint MouseRelease(GtkWidget *widget, GdkEventButton *event);
+#if PLAT_GTK_WIN32
+	static gint ScrollEvent(GtkWidget *widget, GdkEventScroll *event);
+#endif
 	static gint Motion(GtkWidget *widget, GdkEventMotion *event);
 	static gint KeyPress(GtkWidget *widget, GdkEventKey *event);
 	static gint KeyRelease(GtkWidget *widget, GdkEventKey *event);
-	static gint DestroyWindow(GtkWidget *widget, GdkEventAny *event);
+	static void Destroy(GtkObject *object);
 	static void SelectionReceived(GtkWidget *widget, GtkSelectionData *selection_data,
 	                              guint time);
 	static void SelectionGet(GtkWidget *widget, GtkSelectionData *selection_data,
@@ -168,6 +194,7 @@ enum {
 };
 
 static gint scintilla_signals[LAST_SIGNAL] = { 0 };
+static GtkWidgetClass* parent_class = NULL;
 
 GdkAtom ScintillaGTK::clipboard_atom = GDK_NONE;
 
@@ -177,6 +204,10 @@ enum {
     TARGET_COMPOUND_TEXT
 };
 
+static GtkWidget *PWidget(Window &w) { 
+	return reinterpret_cast<GtkWidget *>(w.GetID()); 
+}
+
 static ScintillaGTK *ScintillaFromWidget(GtkWidget *widget) {
 	ScintillaObject *scio = reinterpret_cast<ScintillaObject *>(widget);
 	return reinterpret_cast<ScintillaGTK *>(scio->pscin);
@@ -185,18 +216,35 @@ static ScintillaGTK *ScintillaFromWidget(GtkWidget *widget) {
 ScintillaGTK::ScintillaGTK(_ScintillaObject *sci_) :
 		adjustmentv(0), adjustmenth(0),
 		scrollBarWidth(30), scrollBarHeight(30),
-		pasteBuffer(0), pasteBufferIsRectangular(false),
 		capturedMouse(false), dragWasDropped(false),
-		primarySelectionCopy(0), parentClass(0),
-		ic(NULL), ic_attr(NULL) {
+		parentClass(0),
+		ic(NULL), ic_attr(NULL), lastWheelMouseDirection(0),
+		wheelMouseIntensity(0) {
 	sci = sci_;
 	wMain = GTK_WIDGET(sci);
 
+#if PLAT_GTK_WIN32       
+ 	// There does not seem to be a real standard for indicating that the clipboard
+	// contains a rectangular selection, so copy Developer Studio.
+	cfColumnSelect = static_cast<CLIPFORMAT>(
+		::RegisterClipboardFormat("MSDEVColumnSelect"));
+
+  	// Get intellimouse parameters when running on win32; otherwise use
+	// reasonable default     
+#ifndef SPI_GETWHEELSCROLLLINES
+#define SPI_GETWHEELSCROLLLINES   104
+#endif
+	::SystemParametersInfo(SPI_GETWHEELSCROLLLINES, 0, &linesPerScroll, 0);
+#else
+	linesPerScroll = 4;
+#endif
+	lastWheelMouseTime.tv_sec = 0;
+	lastWheelMouseTime.tv_usec = 0;
+
 	Initialise();
 }
 
 ScintillaGTK::~ScintillaGTK() {
-	delete []primarySelectionCopy;
 }
 
 void ScintillaGTK::RealizeThis(GtkWidget *widget) {
@@ -277,8 +325,8 @@ void ScintillaGTK::RealizeThis(GtkWidget *widget) {
 				gdk_im_begin(ic, widget->window);
 		}
 	}
-	gtk_widget_realize(scrollbarv.GetID());
-	gtk_widget_realize(scrollbarh.GetID());
+	gtk_widget_realize(PWidget(scrollbarv));
+	gtk_widget_realize(PWidget(scrollbarh));
 }
 
 void ScintillaGTK::Realize(GtkWidget *widget) {
@@ -291,8 +339,8 @@ void ScintillaGTK::UnRealizeThis(GtkWidget *widget) {
 		gtk_widget_unmap(widget);
 	}
 	GTK_WIDGET_UNSET_FLAGS(widget, GTK_REALIZED);
-	gtk_widget_unrealize(scrollbarv.GetID());
-	gtk_widget_unrealize(scrollbarh.GetID());
+	gtk_widget_unrealize(PWidget(scrollbarv));
+	gtk_widget_unrealize(PWidget(scrollbarh));
 	if (ic) {
 		gdk_ic_destroy(ic);
 		ic = NULL;
@@ -324,12 +372,12 @@ static void MapWidget(GtkWidget *widget) {
 
 void ScintillaGTK::MapThis() {
 	//Platform::DebugPrintf("ScintillaGTK::map this\n");
-	GTK_WIDGET_SET_FLAGS(wMain.GetID(), GTK_MAPPED);
-	MapWidget(scrollbarh.GetID());
-	MapWidget(scrollbarv.GetID());
+	GTK_WIDGET_SET_FLAGS(PWidget(wMain), GTK_MAPPED);
+	MapWidget(PWidget(scrollbarh));
+	MapWidget(PWidget(scrollbarv));
 	scrollbarv.SetCursor(Window::cursorReverseArrow);
 	scrollbarh.SetCursor(Window::cursorReverseArrow);
-	gdk_window_show(wMain.GetID()->window);
+	gdk_window_show(PWidget(wMain)->window);
 }
 
 void ScintillaGTK::Map(GtkWidget *widget) {
@@ -339,10 +387,10 @@ void ScintillaGTK::Map(GtkWidget *widget) {
 
 void ScintillaGTK::UnMapThis() {
 	//Platform::DebugPrintf("ScintillaGTK::unmap this\n");
-	GTK_WIDGET_UNSET_FLAGS(wMain.GetID(), GTK_MAPPED);
-	gdk_window_hide(wMain.GetID()->window);
-	gtk_widget_unmap(scrollbarh.GetID());
-	gtk_widget_unmap(scrollbarv.GetID());
+	GTK_WIDGET_UNSET_FLAGS(PWidget(wMain), GTK_MAPPED);
+	gdk_window_hide(PWidget(wMain)->window);
+	gtk_widget_unmap(PWidget(scrollbarh));
+	gtk_widget_unmap(PWidget(scrollbarv));
 }
 
 void ScintillaGTK::UnMap(GtkWidget *widget) {
@@ -388,8 +436,8 @@ void ScintillaGTK::SizeRequest(GtkWidget *widget, GtkRequisition *requisition) {
 	requisition->height = 2000;
 	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
 	GtkRequisition child_requisition;
-	gtk_widget_size_request(sciThis->scrollbarh.GetID(), &child_requisition);
-	gtk_widget_size_request(sciThis->scrollbarv.GetID(), &child_requisition);
+	gtk_widget_size_request(PWidget(sciThis->scrollbarh), &child_requisition);
+	gtk_widget_size_request(PWidget(sciThis->scrollbarv), &child_requisition);
 }
 
 void ScintillaGTK::SizeAllocate(GtkWidget *widget, GtkAllocation *allocation) {
@@ -422,12 +470,9 @@ void ScintillaGTK::Initialise() {
 	parentClass = reinterpret_cast<GtkWidgetClass *>(
 	                  gtk_type_class(gtk_container_get_type()));
 
-	pasteBuffer = 0;
-	pasteBufferIsRectangular = false;
-
-	GTK_WIDGET_SET_FLAGS(wMain.GetID(), GTK_CAN_FOCUS);
-	GTK_WIDGET_SET_FLAGS(GTK_WIDGET(wMain.GetID()), GTK_SENSITIVE);
-	gtk_widget_set_events(wMain.GetID(),
+	GTK_WIDGET_SET_FLAGS(PWidget(wMain), GTK_CAN_FOCUS);
+	GTK_WIDGET_SET_FLAGS(GTK_WIDGET(PWidget(wMain)), GTK_SENSITIVE);
+	gtk_widget_set_events(PWidget(wMain),
 	                      GDK_EXPOSURE_MASK
 	                      | GDK_STRUCTURE_MASK
 	                      | GDK_KEY_PRESS_MASK
@@ -441,21 +486,21 @@ void ScintillaGTK::Initialise() {
 
 	adjustmentv = gtk_adjustment_new(0.0, 0.0, 201.0, 1.0, 20.0, 20.0);
 	scrollbarv = gtk_vscrollbar_new(GTK_ADJUSTMENT(adjustmentv));
-	GTK_WIDGET_UNSET_FLAGS(scrollbarv.GetID(), GTK_CAN_FOCUS);
+	GTK_WIDGET_UNSET_FLAGS(PWidget(scrollbarv), GTK_CAN_FOCUS);
 	gtk_signal_connect(GTK_OBJECT(adjustmentv), "value_changed",
 	                   GTK_SIGNAL_FUNC(ScrollSignal), this);
-	gtk_widget_set_parent(scrollbarv.GetID(), wMain.GetID());
-	gtk_widget_show(scrollbarv.GetID());
+	gtk_widget_set_parent(PWidget(scrollbarv), PWidget(wMain));
+	gtk_widget_show(PWidget(scrollbarv));
 
 	adjustmenth = gtk_adjustment_new(0.0, 0.0, 101.0, 1.0, 20.0, 20.0);
 	scrollbarh = gtk_hscrollbar_new(GTK_ADJUSTMENT(adjustmenth));
-	GTK_WIDGET_UNSET_FLAGS(scrollbarh.GetID(), GTK_CAN_FOCUS);
+	GTK_WIDGET_UNSET_FLAGS(PWidget(scrollbarh), GTK_CAN_FOCUS);
 	gtk_signal_connect(GTK_OBJECT(adjustmenth), "value_changed",
 	                   GTK_SIGNAL_FUNC(ScrollHSignal), this);
-	gtk_widget_set_parent(scrollbarh.GetID(), wMain.GetID());
-	gtk_widget_show(scrollbarh.GetID());
+	gtk_widget_set_parent(PWidget(scrollbarh), PWidget(wMain));
+	gtk_widget_show(PWidget(scrollbarh));
 
-	gtk_widget_grab_focus(wMain.GetID());
+	gtk_widget_grab_focus(PWidget(wMain));
 
 	static const GtkTargetEntry targets[] = {
 	    { "STRING", 0, TARGET_STRING },
@@ -465,16 +510,16 @@ void ScintillaGTK::Initialise() {
 	};
 	static const gint n_targets = sizeof(targets) / sizeof(targets[0]);
 
-	gtk_selection_add_targets(GTK_WIDGET(wMain.GetID()), GDK_SELECTION_PRIMARY,
+	gtk_selection_add_targets(GTK_WIDGET(PWidget(wMain)), GDK_SELECTION_PRIMARY,
 	                          targets, n_targets);
 
 	if (!clipboard_atom)
 		clipboard_atom = gdk_atom_intern("CLIPBOARD", FALSE);
 
-	gtk_selection_add_targets(GTK_WIDGET(wMain.GetID()), clipboard_atom,
+	gtk_selection_add_targets(GTK_WIDGET(PWidget(wMain)), clipboard_atom,
 	                          targets, n_targets);
 
-	gtk_drag_dest_set(GTK_WIDGET(wMain.GetID()),
+	gtk_drag_dest_set(GTK_WIDGET(PWidget(wMain)),
 	                  GTK_DEST_DEFAULT_ALL, targets, n_targets,
 	                  static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE));
 
@@ -495,7 +540,7 @@ void ScintillaGTK::StartDrag() {
 	};
 	static const gint n_targets = sizeof(targets) / sizeof(targets[0]);
 	GtkTargetList *tl = gtk_target_list_new(targets, n_targets);
-	gtk_drag_begin(GTK_WIDGET(wMain.GetID()),
+	gtk_drag_begin(GTK_WIDGET(PWidget(wMain)),
 	               tl,
 	               static_cast<GdkDragAction>(GDK_ACTION_COPY | GDK_ACTION_MOVE),
 	               evbtn.button,
@@ -506,7 +551,7 @@ sptr_t ScintillaGTK::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 	switch (iMessage) {
 
 	case SCI_GRABFOCUS:
-		gtk_widget_grab_focus(wMain.GetID());
+		gtk_widget_grab_focus(PWidget(wMain));
 		break;
 
 	case SCI_GETDIRECTFUNCTION:
@@ -540,9 +585,9 @@ void ScintillaGTK::SetTicking(bool on) {
 void ScintillaGTK::SetMouseCapture(bool on) {
 	if (mouseDownCaptures) {
 		if (on) {
-			gtk_grab_add(GTK_WIDGET(wMain.GetID()));
+			gtk_grab_add(GTK_WIDGET(PWidget(wMain)));
 		} else {
-			gtk_grab_remove(GTK_WIDGET(wMain.GetID()));
+			gtk_grab_remove(GTK_WIDGET(PWidget(wMain)));
 		}
 	}
 	capturedMouse = on;
@@ -559,10 +604,13 @@ void ScintillaGTK::FullPaint() {
 	//Platform::DebugPrintf("ScintillaGTK::FullPaint %0d,%0d %0d,%0d\n",
 	//	rcPaint.left, rcPaint.top, rcPaint.right, rcPaint.bottom);
 	paintingAllText = true;
-	Surface sw;
-	sw.Init((wMain.GetID())->window);
-	Paint(&sw, rcPaint);
-	sw.Release();
+	Surface *sw = Surface::Allocate();
+	if (sw) {
+		sw->Init((PWidget(wMain))->window);
+		Paint(sw, rcPaint);
+		sw->Release();
+		delete sw;
+	}
 	paintState = notPainting;
 }
 
@@ -587,10 +635,13 @@ void ScintillaGTK::SyncPaint(PRectangle rc) {
 	paintingAllText = rcPaint.Contains(rcText);
 	//Platform::DebugPrintf("ScintillaGTK::SyncPaint %0d,%0d %0d,%0d\n",
 	//	rcPaint.left, rcPaint.top, rcPaint.right, rcPaint.bottom);
-	Surface sw;
-	sw.Init((wMain.GetID())->window);
-	Paint(&sw, rc);
-	sw.Release();
+	Surface *sw = Surface::Allocate();
+	if (sw) {
+		sw->Init((PWidget(wMain))->window);
+		Paint(sw, rc);
+		sw->Release();
+		delete sw;
+	}
 	if (paintState == paintAbandoned) {
 		// Painting area was insufficient to cover new styling or brace highlight positions
 		FullPaint();
@@ -603,7 +654,7 @@ void ScintillaGTK::ScrollText(int linesToMove) {
 	int diff = vs.lineHeight * -linesToMove;
 	//Platform::DebugPrintf("ScintillaGTK::ScrollText %d %d %0d,%0d %0d,%0d\n", linesToMove, diff,
 	//	rc.left, rc.top, rc.right, rc.bottom);
-	WindowID wi = wMain.GetID();
+	GtkWidget *wi = PWidget(wMain);
 	GdkGC *gc = gdk_gc_new(wi->window);
 	GdkEvent* event;
 
@@ -683,16 +734,16 @@ void ScintillaGTK::ReconfigureScrollBars() {
 
 void ScintillaGTK::NotifyChange() {
 	gtk_signal_emit(GTK_OBJECT(sci), scintilla_signals[COMMAND_SIGNAL],
-	                Platform::LongFromTwoShorts(ctrlID, SCEN_CHANGE), wMain.GetID());
+	                Platform::LongFromTwoShorts(ctrlID, SCEN_CHANGE), PWidget(wMain));
 }
 
 void ScintillaGTK::NotifyFocus(bool focus) {
 	gtk_signal_emit(GTK_OBJECT(sci), scintilla_signals[COMMAND_SIGNAL],
-	                Platform::LongFromTwoShorts(ctrlID, focus ? SCEN_SETFOCUS : SCEN_KILLFOCUS), wMain.GetID());
+	                Platform::LongFromTwoShorts(ctrlID, focus ? SCEN_SETFOCUS : SCEN_KILLFOCUS), PWidget(wMain));
 }
 
 void ScintillaGTK::NotifyParent(SCNotification scn) {
-	scn.nmhdr.hwndFrom = wMain.GetID();
+	scn.nmhdr.hwndFrom = PWidget(wMain);
 	scn.nmhdr.idFrom = ctrlID;
 	gtk_signal_emit(GTK_OBJECT(sci), scintilla_signals[NOTIFY_SIGNAL],
 	                ctrlID, &scn);
@@ -729,17 +780,22 @@ int ScintillaGTK::KeyDefault(int key, int modifiers) {
 
 void ScintillaGTK::Copy() {
 	if (currentPos != anchor) {
-		delete []pasteBuffer;
-		pasteBuffer = CopySelectionRange();
-		pasteBufferIsRectangular = selType == selRectangle;
-		gtk_selection_owner_set(GTK_WIDGET(wMain.GetID()),
+		CopySelectionRange(&copyText);
+		gtk_selection_owner_set(GTK_WIDGET(PWidget(wMain)),
 		                        clipboard_atom,
 		                        GDK_CURRENT_TIME);
+#if PLAT_GTK_WIN32              
+		if (selType == selRectangle) {
+			::OpenClipboard(NULL);
+			::SetClipboardData(cfColumnSelect, 0);
+			::CloseClipboard();
+		}
+#endif              
 	}
 }
 
 void ScintillaGTK::Paste() {
-	gtk_selection_convert(GTK_WIDGET(wMain.GetID()),
+	gtk_selection_convert(GTK_WIDGET(PWidget(wMain)),
 	                      clipboard_atom,
 	                      gdk_atom_intern("STRING", FALSE), GDK_CURRENT_TIME);
 }
@@ -747,11 +803,11 @@ void ScintillaGTK::Paste() {
 void ScintillaGTK::CreateCallTipWindow(PRectangle rc) {
 	ct.wCallTip = gtk_window_new(GTK_WINDOW_POPUP);
 	ct.wDraw = gtk_drawing_area_new();
-	gtk_container_add(GTK_CONTAINER(ct.wCallTip.GetID()), ct.wDraw.GetID());
-	gtk_signal_connect(GTK_OBJECT(ct.wDraw.GetID()), "expose_event",
+	gtk_container_add(GTK_CONTAINER(PWidget(ct.wCallTip)), PWidget(ct.wDraw));
+	gtk_signal_connect(GTK_OBJECT(PWidget(ct.wDraw)), "expose_event",
 	                   GtkSignalFunc(ScintillaGTK::ExposeCT), &ct);
-	gtk_widget_set_events(ct.wDraw.GetID(), GDK_EXPOSURE_MASK);
-	gtk_drawing_area_size(GTK_DRAWING_AREA(ct.wDraw.GetID()),
+	gtk_widget_set_events(PWidget(ct.wDraw), GDK_EXPOSURE_MASK);
+	gtk_drawing_area_size(GTK_DRAWING_AREA(PWidget(ct.wDraw)),
 	                      rc.Width(), rc.Height());
 	ct.wDraw.Show();
 }
@@ -769,7 +825,7 @@ void ScintillaGTK::AddToPopUp(const char *label, int cmd, bool enabled) {
 	                             &itemEntry, this, 1);
 	if (cmd) {
 		GtkWidget *item = gtk_item_factory_get_widget_by_action(
-		                      popup.GetID(), cmd);
+		                      reinterpret_cast<GtkItemFactory *>(popup.GetID()), cmd);
 		if (item)
 			gtk_widget_set_sensitive(item, enabled);
 	}
@@ -777,7 +833,7 @@ void ScintillaGTK::AddToPopUp(const char *label, int cmd, bool enabled) {
 
 bool ScintillaGTK::OwnPrimarySelection() {
 	return (gdk_selection_owner_get(GDK_SELECTION_PRIMARY)
-	        == GTK_WIDGET(wMain.GetID())->window);
+	        == GTK_WIDGET(PWidget(wMain))->window);
 }
 
 void ScintillaGTK::ClaimSelection() {
@@ -785,18 +841,16 @@ void ScintillaGTK::ClaimSelection() {
 	// Whenever the user selects some text, we become the primary selection
 	if (currentPos != anchor) {
 		primarySelection = true;
-		gtk_selection_owner_set(GTK_WIDGET(wMain.GetID()),
+		gtk_selection_owner_set(GTK_WIDGET(PWidget(wMain)),
 		                        GDK_SELECTION_PRIMARY, GDK_CURRENT_TIME);
-		delete []primarySelectionCopy;
-		primarySelectionCopy = NULL;
+		primary.Set(0, 0);
 	} else if (OwnPrimarySelection()) {
-		if (primarySelectionCopy == NULL)
-			gtk_selection_owner_set(NULL, GDK_SELECTION_PRIMARY, GDK_CURRENT_TIME);
 		primarySelection = true;
+		if (primary.s == NULL)
+			gtk_selection_owner_set(NULL, GDK_SELECTION_PRIMARY, GDK_CURRENT_TIME);
 	} else {
-		delete []primarySelectionCopy;
-		primarySelectionCopy = NULL;
 		primarySelection = false;
+		primary.Set(0, 0);
 	}
 }
 
@@ -808,18 +862,47 @@ void ScintillaGTK::ReceivedSelection(GtkSelectionData *selection_data) {
 		        (selection_data->length > 0)) {
 			char *ptr = reinterpret_cast<char *>(selection_data->data);
 			unsigned int len = selection_data->length;
-			for (unsigned int i = 0; i < static_cast<unsigned int>(selection_data->length); i++) {
+			unsigned int i;
+			for (i = 0; i < static_cast<unsigned int>(selection_data->length); i++) {
 				if ((len == static_cast<unsigned int>(selection_data->length)) && (0 == ptr[i]))
 					len = i;
+			}                     
+#if PLAT_GTK_WIN32
+			/* Need to convert \n to correct newline form for this file as win32gtk always returns */
+			/* only \n line delimiter from clipboard */
+			char *tmpstr = new char [2 * len];
+			char *dptr = tmpstr;
+			char *sptr = ptr;
+			unsigned int newlen = len;                     
+			for (i = 0; i < len; i++) {
+				if (*sptr == '\n') {
+					if (pdoc->eolMode != SC_EOL_LF)
+						*dptr++ = '\r';
+					if (pdoc->eolMode != SC_EOL_CR)
+						*dptr++ = '\n';
+					if (pdoc->eolMode == SC_EOL_CRLF)
+						newlen++;
+					sptr++;
+				}
+				else {
+					*dptr++ = *sptr++;
+				}
 			}
+			ptr = tmpstr;
+			len = newlen;
+#endif
 			pdoc->BeginUndoAction();
 			int selStart = SelectionStart();
 			if (selection_data->selection != GDK_SELECTION_PRIMARY) {
 				ClearSelection();
 			}
 			// Check for "\n\0" ending to string indicating that selection is rectangular
+#if PLAT_GTK_WIN32
+			bool isRectangular = ::IsClipboardFormatAvailable(cfColumnSelect);
+#else
 			bool isRectangular = ((selection_data->length > 1) &&
 			                      (ptr[selection_data->length - 1] == 0 && ptr[selection_data->length - 2] == '\n'));
+#endif
 			if (isRectangular) {
 				PasteRectangular(selStart, ptr, len);
 			} else {
@@ -827,6 +910,9 @@ void ScintillaGTK::ReceivedSelection(GtkSelectionData *selection_data) {
 				SetEmptySelection(currentPos + len);
 			}
 			pdoc->EndUndoAction();
+#if PLAT_GTK_WIN32
+			delete []tmpstr;
+#endif
 		}
 	}
 	Redraw();
@@ -838,8 +924,12 @@ void ScintillaGTK::ReceivedDrop(GtkSelectionData *selection_data) {
 		if (selection_data->length > 0) {
 			char *ptr = reinterpret_cast<char *>(selection_data->data);
 			// 3rd argument is false because the deletion of the moved data is handle by GetSelection
+#if PLAT_GTK_WIN32
+			bool isRectangular = ::IsClipboardFormatAvailable(cfColumnSelect);
+#else
 			bool isRectangular = ((selection_data->length > 1) &&
 			                      (ptr[selection_data->length - 1] == 0 && ptr[selection_data->length - 2] == '\n'));
+#endif
 			DropAt(posDrop, ptr, false, isRectangular);
 		}
 	} else {
@@ -849,18 +939,38 @@ void ScintillaGTK::ReceivedDrop(GtkSelectionData *selection_data) {
 	Redraw();
 }
 
-void ScintillaGTK::GetSelection(GtkSelectionData *selection_data, guint info, char *text, bool isRectangular) {
-	char *selBuffer = text;
-	char *tmpBuffer = NULL; // Buffer to be freed
-
+void ScintillaGTK::GetSelection(GtkSelectionData *selection_data, guint info, SelectionText *text) {
 	if (selection_data->selection == GDK_SELECTION_PRIMARY) {
-		if (primarySelectionCopy != NULL) {
-			selBuffer = primarySelectionCopy;
-		} else {
-			tmpBuffer = CopySelectionRange();
-			selBuffer = tmpBuffer;
+		if (primary.s == NULL) {
+			CopySelectionRange(&primary);
+		}
+		text = &primary;
+	}
+
+	char *selBuffer = text->s;
+
+#if PLAT_GTK_WIN32
+	// win32gtk requires \n delimited lines and doesn't work right with
+	// other line formats, so make a copy of the clip text now with 
+	// newlines converted
+	char *tmpstr = new char[text->len + 1];
+	char *sptr = selBuffer;
+	char *dptr = tmpstr;
+	while (*sptr != '\0') {
+		if (pdoc->eolMode == SC_EOL_CR && *sptr == '\r') {
+			*dptr++ = '\n';
+			sptr++;
+		}
+		else if (pdoc->eolMode != SC_EOL_CR && *sptr == '\r') {
+			sptr++;
+		}
+		else {
+			*dptr++ = *sptr++;
 		}
 	}
+	*dptr = '\0';       
+	selBuffer = tmpstr;
+#endif
 
 	if (info == TARGET_STRING) {
 		int len = strlen(selBuffer);
@@ -869,8 +979,12 @@ void ScintillaGTK::GetSelection(GtkSelectionData *selection_data, guint info, ch
 		// and need some way to mark the clipping as being stream or rectangular,
 		// the terminating \0 is included in the length for rectangular clippings.
 		// All other tested aplications behave benignly by ignoring the \0.
-		if (isRectangular)
+		// The #if is here because on Windows cfColumnSelect clip entry is used 
+                // instead as standard indicator of rectangularness (so no need to kludge)
+#if PLAT_GTK_WIN32 == 0
+		if (text->rectangular)
 			len++;
+#endif
 		gtk_selection_data_set(selection_data, GDK_SELECTION_TYPE_STRING,
 		                       8, reinterpret_cast<unsigned char *>(selBuffer),
 		                       len);
@@ -886,7 +1000,9 @@ void ScintillaGTK::GetSelection(GtkSelectionData *selection_data, guint info, ch
 		gdk_free_compound_text(text);
 	}
 
-	delete []tmpBuffer;
+#if PLAT_GTK_WIN32
+	delete []tmpstr;
+#endif
 }
 
 void ScintillaGTK::UnclaimSelection(GdkEventSelection *selection_event) {
@@ -894,8 +1010,7 @@ void ScintillaGTK::UnclaimSelection(GdkEventSelection *selection_event) {
 	if (selection_event->selection == GDK_SELECTION_PRIMARY) {
 		//Platform::DebugPrintf("UnclaimPrimarySelection\n");
 		if (!OwnPrimarySelection()) {
-			delete []primarySelectionCopy;
-			primarySelectionCopy = NULL;
+			primary.Set(0, 0);
 			primarySelection = false;
 			FullPaint();
 		}
@@ -908,8 +1023,8 @@ void ScintillaGTK::Resize(int width, int height) {
 	DropGraphics();
 
 	// Not always needed, but some themes can have different sizes of scrollbars
-	scrollBarWidth = GTK_WIDGET(scrollbarv.GetID())->requisition.width;
-	scrollBarHeight = GTK_WIDGET(scrollbarh.GetID())->requisition.height;
+	scrollBarWidth = GTK_WIDGET(PWidget(scrollbarv))->requisition.width;
+	scrollBarHeight = GTK_WIDGET(PWidget(scrollbarh))->requisition.height;
 
 	// These allocations should never produce negative sizes as they would wrap around to huge
 	// unsigned numbers inside GTK+ causing warnings.
@@ -928,31 +1043,28 @@ void ScintillaGTK::Resize(int width, int height) {
 		alloc.width = 0;
 		alloc.height = 0;
 	}
-	gtk_widget_size_allocate(GTK_WIDGET(scrollbarh.GetID()), &alloc);
+	gtk_widget_size_allocate(GTK_WIDGET(PWidget(scrollbarh)), &alloc);
 
 	alloc.x = width - scrollBarWidth;
 	alloc.y = 0;
 	alloc.width = scrollBarWidth;
 	alloc.height = Platform::Maximum(1, height - scrollBarHeight) + 1;
-	gtk_widget_size_allocate(GTK_WIDGET(scrollbarv.GetID()), &alloc);
+	gtk_widget_size_allocate(GTK_WIDGET(PWidget(scrollbarv)), &alloc);
 
 	SetScrollBars();
 }
 
-gint ScintillaGTK::Press(GtkWidget *widget, GdkEventButton *event) {
-	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
-	//Platform::DebugPrintf("Press %x time=%d state = %x button = %x\n",sciThis,event->time, event->state, event->button);
+gint ScintillaGTK::PressThis(GdkEventButton *event) {
+	//Platform::DebugPrintf("Press %x time=%d state = %x button = %x\n",this,event->time, event->state, event->button);
 	// Do not use GTK+ double click events as Scintilla has its own double click detection
 	if (event->type != GDK_BUTTON_PRESS)
 		return FALSE;
 
-	sciThis->evbtn = *event;
+	evbtn = *event;
 	Point pt;
 	pt.x = int(event->x);
 	pt.y = int(event->y);
-	if (event->window != widget->window)
-		return FALSE;
-	PRectangle rcClient = sciThis->GetClientRectangle();
+	PRectangle rcClient = GetClientRectangle();
 	//Platform::DebugPrintf("Press %0d,%0d in %0d,%0d %0d,%0d\n",
 	//	pt.x, pt.y, rcClient.left, rcClient.top, rcClient.right, rcClient.bottom);
 	if ((pt.x > rcClient.right) || (pt.y > rcClient.bottom)) {
@@ -962,54 +1074,61 @@ gint ScintillaGTK::Press(GtkWidget *widget, GdkEventButton *event) {
 
 	bool ctrl = event->state & GDK_CONTROL_MASK;
 
-	gtk_widget_grab_focus(sciThis->wMain.GetID());
+	gtk_widget_grab_focus(PWidget(wMain));
 	if (event->button == 1) {
-		//sciThis->ButtonDown(pt, event->time,
+		//ButtonDown(pt, event->time,
 		//	event->state & GDK_SHIFT_MASK,
 		//	event->state & GDK_CONTROL_MASK,
 		//	event->state & GDK_MOD1_MASK);
 		// Instead of sending literal modifiers use control instead of alt
 		// This is because all the window managers seem to grab alt + click for moving
-		sciThis->ButtonDown(pt, event->time,
+		ButtonDown(pt, event->time,
 		                    event->state & GDK_SHIFT_MASK,
 		                    event->state & GDK_CONTROL_MASK,
 		                    event->state & GDK_CONTROL_MASK);
 	} else if (event->button == 2) {
 		// Grab the primary selection if it exists
-		Position pos = sciThis->PositionFromLocation(pt);
-		if (sciThis->OwnPrimarySelection() && sciThis->primarySelectionCopy == NULL)
-			sciThis->primarySelectionCopy = sciThis->CopySelectionRange();
+		Position pos = PositionFromLocation(pt);
+		if (OwnPrimarySelection() && primary.s == NULL)
+			CopySelectionRange(&primary);
 
-		sciThis->SetSelection(pos, pos);
-		gtk_selection_convert(GTK_WIDGET(sciThis->wMain.GetID()), GDK_SELECTION_PRIMARY,
+		SetSelection(pos, pos);
+		gtk_selection_convert(GTK_WIDGET(PWidget(wMain)), GDK_SELECTION_PRIMARY,
 		                      gdk_atom_intern("STRING", FALSE), event->time);
-	} else if (event->button == 3 && sciThis->displayPopupMenu) {
+	} else if (event->button == 3 && displayPopupMenu) {
 		// PopUp menu
 		// Convert to screen
 		int ox = 0;
 		int oy = 0;
-		gdk_window_get_origin(sciThis->wMain.GetID()->window, &ox, &oy);
-		sciThis->ContextMenu(Point(pt.x + ox, pt.y + oy));
+		gdk_window_get_origin(PWidget(wMain)->window, &ox, &oy);
+		ContextMenu(Point(pt.x + ox, pt.y + oy));
 	} else if (event->button == 4) {
-		// Wheel scrolling up
+		// Wheel scrolling up (only xwin gtk does it this way)
 		if (ctrl)
-			gtk_adjustment_set_value(GTK_ADJUSTMENT(sciThis->adjustmenth), (
-			                             (sciThis->xOffset) / 2 ) - 6);
+			gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustmenth), (
+			                             (xOffset) / 2 ) - 6);
 		else
-			gtk_adjustment_set_value(GTK_ADJUSTMENT(sciThis->adjustmentv),
-			                         sciThis->topLine - 3);
+			gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustmentv),
+			                         topLine - 3);
 	} else if ( event->button == 5 ) {
-		// Wheel scrolling down
+		// Wheel scrolling down (only xwin gtk does it this way)
 		if (ctrl)
-			gtk_adjustment_set_value(GTK_ADJUSTMENT(sciThis->adjustmenth), (
-			                             (sciThis->xOffset) / 2 ) + 6);
+			gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustmenth), (
+			                             (xOffset) / 2 ) + 6);
 		else
-			gtk_adjustment_set_value(GTK_ADJUSTMENT(sciThis->adjustmentv),
-			                         sciThis->topLine + 3);
+			gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustmentv),
+			                         topLine + 3);
 	}
 	return FALSE;
 }
 
+gint ScintillaGTK::Press(GtkWidget *widget, GdkEventButton *event) {
+	if (event->window != widget->window)
+		return FALSE;
+	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
+	return sciThis->PressThis(event);
+}
+
 gint ScintillaGTK::MouseRelease(GtkWidget *widget, GdkEventButton *event) {
 	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
 	//Platform::DebugPrintf("Release %x %d %d\n",sciThis,event->time,event->state);
@@ -1021,7 +1140,7 @@ gint ScintillaGTK::MouseRelease(GtkWidget *widget, GdkEventButton *event) {
 		pt.y = int(event->y);
 		//Platform::DebugPrintf("Up %x %x %d %d %d\n",
 		//	sciThis,event->window,event->time, pt.x, pt.y);
-		if (event->window != sciThis->wMain.GetID()->window)
+		if (event->window != PWidget(sciThis->wMain)->window)
 			// If mouse released on scroll bar then the position is relative to the
 			// scrollbar, not the drawing window so just repeat the most recent point.
 			pt = sciThis->ptMouseLast;
@@ -1030,6 +1149,71 @@ gint ScintillaGTK::MouseRelease(GtkWidget *widget, GdkEventButton *event) {
 	return FALSE;
 }
 
+// win32gtk has a special wheel mouse event for whatever reason and doesn't
+// use the button4/5 trick used under X windows.
+#if PLAT_GTK_WIN32
+gint ScintillaGTK::ScrollEvent(GtkWidget *widget,
+                               GdkEventScroll *event) {
+	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
+
+	if (widget == NULL || event == NULL)
+		return FALSE;
+
+	// Compute amount and direction to scroll (even tho on win32 there is
+	// intensity of scrolling info in the native message, gtk doesn't
+	// support this so we simulate similarly adaptive scrolling)
+	int cLineScroll;
+	int timeDelta = 1000000;
+	GTimeVal curTime;
+	g_get_current_time(&curTime);
+	if (curTime.tv_sec == sciThis->lastWheelMouseTime.tv_sec)
+		timeDelta = curTime.tv_usec - sciThis->lastWheelMouseTime.tv_usec;
+	else if (curTime.tv_sec == sciThis->lastWheelMouseTime.tv_sec + 1) 
+		timeDelta = 1000000 + (curTime.tv_usec - sciThis->lastWheelMouseTime.tv_usec);
+	if ((event->direction == sciThis->lastWheelMouseDirection) && (timeDelta < 250000)) {
+		if (sciThis->wheelMouseIntensity < 12)
+			sciThis->wheelMouseIntensity++;
+		cLineScroll = sciThis->wheelMouseIntensity;
+	} else {
+		cLineScroll = sciThis->linesPerScroll;
+		if (cLineScroll == 0)
+			cLineScroll = 4;
+		sciThis->wheelMouseIntensity = cLineScroll;
+	}
+	if (event->direction == GDK_SCROLL_UP) {
+		cLineScroll *= -1;
+	}
+	g_get_current_time(&sciThis->lastWheelMouseTime);
+	sciThis->lastWheelMouseDirection = event->direction;
+
+	// Note:  Unpatched versions of win32gtk don't set the 'state' value so 
+	// only regular scrolling is supported there.  Also, unpatched win32gtk
+	// issues spurious button 2 mouse events during wheeling, which can cause
+	// problems (a patch for both was submitted by archaeopteryx.com on 13Jun2001)
+
+	// Data zoom not supported
+	if (event->state & GDK_SHIFT_MASK) {
+		return FALSE;
+	}
+
+	// Text font size zoom
+	if (event->state & GDK_CONTROL_MASK) {
+		if (cLineScroll < 0) {
+			sciThis->KeyCommand(SCI_ZOOMIN);
+			return TRUE;
+		} else {
+			sciThis->KeyCommand(SCI_ZOOMOUT);
+			return TRUE;
+		}
+
+	// Regular scrolling
+	} else {
+		sciThis->ScrollTo(sciThis->topLine + cLineScroll);
+		return TRUE;
+	}
+}
+#endif
+
 gint ScintillaGTK::Motion(GtkWidget *widget, GdkEventMotion *event) {
 	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
 	//Platform::DebugPrintf("Motion %x %d\n",sciThis,event->time);
@@ -1047,12 +1231,8 @@ gint ScintillaGTK::Motion(GtkWidget *widget, GdkEventMotion *event) {
 	}
 	//Platform::DebugPrintf("Move %x %x %d %c %d %d\n",
 	//	sciThis,event->window,event->time,event->is_hint? 'h' :'.', x, y);
-	if (state & GDK_BUTTON1_MASK) {
-		Point pt;
-		pt.x = x;
-		pt.y = y;
-		sciThis->ButtonMove(pt);
-	}
+	Point pt(x, y);
+	sciThis->ButtonMove(pt);
 	return FALSE;
 }
 
@@ -1125,7 +1305,8 @@ static int KeyTranslate(int keyIn) {
 
 gint ScintillaGTK::KeyPress(GtkWidget *widget, GdkEventKey *event) {
 	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
-	//Platform::DebugPrintf("SC-key: %d %x\n",event->keyval, event->state);
+	//Platform::DebugPrintf("SC-key: %d %x [%s]\n", 
+	//	event->keyval, event->state, (event->length > 0) ? event->string : "empty");
 	bool shift = event->state & GDK_SHIFT_MASK;
 	bool ctrl = event->state & GDK_CONTROL_MASK;
 	bool alt = event->state & GDK_MOD1_MASK;
@@ -1134,6 +1315,10 @@ gint ScintillaGTK::KeyPress(GtkWidget *widget, GdkEventKey *event) {
 		key = toupper(key);
 	else if (!ctrl && (key >= GDK_KP_Multiply && key <= GDK_KP_9))
 		key &= 0x7F;
+	// Hack for keys over 256 and below command keys but makes Hungarian work.
+	// This will have to change for Unicode
+	else if ((key >= 0x100) && (key < 0x1000))
+		key &= 0xff;
 	else
 		key = KeyTranslate(key);
 
@@ -1150,12 +1335,15 @@ gint ScintillaGTK::KeyRelease(GtkWidget *, GdkEventKey * /*event*/) {
 	return FALSE;
 }
 
-gint ScintillaGTK::DestroyWindow(GtkWidget *widget, GdkEventAny *) {
-	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
-	//Platform::DebugPrintf("Destroying window %x %x\n", sciThis, widget);
+void ScintillaGTK::Destroy(GtkObject* object) {
+	ScintillaGTK *sciThis = ScintillaFromWidget(GTK_WIDGET(object));
+	//Platform::DebugPrintf("Destroying %x %x\n", sciThis, object);
 	sciThis->Finalise();
+      
+	if (GTK_OBJECT_CLASS (parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+
 	delete sciThis;
-	return FALSE;
 }
 
 static void DrawChild(GtkWidget *widget, GdkRectangle *area) {
@@ -1172,9 +1360,9 @@ void ScintillaGTK::Draw(GtkWidget *widget, GdkRectangle *area) {
 	//Platform::DebugPrintf("Draw %p %0d,%0d %0d,%0d\n", widget, area->x, area->y, area->width, area->height);
 	PRectangle rcPaint(area->x, area->y, area->x + area->width, area->y + area->height);
 	sciThis->SyncPaint(rcPaint);
-	if (GTK_WIDGET_DRAWABLE(sciThis->wMain.GetID())) {
-		DrawChild(sciThis->scrollbarh.GetID(), area);
-		DrawChild(sciThis->scrollbarv.GetID(), area);
+	if (GTK_WIDGET_DRAWABLE(PWidget(sciThis->wMain))) {
+		DrawChild(PWidget(sciThis->scrollbarh), area);
+		DrawChild(PWidget(sciThis->scrollbarv), area);
 	}
 
 }
@@ -1199,10 +1387,13 @@ gint ScintillaGTK::Expose(GtkWidget *, GdkEventExpose *ose, ScintillaGTK *sciThi
 
 	PRectangle rcText = sciThis->GetTextRectangle();
 	sciThis->paintingAllText = sciThis->rcPaint.Contains(rcText);
-	Surface surfaceWindow;
-	surfaceWindow.Init((sciThis->wMain.GetID())->window);
-	sciThis->Paint(&surfaceWindow, sciThis->rcPaint);
-	surfaceWindow.Release();
+	Surface *surfaceWindow = Surface::Allocate();
+	if (surfaceWindow) {
+		surfaceWindow->Init((PWidget(sciThis->wMain))->window);
+		sciThis->Paint(surfaceWindow, sciThis->rcPaint);
+		surfaceWindow->Release();
+		delete surfaceWindow;
+	}
 	if (sciThis->paintState == paintAbandoned) {
 		// Painting area was insufficient to cover new styling or brace highlight positions
 		sciThis->FullPaint();
@@ -1233,7 +1424,7 @@ void ScintillaGTK::SelectionGet(GtkWidget *widget,
                                 GtkSelectionData *selection_data, guint info, guint) {
 	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
 	//Platform::DebugPrintf("Selection get\n");
-	sciThis->GetSelection(selection_data, info, sciThis->pasteBuffer, sciThis->pasteBufferIsRectangular);
+	sciThis->GetSelection(selection_data, info, &sciThis->copyText);
 }
 
 gint ScintillaGTK::SelectionClear(GtkWidget *widget, GdkEventSelection *selection_event) {
@@ -1299,7 +1490,7 @@ void ScintillaGTK::DragDataGet(GtkWidget *widget, GdkDragContext *context,
 	ScintillaGTK *sciThis = ScintillaFromWidget(widget);
 	sciThis->dragWasDropped = true;
 	if (sciThis->currentPos != sciThis->anchor) {
-		sciThis->GetSelection(selection_data, info, sciThis->dragChars, sciThis->dragIsRectangle);
+		sciThis->GetSelection(selection_data, info, &sciThis->drag);
 	}
 	if (context->action == GDK_ACTION_MOVE) {
 		int selStart = sciThis->SelectionStart();
@@ -1328,11 +1519,13 @@ void ScintillaGTK::PopUpCB(ScintillaGTK *sciThis, guint action, GtkWidget *) {
 }
 
 gint ScintillaGTK::ExposeCT(GtkWidget *widget, GdkEventExpose * /*ose*/, CallTip *ctip) {
-	Surface surfaceWindow;
-	//surfaceWindow.Init((ct->wCallTip.GetID())->window);
-	surfaceWindow.Init(widget->window);
-	ctip->PaintCT(&surfaceWindow);
-	surfaceWindow.Release();
+	Surface *surfaceWindow = Surface::Allocate();
+	if (surfaceWindow) {
+		surfaceWindow->Init(widget->window);
+		ctip->PaintCT(surfaceWindow);
+		surfaceWindow->Release();
+		delete surfaceWindow;
+	}
 	return TRUE;
 }
 
@@ -1370,11 +1563,13 @@ guint scintilla_get_type() {
 	return scintilla_type;
 }
 
-void ScintillaGTK::ClassInit(GtkWidgetClass *widget_class) {
+void ScintillaGTK::ClassInit(GtkObjectClass* object_class, GtkWidgetClass *widget_class) {
 	// Define default signal handlers for the class:  Could move more
 	// of the signal handlers here (those that currently attached to wDraw
 	// in Initialise() may require coordinate translation?)
-	widget_class->destroy_event = DestroyWindow;
+
+	object_class->destroy = Destroy;  
+
 	widget_class->size_request = SizeRequest;
 	widget_class->size_allocate = SizeAllocate;
 	widget_class->expose_event = ExposeMain;
@@ -1383,7 +1578,9 @@ void ScintillaGTK::ClassInit(GtkWidgetClass *widget_class) {
 	widget_class->motion_notify_event = Motion;
 	widget_class->button_press_event = Press;
 	widget_class->button_release_event = MouseRelease;
-
+#if PLAT_GTK_WIN32
+	widget_class->scroll_event = ScrollEvent;
+#endif
 	widget_class->key_press_event = KeyPress;
 	widget_class->key_release_event = KeyRelease;
 	widget_class->focus_in_event = FocusIn;
@@ -1410,6 +1607,8 @@ static void scintilla_class_init(ScintillaClass *klass) {
 	GtkObjectClass *object_class = (GtkObjectClass*) klass;
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) klass;
 
+	parent_class = (GtkWidgetClass*) gtk_type_class (gtk_container_get_type ());
+
 	scintilla_signals[COMMAND_SIGNAL] = gtk_signal_new(
 	                                        "command",
 	                                        GTK_RUN_LAST,
@@ -1434,7 +1633,7 @@ static void scintilla_class_init(ScintillaClass *klass) {
 	klass->command = NULL;
 	klass->notify = NULL;
 
-	ScintillaGTK::ClassInit(widget_class);
+	ScintillaGTK::ClassInit(object_class, widget_class);
 }
 
 static void scintilla_init(ScintillaObject *sci) {
