@@ -22,6 +22,8 @@ public:
 	int characterSet;
 	bool eolFilled;
 	bool underline;
+	enum ecaseForced {caseMixed, caseUpper, caseLower};
+	ecaseForced caseForce;
 	bool visible;
 
 	Font font;
@@ -37,10 +39,12 @@ public:
 	Style(const Style &source);
 	~Style();
 	Style &operator=(const Style &source);
-	void Clear(Colour fore_, Colour back_,
+	void Clear(ColourDesired fore_, ColourDesired back_,
 	           int size_,
 	           const char *fontName_, int characterSet_,
-	           bool bold_, bool italic_, bool eolFilled_, bool underline_, bool visible_);
+	           bool bold_, bool italic_, bool eolFilled_, 
+	           bool underline_, ecaseForced caseForce_, bool visible_);
+	void ClearTo(const Style &source);
 	bool EquivalentFontTo(const Style *other) const;
 	void Realise(Surface &surface, int zoomLevel, Style *defaultStyle = 0);
 };
