@@ -1,5 +1,5 @@
 /*
-    aneditor.c
+    aneditor.cxx
     Copyright (C) 2000  Kh. Naba Kumar Singh
 
     This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 /*
- * Most of the code stollen from SciTE and heavily modified.
+ * Most of the code stolen from SciTE and heavily modified.
  * If code sections are later imported from SciTE, utmost care
  * should be taken to ensure that it does not conflict with the present code.
  */
@@ -36,6 +36,9 @@
 #include <time.h>
 #include <unistd.h>
 
+/* Use Gtk+ */
+#include <gtk/gtk.h>
+
 #define GTK
 #include "Platform.h"
 #include "PropSet.h"
@@ -49,11 +52,19 @@
 #include "properties.h"
 #include "aneditor.h"
 
+#include "tm_tagmanager.h"
+
 #define ANE_MARKER_BOOKMARK 0
 #define MAX_PATH 260
 #define MAXIMUM(x, y)	((x>y)?x:y)
 #define MINIMUM(x,y)	((x<y)?x:y)
 
+#define MAX_AUTOCOMPLETE_WORDS 50 /* Maximum number of autocomplete words to show */
+
+/* Colour has mysteriously vanished from scintilla */
+/* Using the substitute */
+typedef class ColourDesired Colour;
+
 template<int sz>
 class EntryMemory {
 	SString entries[sz];
@@ -108,7 +119,6 @@ protected:
 	int lexLanguage;
 	SString overrideExtension;	// User has chosen to use a particular language
 	enum {numWordLists=5};
-	WordList apis;
 	SString functionDefinition;
 	GtkAccelGroup* accelGroup;
 
@@ -246,7 +256,9 @@ protected:
 	void GotoBlockEnd();
 	void EnsureRangeVisible(int posStart, int posEnd);
 	void SetAccelGroup(GtkAccelGroup* acl);
-
+	int GetBookmarkLine( const int nLineStart );
+	void DefineMarker(int marker, int makerType, Colour fore, Colour back);
+	
 public:
 
 	AnEditor(PropSetFile* p);
@@ -263,7 +275,7 @@ static const char *extList[] = {
     "x.cs", "x.properties", "x.conf"
 };
 
-AnEditor::AnEditor(PropSetFile* p) : apis(true){
+AnEditor::AnEditor(PropSetFile* p) {
 
 	characterSet = 0;
 	language = "java";
@@ -498,7 +510,6 @@ void AnEditor::BraceMatch() {
 			indentSize = props->GetInt("indent.size");
 			if (columnAtCaretNext - indentSize > 1)
 				columnAtCaret = columnAtCaretNext - indentSize;
-			//Platform::DebugPrintf(": %d %d %d\n", lineStart, indentPos, columnAtCaret);
 		}
 
 
@@ -646,11 +657,12 @@ void AnEditor::BookmarkClear() {
 }
 
 bool AnEditor::StartCallTip() {
-	//Platform::DebugPrintf("StartCallTip\n");
 	char linebuf[1000];
 	int current = GetLine(linebuf, sizeof(linebuf));
 	int pos = SendEditor(SCI_GETCURRENTPOS);
 	int braces;
+	TMTagAttrType attrs[] = { tm_tag_attr_name_t, tm_tag_attr_type_t, tm_tag_attr_none_t};
+	
 	do {
 		braces = 0;
 		while (current > 0 && (braces || linebuf[current - 1] != '(')) {
@@ -680,14 +692,17 @@ bool AnEditor::StartCallTip() {
 	linebuf[current] = '\0';
 	int rootlen = current - startword;
 	functionDefinition = "";
-	//Platform::DebugPrintf("word  is [%s] %d %d %d\n", linebuf + startword, rootlen, pos, pos - rootlen);
-	if (apis) {
-		const char *word = apis.GetNearestWord (linebuf + startword, rootlen, callTipIgnoreCase);
-		if (word) {
-			functionDefinition = word;
-			SendEditorString(SCI_CALLTIPSHOW, pos - rootlen + 1, word);
-			ContinueCallTip();
-		}
+	const GPtrArray *tags = tm_workspace_find(linebuf+startword, tm_tag_prototype_t
+	  | tm_tag_function_t | tm_tag_macro_with_arg_t, attrs, FALSE);
+	if (tags && (tags->len > 0))
+	{
+		TMTag *tag = (TMTag *) tags->pdata[0];
+		char *tmp = g_strdup_printf("%s %s%s", NVL(tag->atts.entry.var_type, "")
+		  , tag->name, NVL(tag->atts.entry.arglist, ""));
+		functionDefinition = tmp;
+		SendEditorString(SCI_CALLTIPSHOW, pos - rootlen + 1, tmp);
+		g_free(tmp);
+		ContinueCallTip();
 	}
 	return true;
 }
@@ -740,12 +755,20 @@ bool AnEditor::StartAutoComplete() {
 	linebuf[current] = '\0';
 	const char *root = linebuf + startword;
 	int rootlen = current - startword;
-	if (apis) {
-		char *words = apis.GetNearestWords(root, rootlen, autoCompleteIgnoreCase);
-		if (words) {
-			SendEditorString(SCI_AUTOCSHOW, rootlen, words);
-			free(words);
+	const GPtrArray *tags = tm_workspace_find(root, tm_tag_max_t, NULL, TRUE);
+	if (NULL != tags)
+	{
+		GString *words = g_string_sized_new(100);
+		TMTag *tag;
+		for (guint i = 0; ((i < tags->len) && (i < MAX_AUTOCOMPLETE_WORDS)); ++i)
+		{
+			tag = (TMTag *) tags->pdata[i];
+			if (i > 0)
+				g_string_append_c(words, ' ');
+			g_string_append(words, tag->name);
 		}
+		SendEditorString(SCI_AUTOCSHOW, rootlen, words->str);
+		g_string_free(words, TRUE);
 	}
 	return true;
 }
@@ -799,75 +822,21 @@ bool AnEditor::StartAutoCompleteWord() {
 	linebuf[current] = '\0';
 	const char *root = linebuf + startword;
 	int rootlen = current - startword;
-	int doclen = LengthDocument();
-	TextToFind ft = {{0, 0}, 0, {0, 0}};
-	ft.lpstrText = const_cast<char*>(root);
-	ft.chrg.cpMin = 0;
-	ft.chrgText.cpMin = 0;
-	ft.chrgText.cpMax = 0;
-	int flags = SCFIND_WORDSTART | (autoCompleteIgnoreCase ? 0 : SCFIND_MATCHCASE);
-	//int flags = (autoCompleteIgnoreCase ? 0 : SCFIND_MATCHCASE);
-	int posCurrentWord = SendEditor (SCI_GETCURRENTPOS) - rootlen;
-	//DWORD dwStart = timeGetTime();
-	int length = 0;	// variables for reallocatable array creation
-	int newlength;
-#undef WORDCHUNK
-#define WORDCHUNK 100
-	int size = WORDCHUNK;
-	char *words = (char*) malloc(size);
-	*words = '\0';
-	for (;;) {	// search all the document
-		ft.chrg.cpMax = doclen;
-		int posFind = SendEditor(SCI_FINDTEXT, flags, reinterpret_cast<long>(&ft));
-		if (posFind == -1 || posFind >= doclen)
-			break;
-		if (posFind == posCurrentWord) {
-			ft.chrg.cpMin = posFind + rootlen;
-			continue;
+	const GPtrArray *tags = tm_workspace_find(root, tm_tag_max_t, NULL, TRUE);
+	if ((tags) && (tags->len > 0))
+	{
+		GString *words = g_string_sized_new(100);
+		TMTag *tag;
+		for (guint i=0; ((i < tags->len) && (i < MAX_AUTOCOMPLETE_WORDS)); ++i)
+		{
+			tag = (TMTag *) tags->pdata[i];
+			if (i > 0)
+				g_string_append_c(words, ' ');
+			g_string_append(words, tag->name);
 		}
-		char wordstart[WORDCHUNK];
-		GetRange(wEditor, posFind, Platform::Minimum(posFind + WORDCHUNK - 1, doclen), wordstart);
-		char *wordend = wordstart + rootlen;
-		while (iswordcharforsel(*wordend))
-			wordend++;
-		*wordend = '\0';
-		int wordlen = wordend - wordstart;
-		const char *wordbreak = words;
-		const char *wordpos;
-		for (;;) {	// searches the found word in the storage
-			wordpos = strstr (wordbreak, wordstart);
-			if (!wordpos)
-				break;
-			if (wordpos > words && wordpos[ -1] != ' ' ||
-			        wordpos[wordlen] && wordpos[wordlen] != ' ')
-				wordbreak = wordpos + wordlen;
-			else
-				break;
-		}
-		if (!wordpos) {	// add a new entry
-			newlength = length + wordlen;
-			if (length)
-				newlength++;
-			if (newlength >= size) {
-				do
-					size += WORDCHUNK;
-				while (size <= newlength);
-				words = (char*) realloc (words, size);
-			}
-			if (length)
-				words[length++] = ' ';
-			memcpy (words + length, wordstart, wordlen);
-			length = newlength;
-			words[length] = '\0';
-		}
-		ft.chrg.cpMin = posFind + wordlen;
+		SendEditorString(SCI_AUTOCSHOW, rootlen, words->str);
+		g_string_free(words, TRUE);
 	}
-	//DWORD dwEnd = timeGetTime();
-	//Platform::DebugPrintf("<%s> found %d characters took %d\n", root, length, dwEnd - dwStart);
-	if (length) {
-		SendEditorString(SCI_AUTOCSHOW, rootlen, words);
-	}
-	free(words);
 	return true;
 }
 
@@ -1066,7 +1035,7 @@ void AnEditor::AutomaticIndentation(char ch) {
 	}
 }
 
-// Upon a character being added, SciTE may decide to perform some action
+// Upon a character being added, AnEditor may decide to perform some action
 // such as displaying a completion list.
 void AnEditor::CharAdded(char ch) {
 	CharacterRange crange = GetSelection();
@@ -1074,7 +1043,6 @@ void AnEditor::CharAdded(char ch) {
 	int selEnd = crange.cpMax;
 	if ((selEnd == selStart) && (selStart > 0)) {
 		int style = SendEditor(SCI_GETSTYLEAT, selStart - 1, 0);
-		//Platform::DebugPrintf("Char added %d style = %d %d\n", ch, style, braceCount);
 		if (style != 1) {
 			if (SendEditor(SCI_CALLTIPACTIVE)) {
 				if (ch == ')') {
@@ -1112,7 +1080,7 @@ void AnEditor::GoMatchingBrace(bool select) {
 	int braceAtCaret = -1;
 	int braceOpposite = -1;
 	bool isInside = FindMatchingBracePosition(braceAtCaret, braceOpposite, true);
-	// Convert the chracter positions into caret positions based on whether
+	// Convert the character positions into caret positions based on whether
 	// the caret position was inside or outside the braces.
 	if (isInside) {
 		if (braceOpposite > braceAtCaret) {
@@ -1143,6 +1111,19 @@ int ControlIDOfCommand(unsigned long wParam) {
 
 long AnEditor::Command(int cmdID, long wParam, long lParam) {
 	switch (cmdID) {
+			
+	case ANE_INSERTTEXT:
+		SendEditor(SCI_INSERTTEXT,wParam,lParam);
+		break;
+	
+	case ANE_GETBOOKMARK_POS:
+		return GetBookmarkLine( wParam );
+		break;	/* pleonastico */
+
+	case ANE_BOOKMARK_TOGGLE_LINE:
+		BookmarkToggle( wParam );
+		break;
+
 	case ANE_UNDO:
 		SendEditor(SCI_UNDO);
 		break;
@@ -1354,6 +1335,9 @@ long AnEditor::Command(int cmdID, long wParam, long lParam) {
 		IndentationDecrease();
 		break;
 	
+	case ANE_GETLENGTH:
+		return SendEditor(SCI_GETLENGTH);
+
 	default:
 		break;
 	}
@@ -1361,14 +1345,12 @@ long AnEditor::Command(int cmdID, long wParam, long lParam) {
 }
 
 void AnEditor::FoldChanged(int line, int levelNow, int levelPrev) {
-	//Platform::DebugPrintf("Fold %d %x->%x\n", line, levelPrev, levelNow);
 	if (levelNow & SC_FOLDLEVELHEADERFLAG) {
 		SendEditor(SCI_SETFOLDEXPANDED, line, 1);
 	} else if (levelPrev & SC_FOLDLEVELHEADERFLAG) {
-		//Platform::DebugPrintf("Fold removed %d-%d\n", line, SendEditor(SCI_GETLASTCHILD, line));
 		if (!SendEditor(SCI_GETFOLDEXPANDED, line)) {
-			// Removing the fold from one that has been contracted so dhould expand
-			// otherwise lines are left invisibe with no war to make them visible
+			// Removing the fold from one that has been contracted so should expand
+			// otherwise lines are left invisible with no way to make them visible
 			Expand(line, true, false, 0, levelPrev);
 		}
 	}
@@ -1511,7 +1493,6 @@ void AnEditor::EnsureRangeVisible(int posStart, int posEnd) {
 
 bool AnEditor::MarginClick(int position, int modifiers) {
 	int lineClick = SendEditor(SCI_LINEFROMPOSITION, position);
-	//Platform::DebugPrintf("Margin click %d %d %x\n", position, lineClick,
 	//	SendEditor(SCI_GETFOLDLEVEL, lineClick) & SC_FOLDLEVELHEADERFLAG);
 	if (modifiers & SCMOD_SHIFT) {
 		FoldCloseAll();
@@ -1545,9 +1526,7 @@ void AnEditor::NotifySignal(GtkWidget *, gint /*wParam*/, gpointer lParam, AnEdi
 }
 
 void AnEditor::Notify(SCNotification *notification) {
-	//Platform::DebugPrintf("Notify %d\n", notification->nmhdr.code);
 	switch (notification->nmhdr.code) {
-
 	case SCN_KEY:{
 			if(!accelGroup) break;
 			int mods = 0;
@@ -1616,7 +1595,6 @@ static Colour ColourFromString(const char *val) {
 
 void AnEditor::SetOneStyle(Window &win, int style, const char *s) {
 	char *val = StringDup(s);
-	//Platform::DebugPrintf("Style %d is [%s]\n", style, val);
 	char *opt = val;
 	while (opt) {
 		char *cpComma = strchr(opt, ',');
@@ -1748,10 +1726,64 @@ void AnEditor::ReadProperties(const char *fileForExt) {
 
 	SendEditor(SCI_SETLEXER, lexLanguage);
 
-	apis.Clear();
-
 	SString kw0 = props->GetNewExpand("keywords.", fileNameForExtension.c_str());
 	SendEditorString(SCI_SETKEYWORDS, 0, kw0.c_str());
+	/* For C/C++ projects, get list of typedefs for colorizing */
+	if (SCLEX_CPP == lexLanguage)
+	{
+		const TMWorkspace *workspace = tm_get_workspace();
+
+		/* Assign global keywords */
+		if ((workspace) && (workspace->global_tags))
+		{
+			GPtrArray *g_typedefs = tm_tags_extract(workspace->global_tags
+			  , tm_tag_typedef_t | tm_tag_struct_t | tm_tag_class_t);
+			if ((g_typedefs) && (g_typedefs->len > 0))
+			{
+				GString *s = g_string_sized_new(g_typedefs->len * 10);
+				for (guint i = 0; i < g_typedefs->len; ++i)
+				{
+					if (!(TM_TAG(g_typedefs->pdata[i])->atts.entry.scope))
+					{
+						g_string_append(s, TM_TAG(g_typedefs->pdata[i])->name);
+						g_string_append_c(s, ' ');
+					}
+				}
+				SendEditorString(SCI_SETKEYWORDS, 2, s->str);
+				g_string_free(s, TRUE);
+			}
+			g_ptr_array_free(g_typedefs, TRUE);
+		}
+
+		/* Assign project keywords */
+		if ((workspace) && (workspace->work_object.tags_array))
+		{
+			GPtrArray *typedefs = tm_tags_extract(workspace->work_object.tags_array
+			  , tm_tag_typedef_t | tm_tag_struct_t | tm_tag_class_t);
+			if ((typedefs) && (typedefs->len > 0))
+			{
+				GString *s = g_string_sized_new(typedefs->len * 10);
+				for (guint i = 0; i < typedefs->len; ++i)
+				{
+					if (!(TM_TAG(typedefs->pdata[i])->atts.entry.scope))
+					{
+						if (!TM_TAG(typedefs->pdata[i])->name)
+						{
+							g_warning("NULL tag name encountered!");
+							sleep(10000);
+						}
+						g_string_append(s, TM_TAG(typedefs->pdata[i])->name);
+						g_string_append_c(s, ' ');
+					}
+				}
+				SendEditorString(SCI_SETKEYWORDS, 1, s->str);
+				g_string_free(s, TRUE);
+			}
+			g_ptr_array_free(typedefs, TRUE);
+		}
+	}
+	else
+	{
 	SString kw1 = props->GetNewExpand("keywords2.", fileNameForExtension.c_str());
 	SendEditorString(SCI_SETKEYWORDS, 1, kw1.c_str());
 	SString kw2 = props->GetNewExpand("keywords3.", fileNameForExtension.c_str());
@@ -1760,10 +1792,33 @@ void AnEditor::ReadProperties(const char *fileForExt) {
 	SendEditorString(SCI_SETKEYWORDS, 3, kw3.c_str());
 	SString kw4 = props->GetNewExpand("keywords5.", fileNameForExtension.c_str());
 	SendEditorString(SCI_SETKEYWORDS, 4, kw4.c_str());
+	}
 
 	SString fold = props->Get("fold");
 	SendEditorString(SCI_SETPROPERTY, reinterpret_cast<unsigned long>("fold"),
 	                 fold.c_str());
+
+	SString fold_compact = props->Get("fold.compact");
+	SendEditorString(SCI_SETPROPERTY, reinterpret_cast<unsigned long>("fold.compact"),
+	                 fold_compact.c_str());
+
+	SString fold_comment = props->Get("fold.comment");
+	SendEditorString(SCI_SETPROPERTY, reinterpret_cast<unsigned long>("fold.comment"),
+					fold_comment.c_str());
+	
+	SString fold_comment_python = props->Get("fold.comment.python");
+	SendEditorString(SCI_SETPROPERTY, reinterpret_cast<unsigned long>("fold.comment.python"),
+					fold_comment_python.c_str());
+					
+	SString fold_quotes_python = props->Get("fold.quotes.python");
+	SendEditorString(SCI_SETPROPERTY, reinterpret_cast<unsigned long>("fold.quotes.python"),
+					fold_quotes_python.c_str());
+					
+	SString fold_html = props->Get("fold.html");
+	SendEditorString(SCI_SETPROPERTY, reinterpret_cast<unsigned long>("fold.html"),
+					fold_html.c_str());
+
+
 	SString stylingWithinPreprocessor = props->Get("styling.within.preprocessor");
 	SendEditorString(SCI_SETPROPERTY, reinterpret_cast<unsigned long>("styling.within.preprocessor"),
 	                 stylingWithinPreprocessor.c_str());
@@ -1771,23 +1826,6 @@ void AnEditor::ReadProperties(const char *fileForExt) {
 	SendEditorString(SCI_SETPROPERTY, reinterpret_cast<unsigned long>("tab.timmy.whinge.level"),
 	                 ttwl.c_str());
 
-	SString apifilename = props->GetNewExpand("api.", fileNameForExtension.c_str());
-	if (apifilename.length()) {
-		FILE *fp = fopen(apifilename.c_str(), "rb");
-		if (fp) {
-			fseek(fp, 0, SEEK_END);
-			int len = ftell(fp);
-			char *buffer = apis.Allocate(len);
-			if (buffer) {
-				fseek(fp, 0, SEEK_SET);
-				fread(buffer, 1, len, fp);
-				apis.SetFromAllocated();
-			}
-			fclose(fp);
-			//Platform::DebugPrintf("Finished api file %d\n", len);
-		}
-	}
-
 	SendEditor(SCI_SETEOLMODE, SC_EOL_LF);
 
 	characterSet = props->GetInt("character.set");
@@ -1953,28 +1991,57 @@ void AnEditor::ReadProperties(const char *fileForExt) {
 
 	SendEditor(SCI_SETMARGINMASKN, 2, SC_MASK_FOLDERS);
 	SendEditor(SCI_SETMARGINSENSITIVEN, 2, 1);
-
-	if (props->GetInt("fold.use.plus")) {
-		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPEN, SC_MARK_MINUS);
-		SendEditor(SCI_MARKERSETFORE, SC_MARKNUM_FOLDEROPEN, Colour(0xff, 0xff, 0xff).AsLong());
-		SendEditor(SCI_MARKERSETBACK, SC_MARKNUM_FOLDEROPEN, Colour(0, 0, 0).AsLong());
-		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDER, SC_MARK_PLUS);
-		SendEditor(SCI_MARKERSETFORE, SC_MARKNUM_FOLDER, Colour(0xff, 0xff, 0xff).AsLong());
-		SendEditor(SCI_MARKERSETBACK, SC_MARKNUM_FOLDER, Colour(0, 0, 0).AsLong());
-	} else {
-		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPEN, SC_MARK_ARROWDOWN);
-		SendEditor(SCI_MARKERSETFORE, SC_MARKNUM_FOLDEROPEN, Colour(0, 0, 0).AsLong());
-		SendEditor(SCI_MARKERSETBACK, SC_MARKNUM_FOLDEROPEN, Colour(0, 0, 0).AsLong());
-		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDER, SC_MARK_ARROW);
-		SendEditor(SCI_MARKERSETFORE, SC_MARKNUM_FOLDER, Colour(0, 0, 0).AsLong());
-		SendEditor(SCI_MARKERSETBACK, SC_MARKNUM_FOLDER, Colour(0, 0, 0).AsLong());
+	
+	int fold_symbols = props->GetInt("fold.symbols");
+	switch(fold_symbols)
+	{
+		case 0:
+			// Arrow pointing right for contracted folders, arrow pointing down for expanded
+			DefineMarker(SC_MARKNUM_FOLDEROPEN, SC_MARK_ARROWDOWN, Colour(0, 0, 0), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDER, SC_MARK_ARROW, Colour(0, 0, 0), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDERSUB, SC_MARK_EMPTY, Colour(0, 0, 0), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDERTAIL, SC_MARK_EMPTY, Colour(0, 0, 0), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDEREND, SC_MARK_EMPTY, Colour(0xff, 0xff, 0xff), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDEROPENMID, SC_MARK_EMPTY, Colour(0xff, 0xff, 0xff), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_EMPTY, Colour(0xff, 0xff, 0xff), Colour(0, 0, 0));
+			break;
+		case 1:
+			// Plus for contracted folders, minus for expanded
+			DefineMarker(SC_MARKNUM_FOLDEROPEN, SC_MARK_MINUS, Colour(0xff, 0xff, 0xff), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDER, SC_MARK_PLUS, Colour(0xff, 0xff, 0xff), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDERSUB, SC_MARK_EMPTY, Colour(0xff, 0xff, 0xff), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDERTAIL, SC_MARK_EMPTY, Colour(0xff, 0xff, 0xff), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDEREND, SC_MARK_EMPTY, Colour(0xff, 0xff, 0xff), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDEROPENMID, SC_MARK_EMPTY, Colour(0xff, 0xff, 0xff), Colour(0, 0, 0));
+			DefineMarker(SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_EMPTY, Colour(0xff, 0xff, 0xff), Colour(0, 0, 0));
+			break;
+		case 2:
+			// Like a flattened tree control using circular headers and curved joins
+			DefineMarker(SC_MARKNUM_FOLDEROPEN, SC_MARK_CIRCLEMINUS, Colour(0xff, 0xff, 0xff), Colour(0x40, 0x40, 0x40));
+			DefineMarker(SC_MARKNUM_FOLDER, SC_MARK_CIRCLEPLUS, Colour(0xff, 0xff, 0xff), Colour(0x40, 0x40, 0x40));
+			DefineMarker(SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE, Colour(0xff, 0xff, 0xff), Colour(0x40, 0x40, 0x40));
+			DefineMarker(SC_MARKNUM_FOLDERTAIL, SC_MARK_LCORNERCURVE, Colour(0xff, 0xff, 0xff), Colour(0x40, 0x40, 0x40));
+			DefineMarker(SC_MARKNUM_FOLDEREND, SC_MARK_CIRCLEPLUSCONNECTED, Colour(0xff, 0xff, 0xff), Colour(0x40, 0x40, 0x40));
+			DefineMarker(SC_MARKNUM_FOLDEROPENMID, SC_MARK_CIRCLEMINUSCONNECTED, Colour(0xff, 0xff, 0xff), Colour(0x40, 0x40, 0x40));
+			DefineMarker(SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_TCORNERCURVE, Colour(0xff, 0xff, 0xff), Colour(0x40, 0x40, 0x40));
+			break;
+		case 3:
+			// Like a flattened tree control using square headers
+			DefineMarker(SC_MARKNUM_FOLDEROPEN, SC_MARK_BOXMINUS, Colour(0xff, 0xff, 0xff), Colour(0x80, 0x80, 0x80));
+			DefineMarker(SC_MARKNUM_FOLDER, SC_MARK_BOXPLUS, Colour(0xff, 0xff, 0xff), Colour(0x80, 0x80, 0x80));
+			DefineMarker(SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE, Colour(0xff, 0xff, 0xff), Colour(0x80, 0x80, 0x80));
+			DefineMarker(SC_MARKNUM_FOLDERTAIL, SC_MARK_LCORNER, Colour(0xff, 0xff, 0xff), Colour(0x80, 0x80, 0x80));
+			DefineMarker(SC_MARKNUM_FOLDEREND, SC_MARK_BOXPLUSCONNECTED, Colour(0xff, 0xff, 0xff), Colour(0x80, 0x80, 0x80));
+			DefineMarker(SC_MARKNUM_FOLDEROPENMID, SC_MARK_BOXMINUSCONNECTED, Colour(0xff, 0xff, 0xff), Colour(0x80, 0x80, 0x80));
+			DefineMarker(SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_TCORNER, Colour(0xff, 0xff, 0xff), Colour(0x80, 0x80, 0x80));
+			break;
 	}
 
 	// Well, unlike scite, we want it everytime.
 	firstPropertiesRead = true;
 }
 
-// Anjtua: In our case, we read it everytime
+// Anjuta: In our case, we read it everytime
 void AnEditor::ReadPropertiesInitial() {
 	indentationWSVisible = props->GetInt("view.indentation.whitespace", 1);
 	ViewWhitespace(props->GetInt("view.whitespace"));
@@ -2012,6 +2079,26 @@ void AnEditor::ReadPropertiesInitial() {
 	SendEditor(SCI_SETMARGINWIDTHN, 2, foldMargin ? foldMarginWidth : 0);
 }
 
+
+
+void AnEditor::DefineMarker(int marker, int markerType, Colour fore, Colour back)
+{
+	SendEditor(SCI_MARKERDEFINE, marker, markerType);
+	SendEditor(SCI_MARKERSETFORE, marker, fore.AsLong());
+	SendEditor(SCI_MARKERSETBACK, marker, back.AsLong());
+}
+
+
+int AnEditor::GetBookmarkLine( const int nLineStart )
+{
+	int nNextLine = SendEditor(SCI_MARKERNEXT, nLineStart+1, 1 << ANE_MARKER_BOOKMARK);
+	//printf( "...look %d --> %d\n", nLineStart, nNextLine );
+	if ( (nNextLine < 0) || (nNextLine == nLineStart) )
+		return -1 ;
+	else
+		return nNextLine;
+}
+
 static GList* editors;
 
 static AnEditor*
@@ -2067,7 +2154,9 @@ aneditor_get_widget(AnEditorID handle)
    AnEditor *ed;
    ed = aneditor_get(handle);
    if(!ed) return NULL;
-   return ed->GetID();
+
+   // Forced conversion is safe here, so do it.
+   return (GtkWidget*)ed->GetID();
 }
 
 glong
@@ -2078,3 +2167,5 @@ aneditor_command(AnEditorID handle, gint command, glong wparam, glong lparam)
    if(!ed) return 0;
    return ed->Command(command, wparam, lparam);
 }
+
+
