@@ -23,6 +23,7 @@
 #include <unistd.h>
 #include <signal.h>
 #include <string.h>
+#include <syslog.h>
 
 #include <sys/wait.h>
 #include <gnome.h>
@@ -45,18 +46,25 @@
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 #include "properties.h"
-
 #include "commands.h"
+/*-------------------------------------------------------------------*/
+#define	LOCALS_PLUGIN_DIR	"/.anjuta/plugins"
 
 extern gboolean closing_state;
 static GdkCursor *app_cursor;
-
+/*-------------------------------------------------------------------*/
 void anjuta_child_terminated (int t);
 static void on_anjuta_window_selected (GtkMenuItem * menuitem,
 				       gpointer user_data);
-static void anjuta_clear_windows_menu (void);
-static void anjuta_fill_windows_menu (void);
 static void anjuta_show_text_editor (TextEditor * te);
+static void plugins_foreach_delete( gpointer data, gpointer user_data );
+static void free_plug_ins( GList * pList );
+static PlugInErr plug_in_init( AnjutaAddInPtr self, const gchar *szModName );
+static GList*scan_AddIns_in_directory( AnjutaApp* pApp, const gchar *szDirName, GList *pList );
+static int select_all_files (const struct dirent *e);
+
+/*-------------------------------------------------------------------*/
+
 
 static void
 anjuta_fatal_signal_handler (int t)
@@ -105,6 +113,15 @@ anjuta_new ()
 			on_save_as_filesel_ok_clicked,
 			on_save_as_filesel_cancel_clicked, NULL
 		};
+		/* Must declare static, because it will be used forever */
+		static FileSelData fsd3 = { N_("Save File As"), NULL,
+			on_build_msg_save_ok_clicked,
+			on_build_msg_save_cancel_clicked, NULL
+		};
+		app->size = sizeof(AnjutaApp);
+		app->addIns_list	= NULL ;
+		app->szDirPlugInDir	= PACKAGE_DATA_DIR "/plugins" ;
+		app->bUseComponentUI	= FALSE ;
 		app->shutdown_in_progress = FALSE;
 		app->registered_windows = NULL;
 		app->registered_child_processes = NULL;
@@ -126,9 +143,15 @@ anjuta_new ()
 		app->first_time_expose = TRUE;
 
 		create_anjuta_gui (app);
+		
 		app->dirs = anjuta_dirs_new ();
 		app->fileselection = create_fileselection_gui (&fsd1);
+		
+		/* Set to the current dir */
+		fileselection_set_dir (app->fileselection, (gchar *)get_current_dir_name());
+		
 		app->save_as_fileselection = create_fileselection_gui (&fsd2);
+		app->save_as_build_msg_sel = create_fileselection_gui (&fsd3);
 		app->find_replace = find_replace_new ();
 		app->find_in_files = find_in_files_new ();
 		app->preferences = preferences_new ();
@@ -141,6 +164,9 @@ anjuta_new ()
 		app->command_editor = command_editor_new (app->preferences->props_global,
 					app->preferences->props_local, app->project_dbase->props);
 		app->tags_manager = tags_manager_new ();
+		app->tm_workspace = tm_get_workspace();
+		if (TRUE != tm_workspace_load_global_tags(PACKAGE_DATA_DIR "/system.tags"))
+			g_warning("Unable to load global tag file");
 		app->help_system = anjuta_help_new();
 
 		app->widgets.the_client = app->widgets.vpaned;
@@ -166,7 +192,18 @@ anjuta_new ()
 		anjuta_update_title ();
 		launcher_init ();
 		debugger_init ();
-
+		app->addIns_list = scan_AddIns_in_directory( app, app->szDirPlugInDir, NULL );
+		{		
+			gchar *plugin_dir;
+			char const * const home_dir = getenv ("HOME");
+			/* Load the user plugins */
+			if (home_dir != NULL)
+			{
+				plugin_dir = g_strconcat (home_dir, LOCALS_PLUGIN_DIR, NULL);
+				app->addIns_list = scan_AddIns_in_directory( app, plugin_dir, app->addIns_list );
+				g_free (plugin_dir);
+			}
+		}
 		anjuta_load_yourself (app->preferences->props);
 		gtk_widget_set_uposition (app->widgets.window, app->win_pos_x,
 					  app->win_pos_y);
@@ -177,6 +214,7 @@ anjuta_new ()
 		debug_toolbar_update ();
 		format_toolbar_update ();
 		browser_toolbar_update ();
+		anjuta_apply_preferences();
 	}
 	else
 	{
@@ -213,6 +251,7 @@ anjuta_append_text_editor (gchar * filename)
 	{
 	case TEXT_EDITOR_PAGED:
 		label = gtk_label_new (te->filename);
+		te->widgets.tab_label = label;
 		gtk_widget_show (label);
 		eventbox = gtk_event_box_new ();
 		gtk_widget_show (eventbox);
@@ -415,13 +454,13 @@ anjuta_get_current_selection ()
 	return chars;
 }
 
-void
+TextEditor *
 anjuta_goto_file_line (gchar * fname, guint lineno)
 {
-	anjuta_goto_file_line_mark (fname, lineno, TRUE);
+	return anjuta_goto_file_line_mark (fname, lineno, TRUE);
 }
 
-void
+TextEditor *
 anjuta_goto_file_line_mark (gchar * fname, guint lineno, gboolean mark)
 {
 	gchar *fn;
@@ -429,9 +468,9 @@ anjuta_goto_file_line_mark (gchar * fname, guint lineno, gboolean mark)
 
 	TextEditor *te;
 
-	g_return_if_fail (fname != NULL);
+	g_return_val_if_fail (fname != NULL, NULL);
 	fn = anjuta_get_full_filename (fname);
-	g_return_if_fail (fname != NULL);
+	g_return_val_if_fail (fname != NULL, NULL);
 	node = app->text_editor_list;
 	while (node)
 	{
@@ -448,7 +487,7 @@ anjuta_goto_file_line_mark (gchar * fname, guint lineno, gboolean mark)
 			anjuta_show_text_editor (te);
 			anjuta_grab_text_focus ();
 			g_free (fn);
-			return;
+			return te ;
 		}
 		node = g_list_next (node);
 	}
@@ -456,7 +495,7 @@ anjuta_goto_file_line_mark (gchar * fname, guint lineno, gboolean mark)
 	if (lineno >= 0 && te)
 		text_editor_goto_line (te, lineno, mark);
 	g_free (fn);
-	return;
+	return te ;
 }
 
 static void
@@ -631,7 +670,9 @@ anjuta_save_settings ()
 
 gboolean anjuta_save_yourself (FILE * stream)
 {
+#ifdef	USE_STD_PREFERENCES
 	GList *node;
+#endif
 	PropsID pr;
 	gchar* key;
 
@@ -690,7 +731,7 @@ gboolean anjuta_save_yourself (FILE * stream)
 		 prop_get_int (pr, "margin.marker.visible", 0));
 	fprintf (stream, "margin.fold.visible=%d\n",
 		 prop_get_int (pr, "margin.fold.visible", 0));
-
+#ifdef	USE_STD_PREFERENCES
 	fprintf (stream, "anjuta.recent.files=");
 	node = app->recent_files;
 	while (node)
@@ -698,6 +739,7 @@ gboolean anjuta_save_yourself (FILE * stream)
 		fprintf (stream, "\\\n%s ", (gchar *) node->data);
 		node = g_list_next (node);
 	}
+	
 	fprintf (stream, "\n\n");
 
 	fprintf (stream, "anjuta.recent.projects=");
@@ -707,6 +749,11 @@ gboolean anjuta_save_yourself (FILE * stream)
 		fprintf (stream, "\\\n%s ", (gchar *) node->data);
 		node = g_list_next (node);
 	}
+#else
+	anjuta_session_save_strings( SECSTR(SECTION_RECENTFILES), app->recent_files );
+	anjuta_session_save_strings( SECSTR(SECTION_RECENTPROJECTS), app->recent_projects );
+#endif
+
 	fprintf (stream, "\n\n");
 	messages_save_yourself (app->messages, stream);
 	project_dbase_save_yourself (app->project_dbase, stream);
@@ -748,9 +795,17 @@ gboolean anjuta_load_yourself (PropsID pr)
 	gtk_paned_set_position (GTK_PANED (app->widgets.hpaned), length);
 
 	glist_strings_free (app->recent_files);
+#ifdef	USE_STD_PREFERENCES
 	app->recent_files = glist_from_data (pr, "anjuta.recent.files");
+#else
+	app->recent_files = anjuta_session_load_strings( SECSTR(SECTION_RECENTFILES), NULL );
+#endif
 	glist_strings_free (app->recent_projects);
+#ifdef	USE_STD_PREFERENCES
 	app->recent_projects = glist_from_data (pr, "anjuta.recent.projects");
+#else
+	app->recent_projects = anjuta_session_load_strings( SECSTR(SECTION_RECENTPROJECTS), NULL );
+#endif
 	messages_load_yourself (app->messages, pr);
 	project_dbase_load_yourself (app->project_dbase, pr);
 	preferences_load_yourself (app->preferences, pr);
@@ -779,22 +834,37 @@ anjuta_update_title ()
 			extended_toolbar_update ();
 		}
 		else
+		{
 			window = te->widgets.window;
+		}
 
 		if (text_editor_is_saved (te))
 		{
-			buff1 =
-				g_strdup_printf (_("Anjuta: %s(Saved)"),
-						 te->full_filename);
+			if (te->full_filename)
+			{
+				buff1 =
+					g_strdup_printf (_("Anjuta: %s(Saved)"),
+							 te->full_filename);
+			} else
+			{
+				buff1 = g_strdup("");
+			}
 			buff2 =
 				g_strdup_printf (_("Anjuta: %s"),
 						 te->filename);
 		}
 		else
 		{
-			buff1 =
-				g_strdup_printf (_("Anjuta: %s(Unsaved)"),
-						 te->full_filename);
+			if (te->full_filename)
+			{
+				buff1 =
+					g_strdup_printf (_("Anjuta: %s(Unsaved)"),
+							 te->full_filename);
+			}
+			else
+			{
+				buff1 = g_strdup("");
+			}
 			buff2 =
 				g_strdup_printf (_("Anjuta: %s"),
 						 te->filename);
@@ -806,13 +876,51 @@ anjuta_update_title ()
 						   te->full_filename);
 		}
 		else
+		{
 			gtk_window_set_title (GTK_WINDOW (window), buff2);
+		}
 		g_free (buff1);
 		g_free (buff2);
 	}
 	else
+	{
 		gtk_window_set_title (GTK_WINDOW (app->widgets.window),
 				      _("Anjuta: No file"));
+	}
+}
+
+void
+anjuta_update_page_label (TextEditor *te)
+{
+	GtkRcStyle *rc_style;
+	GdkColor tmpcolor;
+
+	if (te == NULL)
+		return;
+	
+	if (te->mode == TEXT_EDITOR_WINDOWED)
+		return;
+	
+	if (te->widgets.tab_label == NULL)
+		return;
+			
+	if (text_editor_is_saved(te))
+	{
+		gdk_color_parse("black",&tmpcolor);
+	}
+	else
+	{
+		gdk_color_parse("red",&tmpcolor);
+	}
+	
+	rc_style = gtk_rc_style_new();
+			
+	rc_style->fg[GTK_STATE_NORMAL] = tmpcolor;
+	rc_style->color_flags[GTK_STATE_NORMAL] = GTK_RC_FG;
+
+	gtk_widget_modify_style(te->widgets.tab_label, rc_style);
+	
+	gtk_rc_style_unref(rc_style);
 }
 
 void
@@ -823,6 +931,9 @@ anjuta_apply_preferences (void)
 	gint no_tag;
 
 	Preferences *pr = app->preferences;
+	
+	app->bUseComponentUI = preferences_get_int (pr, USE_COMPONENTS);
+	
 	no_tag = preferences_get_int (pr, EDITOR_TAG_HIDE);
 	if (no_tag == TRUE)
 	{
@@ -871,6 +982,20 @@ anjuta_refresh_breakpoints (TextEditor* te)
 	breakpoints_dbase_set_all_in_editor (debugger.breakpoints_dbase, te);
 }
 
+void
+anjuta_application_exit(void)
+{
+	g_return_if_fail(app != NULL );
+	if(		( NULL != app->project_dbase )
+		&&	(app->project_dbase->project_is_open ) )
+	{
+		project_dbase_close_project(app->project_dbase) ;
+	}
+	tm_workspace_free(tm_get_workspace());
+	free_plug_ins( app->addIns_list );
+	app->addIns_list	= NULL ;
+}
+
 void
 anjuta_clean_exit ()
 {
@@ -888,7 +1013,7 @@ anjuta_clean_exit ()
 
 /* Is it necessary to free up all the memos on exit? */
 /* Assuming that it is not, I am disabling the following */
-/* Basicaly, because it is faster */
+/* Basically, because it is faster */
 
 #if 0				/* From here */
 	if (app->project_dbase->project_is_open)
@@ -970,12 +1095,15 @@ anjuta_clean_exit ()
 	app->save_as_fileselection = NULL;
 	app->preferences = NULL;
 	main_menu_unref ();
+
 	app = NULL;
 #endif /* To here */
-
+	
 	gtk_main_quit ();
 }
 
+
+
 void
 anjuta_status (gchar * mesg, ...)
 {
@@ -1445,7 +1573,7 @@ anjuta_load_cmdline_files ()
 	}
 }
 
-static void
+void
 anjuta_clear_windows_menu ()
 {
 	guint count;
@@ -1471,7 +1599,7 @@ anjuta_delete_all_marker (gint marker)
 	}
 }
 
-static void
+void
 anjuta_fill_windows_menu ()
 {
 	gint count, i;
@@ -1590,7 +1718,7 @@ anjuta_update_app_status (gboolean set_job, gchar* job_name)
 	str =
 		g_strdup_printf(
 		_("  Project: %s       Zoom: %d       Line: %04d       Col: %03d       %s       Job: %s"),
-		 prj, zoom, line, col, edit, job);
+		 prj, zoom, line+1, col, edit, job);
 	gnome_appbar_set_default (GNOME_APPBAR (app->widgets.appbar), str);
 	g_free (str);
 	g_free (prj);
@@ -1642,3 +1770,266 @@ anjuta_get_file_property (gchar* key, gchar* filename, gint default_value)
 	value = atoi (value_str);
 	return value;
 }
+
+TextEditor *
+anjuta_get_te_from_path( const gchar *szFullPath )
+{
+	GList *node;
+
+	TextEditor *te;
+
+	g_return_val_if_fail ( szFullPath != NULL, NULL );
+	node = app->text_editor_list;
+	while (node)
+	{
+		te = (TextEditor *) node->data;
+		if (te->full_filename != NULL)
+		{
+			if ( !strcmp ( szFullPath, te->full_filename) )
+			{
+				return te ;
+			}
+		}
+		node = g_list_next (node);
+	}
+	return NULL ;
+}
+
+/* Saves a file to keep it synchronized with external programs */
+void 
+anjuta_save_file_if_modified( const gchar *szFullPath )
+{
+	TextEditor *te;
+
+	g_return_if_fail ( szFullPath != NULL );
+
+	te = anjuta_get_te_from_path( szFullPath );
+	if( NULL != te )
+	{
+		if( !text_editor_is_saved (te) )
+		{
+			text_editor_save_file (te);
+		}
+	}
+	return;
+}
+
+/* If an external program changed the file, we must reload it */
+void 
+anjuta_reload_file( const gchar *szFullPath )
+{
+	TextEditor *te;
+
+	g_return_if_fail ( szFullPath != NULL );
+
+	te = anjuta_get_te_from_path( szFullPath );
+	if( NULL != te )
+	{
+		glong	nNowPos = te->current_line ;
+		/*text_editor_check_disk_status ( te, TRUE );asd sdf*/
+		text_editor_load_file (te);
+		text_editor_goto_line ( te,  nNowPos, FALSE );
+	}
+	return;
+}
+
+
+
+AnjutaAddInPtr plug_in_new(void)
+{
+	AnjutaAddInPtr	self = g_new( AnjutaAddIn, 1 );
+	if( NULL != self )
+	{
+		self->m_Handle			= NULL ;
+		self->m_szModName		= NULL ;
+		self->m_bStarted		= FALSE ;
+		self->m_UserData		= NULL ;		
+	
+		self->GetDescr			= NULL ;
+		self->GetVersion		= NULL ;
+		self->Init				= NULL ;
+		self->CleanUp			= NULL ;
+		self->Activate			= NULL ;
+		self->GetMenuTitle		= NULL ;
+		self->GetTooltipText	= NULL ;
+	}
+	return self ;
+}
+
+void 
+plug_in_delete( AnjutaAddInPtr self )
+{
+	g_return_if_fail( NULL != self );
+	
+	if( self->m_Handle && self->CleanUp )
+		(*self->CleanUp)( self->m_Handle, self->m_UserData, app );
+
+	if( NULL != self->m_szModName )
+		g_free( self->m_szModName );
+
+	if( self->m_Handle )
+		g_module_close( self->m_Handle );
+
+	self->m_Handle	= NULL ;
+	
+	g_free( self );
+}
+
+
+static PlugInErr
+plug_in_init( AnjutaAddInPtr self, const gchar *szModName )
+{
+	PlugInErr	pieError = PIE_OK ;
+	
+	g_return_val_if_fail( (NULL != self), PIE_BADPARMS );
+	g_return_val_if_fail( (szModName != NULL ) && strlen(szModName) , PIE_BADPARMS ) ;
+	
+	self->m_szModName	= g_strdup( szModName ) ;
+	self->m_Handle		= g_module_open ( szModName , 0 );
+	if( NULL != self->m_Handle )
+	{
+		/* Symbol load */
+		if( 		g_module_symbol( self->m_Handle, "GetDescr", (gpointer*)&self->GetDescr ) 
+			&&	g_module_symbol( self->m_Handle, "GetVersion", (gpointer*)&self->GetVersion )
+			&&	g_module_symbol( self->m_Handle, "Init", (gpointer*)&self->Init )
+			&&	g_module_symbol( self->m_Handle, "CleanUp", (gpointer*)&self->CleanUp )
+			&&	g_module_symbol( self->m_Handle, "Activate", (gpointer*)&self->Activate )
+			&&	g_module_symbol( self->m_Handle, "GetMenuTitle", (gpointer*)&self->GetMenuTitle )
+			&&	g_module_symbol( self->m_Handle, "GetTooltipText", (gpointer*)&self->GetTooltipText ) )
+		{
+			self->m_bStarted = (*self->Init)( self->m_Handle, &self->m_UserData, app ) ;
+			if( !self->m_bStarted )
+			{
+				pieError = PIE_INITFAILED ;
+			}			
+		} else
+		{
+			self->GetDescr		= NULL ;
+			self->GetVersion	= NULL ;
+			self->Init			= NULL ;
+			self->CleanUp		= NULL ;
+			self->Activate		= NULL ;
+			self->GetMenuTitle	= NULL ;
+			self->m_bStarted	= FALSE ;
+			pieError = PIE_SYMBOLSNOTFOUND ;
+		}
+	} else
+		pieError = PIE_NOTLOADED ;
+	return pieError ;
+}
+
+static gboolean
+Load_plugIn( AnjutaAddInPtr self, const gchar *szModName )
+{
+	gboolean		bOK = FALSE ;
+	PlugInErr	lErr = plug_in_init( self, szModName ) ;
+	const gchar	*pErr = g_module_error();
+	if( NULL == pErr )
+		pErr = "";
+	switch( lErr )
+	{
+	default:
+		syslog( LOG_WARNING|LOG_USER, _("Unable to Load Plug in Module %s. Err: %s"), szModName, pErr );
+		break;
+	case PIE_NOTLOADED:
+		syslog( LOG_WARNING|LOG_USER, _("Unable to Load Plug in Module %s. Err: %s"), szModName, pErr );
+		break;
+	case PIE_SYMBOLSNOTFOUND:
+		syslog( LOG_WARNING|LOG_USER, _("Plug in Module protocol unknown %s Err:%s"), szModName, pErr );
+		break;
+	case PIE_INITFAILED:
+		syslog( LOG_WARNING|LOG_USER, _("Unable to Start Plug in Module %s"), szModName );
+		break;
+	case PIE_BADPARMS:
+		syslog( LOG_WARNING|LOG_USER, _("Bad Parameters to Plug in %s. Err: %s"), szModName, pErr );
+		break;
+	case PIE_OK:
+		bOK = TRUE ;
+		break;
+	}
+		
+	if( ! bOK )
+		anjuta_error ( _("Unable to load plugin %s.\nError %s"), szModName, pErr  );
+	return bOK ;
+}
+
+
+static void
+plugins_foreach_delete( gpointer data, gpointer user_data )
+{
+	plug_in_delete( (AnjutaAddInPtr)data );
+}
+
+static void
+free_plug_ins( GList * pList )
+{
+	if( pList )
+	{
+		g_list_foreach( pList, plugins_foreach_delete, NULL );
+		g_list_free (pList);
+	}
+}
+
+static int
+select_all_files (const struct dirent *e)
+{
+	return TRUE; 
+}
+
+
+static GList *
+scan_AddIns_in_directory (AnjutaApp* pApp, const gchar *szDirName, GList *pList)
+{
+	GList *files;
+	GList *node;
+
+	if (!g_module_supported ())
+		return NULL ;
+	
+	g_return_val_if_fail (pApp != NULL, pList);
+	g_return_val_if_fail (((NULL != szDirName) && strlen(szDirName)), pList);
+
+	files = NULL;
+
+	/* Can not use p->top_proj_dir yet. */
+
+	/* Can not use project_dbase_get_module_dir() yet */
+	files = scan_files_in_dir (szDirName, select_all_files);
+
+	node = files;
+	while (node)
+	{
+		const char* entry_name;
+		gboolean bOK = FALSE;
+			
+		entry_name = (const char*) node->data;
+		node = g_list_next (node);
+			
+		if ((NULL != entry_name) && strlen (entry_name) > 3 && (NULL != strstr (entry_name, ".so")))
+		{
+			/* FIXME: e questo ? bah library_file = g_module_build_path (module_dir, module_file); */
+			gchar *pPIPath = g_strdup_printf ("%s/%s", szDirName, entry_name);
+			AnjutaAddInPtr pPlugIn = plug_in_new ();
+
+			if ((NULL != pPlugIn) && (NULL != pPIPath))
+			{
+				if (Load_plugIn (pPlugIn, pPIPath))
+				{
+					pList = g_list_prepend (pList, pPlugIn);
+					bOK = TRUE ;
+				}
+			} else
+				anjuta_error (_("Out of Memory scanning for plugins."));
+
+			g_free (pPIPath);
+
+			if (!bOK && (NULL != pPlugIn))
+				plug_in_delete (pPlugIn);
+		}
+	}
+
+	free_string_list (files);
+
+	return pList;
+}
+
