@@ -867,7 +867,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[3] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for Implicit declarations"));
+						 ("Warning for implicit declarations"));
 	gtk_widget_show (checkbutton[3]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[3], 0, 1, 2, 3,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -875,7 +875,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[4] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for mismached Return types"));
+						 ("Warning for mismatched return types"));
 	gtk_widget_show (checkbutton[4]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[4], 0, 1, 3, 4,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -883,7 +883,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[5] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for Unused Variables"));
+						 ("Warning for unused variables"));
 	gtk_widget_show (checkbutton[5]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[5], 0, 1, 4, 5,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -891,7 +891,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[6] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for unhandled Switch case with enums"));
+						 ("Warning for unhandled switch case with enums"));
 	gtk_widget_show (checkbutton[6]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[6], 0, 1, 5, 6,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -899,7 +899,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[7] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for nested Comment"));
+						 ("Warning for nested comment"));
 	gtk_widget_show (checkbutton[7]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[7], 0, 1, 6, 7,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -907,7 +907,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[8] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for Uninitialized variable use"));
+						 ("Warning for uninitialized variable use"));
 	gtk_widget_show (checkbutton[8]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[8], 0, 1, 7, 8,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -915,7 +915,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[9] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for missing Parentheses"));
+						 ("Warning for missing parentheses"));
 	gtk_widget_show (checkbutton[9]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[9], 0, 1, 8, 9,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -923,7 +923,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[10] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for Un-Traditional syntax"));
+						 ("Warning on differences to traditional syntax"));
 	gtk_widget_show (checkbutton[10]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[10], 0, 1, 9, 10,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -931,7 +931,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[11] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for Variable Shadowing"));
+						 ("Warning for variable shadowing"));
 	gtk_widget_show (checkbutton[11]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[11], 0, 1, 10, 11,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -939,7 +939,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[12] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for suspected Pointer-arith"));
+						 ("Warning for suspected pointer arithmetic"));
 	gtk_widget_show (checkbutton[12]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[12], 0, 1, 11, 12,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -947,7 +947,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[13] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for Missing-prototypes"));
+						 ("Warning for missing prototypes"));
 	gtk_widget_show (checkbutton[13]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[13], 0, 1, 12, 13,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -955,7 +955,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[14] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for Un-Inline-able declarations"));
+						 ("Warning if declarations cannot be inlined"));
 	gtk_widget_show (checkbutton[14]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[14], 0, 1, 13, 14,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -963,7 +963,7 @@ create_compiler_options_page5 (CompilerOptions * co)
 
 	checkbutton[15] =
 		gtk_check_button_new_with_label (_
-						 ("Warning for Overloaded-virtuals"));
+						 ("Warning for overloaded virtuals"));
 	gtk_widget_show (checkbutton[15]);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton[15], 0, 1, 14, 15,
 			  (GtkAttachOptions) (GTK_FILL),
@@ -1009,7 +1009,7 @@ create_compiler_options_page6 (CompilerOptions * co)
 	radiobutton[1] =
 		gtk_radio_button_new_with_label (vbox2_group,
 						 _
-						 ("Low level optimization (Machine dependent)"));
+						 ("Low-level optimization (machine-dependent)"));
 	vbox2_group =
 		gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton[1]));
 	gtk_widget_show (radiobutton[1]);
@@ -1019,7 +1019,7 @@ create_compiler_options_page6 (CompilerOptions * co)
 	radiobutton[2] =
 		gtk_radio_button_new_with_label (vbox2_group,
 						 _
-						 ("Higher level optimization (without space-speed tradeoff)"));
+						 ("Higher-level optimization (without space-speed tradeoff)"));
 	vbox2_group =
 		gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton[2]));
 	gtk_widget_show (radiobutton[2]);
@@ -1029,7 +1029,7 @@ create_compiler_options_page6 (CompilerOptions * co)
 	radiobutton[3] =
 		gtk_radio_button_new_with_label (vbox2_group,
 						 _
-						 ("Fastest code optimization (Function inlined where ever possible"));
+						 ("Fastest-code optimization (functions inlined whereever possible"));
 	vbox2_group =
 		gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton[3]));
 	gtk_widget_show (radiobutton[3]);
@@ -1068,13 +1068,15 @@ create_compiler_options_page7 (CompilerOptions * co)
 {
 	GtkWidget *frame;
 	GtkWidget *frame1;
-	GtkWidget *event_box1;
-	GtkWidget *event_box2;
 	GtkWidget *vbox1;
+	GtkWidget *table1;
+	GtkWidget *label1;
+	GtkWidget *label2;
+	GtkWidget *label3;
 	GtkWidget *entry1;
 	GtkWidget *entry2;
+	GtkWidget *entry3;
 	GtkWidget *checkbutton[2];
-	GtkWidget *hseparator2;
 	gint i;
 
 	frame = gtk_frame_new (NULL);
@@ -1099,41 +1101,56 @@ create_compiler_options_page7 (CompilerOptions * co)
 	gtk_box_pack_start (GTK_BOX (vbox1), checkbutton[1], FALSE, FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (checkbutton[1]), 5);
 
-	hseparator2 = gtk_hseparator_new ();
-	gtk_widget_show (hseparator2);
-	gtk_box_pack_start (GTK_BOX (vbox1), hseparator2, TRUE, TRUE, 0);
-
-	frame1 =
-		gtk_frame_new (_
-			       ("Additional compiler options (command line options)"));
+	frame1 = gtk_frame_new (_("Additional Options"));
 	gtk_widget_show (frame1);
-	gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox1), frame1, FALSE, TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (frame1), 5);
 
-	event_box1 = gtk_event_box_new ();
-	gtk_widget_show (event_box1);
-	gtk_container_add (GTK_CONTAINER (frame1), event_box1);
-	gtk_container_set_border_width (GTK_CONTAINER (event_box1), 5);
-
+	table1 = gtk_table_new(3, 2, FALSE);
+	gtk_widget_show (table1);
+	gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
+	gtk_table_set_row_spacings (GTK_TABLE (table1), 5);
+	gtk_table_set_col_spacings (GTK_TABLE (table1), 5);
+	gtk_container_add (GTK_CONTAINER (frame1), table1);
+	
+	label1 = gtk_label_new("Compiler flags:");
+	gtk_widget_show (label1);
+	gtk_misc_set_alignment (GTK_MISC (label1), 0, -1);
+	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
+			  (GtkAttachOptions) 0,
+			  (GtkAttachOptions) 0, 0, 0);
+	
 	entry1 = gtk_entry_new ();
 	gtk_widget_show (entry1);
-	gtk_container_add (GTK_CONTAINER (event_box1), entry1);
-
-	frame1 =
-		gtk_frame_new (_
-			       ("Additional linker options (command line options)"));
-	gtk_widget_show (frame1);
-	gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (frame1), 5);
-
-	event_box2 = gtk_event_box_new ();
-	gtk_widget_show (event_box2);
-	gtk_container_add (GTK_CONTAINER (frame1), event_box2);
-	gtk_container_set_border_width (GTK_CONTAINER (event_box2), 5);
-
+	gtk_table_attach (GTK_TABLE (table1), entry1, 1, 2, 0, 1,
+			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+	
+	label2 = gtk_label_new("Linker flags:");
+	gtk_widget_show (label2);
+	gtk_misc_set_alignment (GTK_MISC (label2), 0, -1);
+	gtk_table_attach (GTK_TABLE (table1), label2, 0, 1, 1, 2,
+			  (GtkAttachOptions) 0,
+			  (GtkAttachOptions) 0, 0, 0);
+	
 	entry2 = gtk_entry_new ();
 	gtk_widget_show (entry2);
-	gtk_container_add (GTK_CONTAINER (event_box2), entry2);
+	gtk_table_attach (GTK_TABLE (table1), entry2, 1, 2, 1, 2,
+			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+
+	label3 = gtk_label_new("Additional libraries:");
+	gtk_widget_show (label3);
+	gtk_misc_set_alignment (GTK_MISC (label3), 0, -1);
+	gtk_table_attach (GTK_TABLE (table1), label3, 0, 1, 2, 3,
+			  (GtkAttachOptions) 0,
+			  (GtkAttachOptions) 0, 0, 0);
+	
+	entry3 = gtk_entry_new ();
+	gtk_widget_show (entry3);
+	gtk_table_attach (GTK_TABLE (table1), entry3, 1, 2, 2, 3,
+			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
 
 	for (i = 0; i < 2; i++)
 	{
@@ -1141,10 +1158,12 @@ create_compiler_options_page7 (CompilerOptions * co)
 		gtk_widget_ref (co->widgets.other_button[i]);
 	}
 
-	co->widgets.other_c_options_entry = entry1;
+	co->widgets.other_c_flags_entry = entry1;
 	gtk_widget_ref (entry1);
-	co->widgets.other_l_options_entry = entry2;
+	co->widgets.other_l_flags_entry = entry2;
 	gtk_widget_ref (entry2);
+	co->widgets.other_l_libs_entry = entry3;
+	gtk_widget_ref (entry3);
 
 	return frame;
 }
