@@ -115,6 +115,23 @@ find_in_files_hide (FindInFiles * ff)
 	ff->is_showing = FALSE;
 }
 
+void
+find_in_files_save_session ( FindInFiles* ff, ProjectDBase *p )
+{
+	g_return_if_fail( NULL != p );
+	g_return_if_fail( NULL != ff );
+	session_save_strings( p, SECSTR(SECTION_FIND_IN_FILES), ff->regexp_history );
+}
+
+void
+find_in_files_load_session( FindInFiles * ff, ProjectDBase *p )
+{
+	g_return_if_fail( NULL != p );
+	g_return_if_fail( NULL != ff );
+	ff->regexp_history = session_load_strings( p, SECSTR(SECTION_FIND_IN_FILES), ff->regexp_history );
+}
+
+
 /* Private */
 
 void
