@@ -31,6 +31,9 @@
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 
+
+static const gchar SECTION_REPLACE [] =  {"replace_text"};
+
 static void create_find_replace_gui (FindAndReplace * fr);
 
 static GtkWidget *create_replace_messagebox (void);
@@ -101,9 +104,54 @@ find_replace_destroy (FindAndReplace * fr)
 
 gboolean find_replace_save_yourself (FindAndReplace * fr, FILE * stream)
 {
+	find_text_save_settings (fr->find_text);
 	return TRUE;
 }
 
+void
+find_replace_save_session ( FindAndReplace * fr, ProjectDBase *p )
+{
+	g_return_if_fail( NULL != p );
+	g_return_if_fail( NULL != fr );
+	session_save_strings( p, SECSTR(SECTION_REPLACETEXT), fr->replace_history );
+	if( fr->find_text )
+		find_text_save_session ( fr->find_text, p );
+}
+
+void
+find_replace_load_session ( FindAndReplace * fr, ProjectDBase *p )
+{
+	g_return_if_fail( NULL != p );
+	g_return_if_fail( NULL != fr );
+	fr->replace_history	= session_load_strings( p, SECSTR(SECTION_REPLACETEXT), fr->replace_history );
+	if( fr->find_text )
+		find_text_load_session ( fr->find_text, p );
+}
+
+/*
+gboolean find_replace_save_project ( FindAndReplace * fr, ProjectDBase *p )
+{
+	g_return_val_if_fail( NULL != p, FALSE );
+	
+	session_clear_section( p, SECTION_SEARCH );
+	session_clear_section( p, SECTION_REPLACE );
+	if( fr->find_text->find_history )
+	{
+		const gchar *szFnd ;
+		int 		i;
+		int			nLen = g_list_length (fr->find_text->find_history); 
+		for (i = 0; i < nLen ; i++)
+		{
+			szFnd = (gchar*)g_list_nth (ft->find_history, i)->data ;
+			if( szFnd && szFnd[0] )
+				session_save_string( p, SECTION_SEARCH, i, szFnd );
+		}
+	}
+
+	return TRUE;
+}
+*/
+
 gboolean find_replace_load_yourself (FindAndReplace * fr, PropsID props)
 {
 	return TRUE;
@@ -164,6 +212,7 @@ find_replace_show (FindAndReplace * fr)
 
 	if (fr->is_showing)
 	{
+		gtk_widget_show (fr->r_gui.GUI);
 		gdk_window_raise (fr->r_gui.GUI->window);
 		return;
 	}
@@ -443,13 +492,13 @@ create_find_replace_gui (FindAndReplace * fr)
 	GTK_WIDGET_SET_FLAGS (button9, GTK_CAN_DEFAULT);
 
 	gnome_dialog_append_button (GNOME_DIALOG (dialog2),
-				    GNOME_STOCK_BUTTON_CANCEL);
+				    GNOME_STOCK_BUTTON_CLOSE);
 	button10 = g_list_last (GNOME_DIALOG (dialog2)->buttons)->data;
 	gtk_widget_show (button10);
 	GTK_WIDGET_SET_FLAGS (button10, GTK_CAN_DEFAULT);
 
 	gnome_dialog_append_button (GNOME_DIALOG (dialog2),
-				    GNOME_STOCK_BUTTON_OK);
+				    "Replace");
 	button11 = g_list_last (GNOME_DIALOG (dialog2)->buttons)->data;
 	gtk_widget_show (button11);
 	GTK_WIDGET_SET_FLAGS (button11, GTK_CAN_DEFAULT);
@@ -505,6 +554,7 @@ create_find_replace_gui (FindAndReplace * fr)
 	gtk_widget_ref (fr->r_gui.whole_word_check);
 
 	gtk_widget_grab_focus (combo_entry2);
+	gtk_window_set_transient_for(GTK_WINDOW(fr->r_gui.GUI), GTK_WINDOW(app->widgets.window));
 }
 
 static GtkWidget *
