@@ -26,6 +26,8 @@
 #include "find_text.h"
 #include "utilities.h"
 
+static const gchar SecFT [] = {"FindText"};
+/*static const gchar SECTION_SEARCH [] =  {"search_text"};*/
 
 FindText *
 find_text_new ()
@@ -35,13 +37,22 @@ find_text_new ()
 	if (ft)
 	{
 		ft->find_history = NULL;
-		ft->area = 1;
+		/*ft->area = 1;
 		ft->forward = TRUE;
 		ft->regexp = FALSE;
 		ft->ignore_case = FALSE;
 		ft->whole_word = FALSE;
 
+		ft->is_showing = FALSE;*/
+		
+		ft->area		= GetProfileInt( SecFT, "area", 1) ;
+		ft->forward		= GetProfileBool( SecFT, "forward", TRUE ) ;
+		ft->regexp		= GetProfileBool( SecFT, "regexp", FALSE ) ;
+		ft->ignore_case = GetProfileBool( SecFT, "ignore_case", FALSE ) ;
+		ft->whole_word	= GetProfileBool( SecFT, "whole_word", FALSE ) ;
+
 		ft->is_showing = FALSE;
+		
 		ft->pos_x = 100;
 		ft->pos_y = 100;
 		create_find_text_gui (ft);
@@ -49,6 +60,21 @@ find_text_new ()
 	return ft;
 }
 
+void
+find_text_save_settings (FindText * ft)
+{
+	if (ft)
+	{
+		/* Passivation */
+		WriteProfileInt( SecFT, "area", ft->area ) ;
+		WriteProfileBool( SecFT, "forward", ft->forward ) ;
+		WriteProfileBool( SecFT, "regexp", ft->regexp ) ;
+		WriteProfileBool( SecFT, "ignore_case", ft->ignore_case ) ;
+		WriteProfileBool( SecFT, "whole_word", ft->whole_word ) ;
+	}
+}
+
+
 void
 find_text_destroy (FindText * ft)
 {
@@ -78,6 +104,15 @@ find_text_destroy (FindText * ft)
 	}
 }
 
+void
+find_text_load_session( FindText * ft, ProjectDBase *p )
+{
+	g_return_if_fail( NULL != p );
+	g_return_if_fail( NULL != ft );
+
+	ft->find_history = session_load_strings( p, SECSTR(SECTION_FINDTEXT), ft->find_history );
+}
+
 void
 find_text_show (FindText * ft)
 {
@@ -124,6 +159,7 @@ find_text_show (FindText * ft)
 	entry_set_text_n_select (ft->f_gui.entry, NULL, TRUE);
 	if (ft->is_showing)
 	{
+		gtk_widget_show (ft->f_gui.GUI);
 		gdk_window_raise (ft->f_gui.GUI->window);
 		return;
 	}
@@ -135,8 +171,6 @@ find_text_show (FindText * ft)
 void
 find_text_hide (FindText * ft)
 {
-	gboolean radio0, radio1, state;
-
 	if (!ft)
 		return;
 	if (ft->is_showing == FALSE)
@@ -146,50 +180,6 @@ find_text_hide (FindText * ft)
 	gtk_widget_hide (ft->f_gui.GUI);
 	ft->is_showing = FALSE;
 
-	state =
-		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-					      (ft->f_gui.ignore_case_check));
-	if (state == TRUE)
-		ft->ignore_case = TRUE;
-	else
-		ft->ignore_case = FALSE;
-
-	state =
-		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-					      (ft->f_gui.whole_word_check));
-	if (state == TRUE)
-		ft->whole_word = TRUE;
-	else
-		ft->whole_word = FALSE;
-
-
-	radio0 =
-		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-					      (ft->f_gui.from_begin_radio));
-	radio1 =
-		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-					      (ft->f_gui.from_cur_loc_radio));
-
-	if (radio0)
-		ft->area = TEXT_EDITOR_FIND_SCOPE_WHOLE;
-	else
-		ft->area = TEXT_EDITOR_FIND_SCOPE_CURRENT;
-
-	state =
-		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-					      (ft->f_gui.forward_radio));
-	if (state == TRUE)
-		ft->forward = TRUE;
-	else
-		ft->forward = FALSE;
-
-	state =
-		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
-					      (ft->f_gui.regexp_radio));
-	if (state == TRUE)
-		ft->regexp = TRUE;
-	else
-		ft->regexp = FALSE;
 
 	ft->find_history = update_string_list (ft->find_history,
 					       gtk_entry_get_text (GTK_ENTRY
@@ -201,7 +191,29 @@ find_text_hide (FindText * ft)
 					       (app->widgets.toolbar.
 						main_toolbar.find_combo),
 					       ft->find_history);
+	/*if( app->project_dbase )
+		find_text_save_project( ft, app->project_dbase );*/
+}
 
+gboolean 
+find_text_save_session ( FindText * ft, ProjectDBase *p )
+{
+	g_return_val_if_fail( NULL != p, FALSE );
+	session_save_strings( p, SECSTR(SECTION_FINDTEXT), ft->find_history );
+	/*session_clear_section( p, SECTION_SEARCH );
+	if( ft->find_history )
+	{
+		const gchar *szFnd ;
+		int 		i;
+		int			nLen = g_list_length (ft->find_history); 
+		for (i = 0; i < nLen ; i++)
+		{
+			szFnd = (gchar*)g_list_nth (ft->find_history, i)->data ;
+			if( szFnd && szFnd[0] )
+				session_save_string( p, SECTION_SEARCH, i, szFnd );
+		}
+	}*/
+	return TRUE;
 }
 
 void
@@ -282,15 +294,15 @@ create_find_text_gui (FindText * ft)
 	vbox1_group =
 		gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton1));
 	gtk_widget_show (radiobutton1);
-	gtk_box_pack_start (GTK_BOX (vbox1), radiobutton1, FALSE, TRUE, 0);
 
 	radiobutton2 =
 		gtk_radio_button_new_with_label (vbox1_group,
-						 _("From Cursor"));
+						 _("From cursor"));
 	vbox1_group =
 		gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton2));
 	gtk_widget_show (radiobutton2);
 	gtk_box_pack_start (GTK_BOX (vbox1), radiobutton2, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox1), radiobutton1, FALSE, TRUE, 0);
 
 	frame4 = gtk_frame_new (_(" Direction "));
 	gtk_widget_show (frame4);
@@ -370,13 +382,13 @@ create_find_text_gui (FindText * ft)
 	GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
 
 	gnome_dialog_append_button (GNOME_DIALOG (dialog1),
-				    GNOME_STOCK_BUTTON_CANCEL);
+				    GNOME_STOCK_BUTTON_CLOSE);
 	button2 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
 	gtk_widget_show (button2);
 	GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);
 
 	gnome_dialog_append_button (GNOME_DIALOG (dialog1),
-				    GNOME_STOCK_BUTTON_OK);
+				    "Find");
 	button3 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
 	gtk_widget_show (button3);
 	GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);
@@ -420,6 +432,7 @@ create_find_text_gui (FindText * ft)
 	gtk_widget_ref (ft->f_gui.whole_word_check);
 
 	gtk_widget_grab_focus (combo_entry1);
+	gtk_window_set_transient_for(GTK_WINDOW(ft->f_gui.GUI), GTK_WINDOW(app->widgets.window));
 }
 
 void
@@ -430,8 +443,54 @@ on_find_text_ok_clicked (GtkButton * button, gpointer user_data)
 	gchar buff[512];
 	gint ret;
 	FindText *ft = user_data;
+	gboolean radio0, radio1, state;
+
+	state =
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
+					      (ft->f_gui.ignore_case_check));
+	if (state == TRUE)
+		ft->ignore_case = TRUE;
+	else
+		ft->ignore_case = FALSE;
+
+	state =
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
+					      (ft->f_gui.whole_word_check));
+	if (state == TRUE)
+		ft->whole_word = TRUE;
+	else
+		ft->whole_word = FALSE;
+
+
+	radio0 =
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
+					      (ft->f_gui.from_begin_radio));
+	radio1 =
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
+					      (ft->f_gui.from_cur_loc_radio));
+
+	if (radio0)
+		ft->area = TEXT_EDITOR_FIND_SCOPE_WHOLE;
+	else
+		ft->area = TEXT_EDITOR_FIND_SCOPE_CURRENT;
+
+	state =
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
+					      (ft->f_gui.forward_radio));
+	if (state == TRUE)
+		ft->forward = TRUE;
+	else
+		ft->forward = FALSE;
+
+	state =
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
+					      (ft->f_gui.regexp_radio));
+	if (state == TRUE)
+		ft->regexp = TRUE;
+	else
+		ft->regexp = FALSE;
+
 
-	find_text_hide (ft);
 	te = anjuta_get_current_text_editor ();
 	if (!te) return;
 	str = gtk_entry_get_text (GTK_ENTRY (ft->f_gui.entry));
@@ -453,7 +512,22 @@ on_find_text_ok_clicked (GtkButton * button, gpointer user_data)
 	}
 	ret = text_editor_find (te, string, ft->area, ft->forward, ft->regexp, ft->ignore_case, ft->whole_word);
 	g_free (string);
-	if (ret < 1)	anjuta_error (buff);
+	if (ret < 0)
+	{
+		if (ft->area == TEXT_EDITOR_FIND_SCOPE_CURRENT)
+		{
+			// Ask if user wants to wrap around the doc
+			messagebox2 (GNOME_MESSAGE_BOX_QUESTION,
+					_("No matches. Wrap search around the document?"),
+					GNOME_STOCK_BUTTON_YES,
+					GNOME_STOCK_BUTTON_NO,
+					GTK_SIGNAL_FUNC(on_find_text_start_over), 
+					GTK_SIGNAL_FUNC(on_find_text_cancel_clicked),
+					user_data);
+		}
+		else
+			anjuta_error (buff);
+	}
 }
 
 void
@@ -476,3 +550,20 @@ on_find_text_delete_event (GtkWidget * widget,
 	find_text_hide (ft);
 	return TRUE;
 }
+
+
+void
+on_find_text_start_over (GtkButton * button, gpointer user_data)
+{
+	TextEditor *te = anjuta_get_current_text_editor();
+	long length;
+
+	length = aneditor_command(te->editor_id, ANE_GETLENGTH, 0, 0);
+	
+	if (app->find_replace->find_text->forward == TRUE)		
+		aneditor_command (te->editor_id, ANE_GOTOLINE, 0, 0); // search from doc start
+	else
+		aneditor_command (te->editor_id, ANE_GOTOLINE, length, 0); // search from doc end
+
+	on_find_text_ok_clicked(NULL, app->find_replace->find_text);
+}
