@@ -30,6 +30,10 @@
 void on_file_menu_realize (GtkWidget * widget, gpointer data);
 
 void on_project_menu_realize (GtkWidget * widget, gpointer data);
+/*-------------------------------------------------------------------------------------------*/
+static void on_plugins_menu_realize (GtkWidget * widget, gpointer data);
+static void on_plugins_menu_item_activate (GtkMenuItem * item, AnjutaAddInPtr pPlug );
+/*-------------------------------------------------------------------------------------------*/
 
 void
 create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
@@ -43,31 +47,47 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->file.save_as_file = file1_menu_uiinfo[3].widget;
 	mb->file.save_all_file = file1_menu_uiinfo[4].widget;
 	mb->file.close_file = file1_menu_uiinfo[5].widget;
-	mb->file.reload_file = file1_menu_uiinfo[6].widget;
-	mb->file.new_project = file1_menu_uiinfo[8].widget;
-	mb->file.open_project = file1_menu_uiinfo[9].widget;
-	mb->file.save_project = file1_menu_uiinfo[10].widget;
-	mb->file.close_project = file1_menu_uiinfo[11].widget;
-	mb->file.rename = file1_menu_uiinfo[13].widget;
+	mb->file.close_all_file = file1_menu_uiinfo[6].widget;
+	mb->file.reload_file = file1_menu_uiinfo[7].widget;
+	mb->file.new_project = file1_menu_uiinfo[9].widget;
+	mb->file.open_project = file1_menu_uiinfo[10].widget;
+	mb->file.save_project = file1_menu_uiinfo[11].widget;
+	mb->file.close_project = file1_menu_uiinfo[12].widget;
+	mb->file.rename = file1_menu_uiinfo[14].widget;
 	
 	/* Unimplemented */
-	gtk_widget_hide (file1_menu_uiinfo[13].widget);
 	gtk_widget_hide (file1_menu_uiinfo[14].widget);
+	gtk_widget_hide (file1_menu_uiinfo[15].widget);
 
-	mb->file.page_setup = file1_menu_uiinfo[15].widget;
-	mb->file.print = file1_menu_uiinfo[16].widget;
-	mb->file.recent_files = file1_menu_uiinfo[18].widget;
-	mb->file.recent_projects = file1_menu_uiinfo[19].widget;
-	mb->file.exit = file1_menu_uiinfo[21].widget;
-	for (i = 0; i < 22; i++)
+	mb->file.page_setup = file1_menu_uiinfo[16].widget;
+	mb->file.print = file1_menu_uiinfo[17].widget;
+	mb->file.recent_files = file1_menu_uiinfo[19].widget;
+	mb->file.recent_projects = file1_menu_uiinfo[20].widget;
+	mb->file.exit = file1_menu_uiinfo[22].widget;
+	for (i = 0; i < 23 ; i++)
+	{
 		gtk_widget_ref (file1_menu_uiinfo[i].widget);
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(file1_menu_uiinfo[i].widget));
+	}
 
 	mb->edit.uppercase = transform1_submenu_uiinfo[0].widget;
 	mb->edit.lowercase = transform1_submenu_uiinfo[1].widget;
 	mb->edit.convert = transform1_submenu_uiinfo[2].widget;
 	for (i = 0; i < 3; i++)
+	{
 		gtk_widget_ref (transform1_submenu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(transform1_submenu_uiinfo[i].widget));
+	}
+	
+	mb->edit.insert_c_gpl = inserttext1_submenu_uiinfo[0].widget;
+	mb->edit.insert_cpp_gpl = inserttext1_submenu_uiinfo[1].widget;
+	mb->edit.insert_datetime = inserttext1_submenu_uiinfo[2].widget;
+	for (i = 0; i < 3; i++)
+	{
+		gtk_widget_ref (inserttext1_submenu_uiinfo[i].widget);
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(inserttext1_submenu_uiinfo[i].widget));
+	}
+	
 	mb->edit.undo = edit1_menu_uiinfo[0].widget;
 	mb->edit.redo = edit1_menu_uiinfo[1].widget;
 	mb->edit.cut = edit1_menu_uiinfo[3].widget;
@@ -77,22 +97,27 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->edit.select_all = select1_submenu_uiinfo[0].widget;
 	mb->edit.select_brace = select1_submenu_uiinfo[1].widget;
 	mb->edit.select_block = select1_submenu_uiinfo[2].widget;
-	mb->edit.autocomplete = edit1_menu_uiinfo[11].widget;
-	mb->edit.calltip = edit1_menu_uiinfo[12].widget;
+	mb->edit.autocomplete = edit1_menu_uiinfo[12].widget;
+	mb->edit.calltip = edit1_menu_uiinfo[13].widget;
 	
-	mb->edit.find = edit1_menu_uiinfo[14].widget;
-	mb->edit.find_in_files = edit1_menu_uiinfo[15].widget;
-	mb->edit.find_replace = edit1_menu_uiinfo[16].widget;
+	mb->edit.find = edit1_menu_uiinfo[15].widget;
+	mb->edit.find_in_files = edit1_menu_uiinfo[16].widget;
+	mb->edit.find_replace = edit1_menu_uiinfo[17].widget;
 	mb->edit.goto_line = goto1_submenu_uiinfo[0].widget;
 	mb->edit.goto_brace = goto1_submenu_uiinfo[1].widget;
 	mb->edit.goto_block_start = goto1_submenu_uiinfo[2].widget;
 	mb->edit.goto_block_end = goto1_submenu_uiinfo[3].widget;
 	mb->edit.goto_prev_mesg = goto1_submenu_uiinfo[4].widget;
 	mb->edit.goto_next_mesg = goto1_submenu_uiinfo[5].widget;
-	mb->edit.edit_app_gui = edit1_menu_uiinfo[20].widget;
-	for (i = 0; i < 21; i++)
+	mb->edit.repeat_find = edit1_menu_uiinfo[18].widget;
+	mb->edit.edit_app_gui = edit1_menu_uiinfo[22].widget;
+	mb->edit.plugins = edit1_menu_uiinfo[24].widget;
+	for (i = 0; i < NUM_EDIT_MENUES ; i++)
+	{
 		gtk_widget_ref (edit1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(edit1_menu_uiinfo[i].widget));
+	}
+	
 	mb->view.main_toolbar = toolbar1_submenu_uiinfo[0].widget;
 	mb->view.extended_toolbar = toolbar1_submenu_uiinfo[2].widget;
 	mb->view.tags_toolbar = toolbar1_submenu_uiinfo[4].widget;
@@ -100,8 +125,11 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->view.browser_toolbar = toolbar1_submenu_uiinfo[8].widget;
 	mb->view.format_toolbar = toolbar1_submenu_uiinfo[10].widget;
 	for (i = 0; i < 11; i++)
+	{
 		gtk_widget_ref (toolbar1_submenu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(toolbar1_submenu_uiinfo[i].widget));
+	}
+	
 	mb->view.editor_linenos = editor1_submenu_uiinfo[0].widget;
 	mb->view.editor_markers = editor1_submenu_uiinfo[1].widget;
 	mb->view.editor_folds = editor1_submenu_uiinfo[2].widget;
@@ -109,8 +137,12 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->view.editor_whitespaces = editor1_submenu_uiinfo[4].widget;
 	mb->view.editor_eolchars = editor1_submenu_uiinfo[5].widget;
 	for (i = 0; i < 6; i++)
+	{
 		gtk_widget_ref (editor1_submenu_uiinfo[i].widget);
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(editor1_submenu_uiinfo[i].widget));
+	}
 
+	
 	mb->view.messages = view1_menu_uiinfo[0].widget;
 	mb->view.project_listing = view1_menu_uiinfo[1].widget;
 	mb->view.bookmarks = view1_menu_uiinfo[2].widget;
@@ -122,9 +154,13 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->view.signals = view1_menu_uiinfo[16].widget;
 	mb->view.dump_window = view1_menu_uiinfo[17].widget;
 	mb->view.console = view1_menu_uiinfo[19].widget;
-	for (i = 0; i < 20; i++)
+	mb->view.show_hide_locals = view1_menu_uiinfo[20].widget;
+	for (i = 0; i < NVIEWMENUS ; i++)
+	{
 		gtk_widget_ref (view1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(view1_menu_uiinfo[i].widget));
+	}
+	
 	/* unimplemented */
 	gtk_widget_hide (view1_menu_uiinfo[17].widget);
 	gtk_widget_hide (view1_menu_uiinfo[18].widget);
@@ -142,8 +178,12 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->project.add_po = import_file1_menu_uiinfo[5].widget;
 	mb->project.add_doc = import_file1_menu_uiinfo[6].widget;
 	for (i = 0; i < 7; i++)
+	{
 		gtk_widget_ref (import_file1_menu_uiinfo[i].widget);
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(import_file1_menu_uiinfo[i].widget));
+	}
 
+	
 	mb->project.new_file = project1_menu_uiinfo[0].widget;
 	mb->project.remove = project1_menu_uiinfo[3].widget;
 	mb->project.readme = project1_menu_uiinfo[5].widget;
@@ -153,8 +193,11 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->project.configure = project1_menu_uiinfo[10].widget;
 	mb->project.info = project1_menu_uiinfo[11].widget;
 	for (i = 0; i < 12; i++)
+	{
 		gtk_widget_ref (project1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(project1_menu_uiinfo[i].widget));
+	}
+	
 	/* Unimplemented */
 	gtk_widget_hide (project1_menu_uiinfo[0].widget);
 
@@ -168,8 +211,11 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->format.toggle_fold = format1_menu_uiinfo[10].widget;
 	mb->format.detach = format1_menu_uiinfo[12].widget;
 	for (i = 0; i < 13; i++)
+	{
 		gtk_widget_ref (format1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(format1_menu_uiinfo[i].widget));
+	}
+	
 	mb->build.compile = build1_menu_uiinfo[0].widget;
 	mb->build.build = build1_menu_uiinfo[2].widget;
 	mb->build.build_all = build1_menu_uiinfo[3].widget;
@@ -183,8 +229,11 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->build.execute = build1_menu_uiinfo[16].widget;
 	mb->build.execute_params = build1_menu_uiinfo[17].widget;
 	for (i = 0; i < 18; i++)
+	{
 		gtk_widget_ref (build1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(build1_menu_uiinfo[i].widget));
+	}
+	
 	mb->bookmark.toggle = bookmark1_menu_uiinfo[0].widget;
 	mb->bookmark.first = bookmark1_menu_uiinfo[2].widget;
 	mb->bookmark.prev = bookmark1_menu_uiinfo[3].widget;
@@ -192,8 +241,11 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->bookmark.last = bookmark1_menu_uiinfo[5].widget;
 	mb->bookmark.clear = bookmark1_menu_uiinfo[7].widget;
 	for (i = 0; i < 8; i++)
+	{
 		gtk_widget_ref (bookmark1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(bookmark1_menu_uiinfo[i].widget));
+	}
+	
 	mb->debug.cont = execution1_submenu_uiinfo[0].widget;
 	mb->debug.step_in = execution1_submenu_uiinfo[1].widget;
 	mb->debug.step_over = execution1_submenu_uiinfo[2].widget;
@@ -201,8 +253,11 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->debug.run_to_cursor = execution1_submenu_uiinfo[4].widget;
 
 	for (i = 0; i < 5; i++)
+	{
 		gtk_widget_ref (execution1_submenu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(execution1_submenu_uiinfo[i].widget));
+	}
+	
 	mb->debug.tog_break = breakpoints1_submenu_uiinfo[0].widget;
 	mb->debug.set_break = breakpoints1_submenu_uiinfo[1].widget;
 	mb->debug.show_breakpoints = breakpoints1_submenu_uiinfo[3].widget;
@@ -211,7 +266,10 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->debug.clear_all_breakpoints =
 		breakpoints1_submenu_uiinfo[5].widget;
 	for (i = 0; i < 6; i++)
+	{
 		gtk_widget_ref (breakpoints1_submenu_uiinfo[i].widget);
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(breakpoints1_submenu_uiinfo[i].widget));
+	}
 
 	mb->debug.info_targets = info1_submenu_uiinfo[0].widget;
 	mb->debug.info_program = info1_submenu_uiinfo[1].widget;
@@ -222,8 +280,11 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->debug.info_frame = info1_submenu_uiinfo[6].widget;
 	mb->debug.info_args = info1_submenu_uiinfo[7].widget;
 	for (i = 0; i < 8; i++)
+	{
 		gtk_widget_ref (info1_submenu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(info1_submenu_uiinfo[i].widget));
+	}
+	
 	mb->debug.start_debug = debug1_menu_uiinfo[0].widget;
 	mb->debug.open_exec = debug1_menu_uiinfo[2].widget;
 	mb->debug.load_core = debug1_menu_uiinfo[3].widget;
@@ -237,8 +298,11 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->debug.add_watch = debug1_menu_uiinfo[19].widget;
 	mb->debug.stop = debug1_menu_uiinfo[21].widget;
 	for (i = 0; i < 22; i++)
+	{
 		gtk_widget_ref (debug1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(debug1_menu_uiinfo[i].widget));
+	}
+	
 	mb->utilities.grep = utilities1_menu_uiinfo[0].widget;
 	mb->utilities.compare = utilities1_menu_uiinfo[1].widget;
 	mb->utilities.diff = utilities1_menu_uiinfo[2].widget;
@@ -248,21 +312,30 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->utilities.cross_ref = utilities1_menu_uiinfo[7].widget;
 	mb->utilities.trace = utilities1_menu_uiinfo[8].widget;
 	for (i = 0; i < 9; i++)
+	{
 		gtk_widget_ref (utilities1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(utilities1_menu_uiinfo[i].widget));
+	}
+	
 	mb->windows.new = windows1_menu_uiinfo[0].widget;
 	mb->windows.close = windows1_menu_uiinfo[1].widget;
 	for (i = 0; i < 2; i++)
+	{
 		gtk_widget_ref (utilities1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(windows1_menu_uiinfo[i].widget));
+	}
+	
 	mb->settings.compiler = settings1_menu_uiinfo[0].widget;
 	mb->settings.src_paths = settings1_menu_uiinfo[1].widget;
 	mb->settings.commands = settings1_menu_uiinfo[2].widget;
 	mb->settings.preferences = settings1_menu_uiinfo[4].widget;
 	mb->settings.default_preferences = settings1_menu_uiinfo[5].widget;
 	for (i = 0; i < 6; i++)
+	{
 		gtk_widget_ref (settings1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(settings1_menu_uiinfo[i].widget));
+	}
+	
 	mb->help.gnome = help1_menu_uiinfo[2].widget;
 	mb->help.man = help1_menu_uiinfo[3].widget;
 	mb->help.info = help1_menu_uiinfo[4].widget;
@@ -270,8 +343,11 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 	mb->help.search = help1_menu_uiinfo[7].widget;
 	mb->help.about = help1_menu_uiinfo[17].widget;
 	for (i = 1; i < 18; i++)
+	{
 		gtk_widget_ref (help1_menu_uiinfo[i].widget);
-
+		gtk_accel_group_attach(GNOME_APP(ap)->accel_group, GTK_OBJECT(help1_menu_uiinfo[i].widget));
+	}
+	
 	/* unimplemented */
 	gtk_widget_hide (menubar1_uiinfo[8].widget);
 
@@ -280,6 +356,9 @@ create_main_menubar (GtkWidget * ap, MainMenuBar * mb)
 
 	gtk_signal_connect (GTK_OBJECT (mb->file.recent_projects), "realize",
 			    GTK_SIGNAL_FUNC (on_project_menu_realize), NULL);
+	
+	gtk_signal_connect (GTK_OBJECT (mb->edit.plugins), "realize",
+			    GTK_SIGNAL_FUNC (on_plugins_menu_realize), NULL);
 }
 
 void
@@ -294,19 +373,21 @@ void
 main_menu_unref ()
 {
 	gint i;
-	for (i = 0; i < 22; i++)
+	for (i = 0; i < 23; i++)
 		gtk_widget_unref (file1_menu_uiinfo[i].widget);
 	for (i = 0; i < 3; i++)
 		gtk_widget_unref (select1_submenu_uiinfo[i].widget);
 	for (i = 0; i < 6; i++)
 		gtk_widget_unref (goto1_submenu_uiinfo[i].widget);
-	for (i = 0; i < 20; i++)
+	for (i = 0; i < 3; i++)
+		gtk_widget_unref (inserttext1_submenu_uiinfo[i].widget);
+	for (i = 0; i < NUM_EDIT_MENUES ; i++)
 		gtk_widget_unref (edit1_menu_uiinfo[i].widget);
 	for (i = 0; i < 11; i++)
 		gtk_widget_unref (toolbar1_submenu_uiinfo[i].widget);
 	for (i = 0; i < 6; i++)
 		gtk_widget_unref (editor1_submenu_uiinfo[i].widget);
-	for (i = 0; i < 16; i++)
+	for (i = 0; i < NVIEWMENUS; i++)
 		gtk_widget_unref (view1_menu_uiinfo[i].widget);
 	for (i = 0; i < 17; i++)
 		gtk_widget_unref (project1_menu_uiinfo[i].widget);
@@ -381,6 +462,57 @@ create_submenu (gchar * title, GList * strings, GtkSignalFunc callback_func)
 	return GTK_WIDGET (submenu);
 }
 
+static 
+GtkWidget *
+create_submenu_plugin (gchar * title, GList * pList, GtkSignalFunc callback_func)
+{
+	GtkWidget *submenu;
+	GtkWidget *item;
+	gint i;
+	submenu = gtk_menu_new ();
+
+	item = gtk_menu_item_new ();
+	gtk_menu_append (GTK_MENU (submenu), item);
+	gtk_widget_show (item);
+
+	item = gtk_menu_item_new ();
+	gtk_menu_append (GTK_MENU (submenu), item);
+	gtk_widget_show (item);
+
+	item = gtk_menu_item_new_with_label (title);
+	gtk_widget_set_sensitive (item, FALSE);
+	gtk_menu_append (GTK_MENU (submenu), item);
+	gtk_widget_show (item);
+
+	item = gtk_menu_item_new ();
+	gtk_menu_append (GTK_MENU (submenu), item);
+	gtk_widget_show (item);
+
+	for (i = 0; i < g_list_length (pList); i++)
+	{
+		AnjutaAddInPtr	pPlug;
+		gchar			*szTitle ;
+		pPlug = (AnjutaAddInPtr) (g_list_nth (pList, i)->data);
+		szTitle = (*pPlug->GetMenuTitle)( pPlug->m_Handle, pPlug->m_UserData ) ;
+		item = gtk_menu_item_new_with_label ( szTitle );
+		g_free( szTitle );
+		gtk_menu_append (GTK_MENU (submenu), item);
+		gtk_widget_show (item);
+		gtk_signal_connect (GTK_OBJECT (item), "activate",
+				    callback_func, pPlug );
+	}
+	item = gtk_menu_item_new ();
+	gtk_menu_append (GTK_MENU (submenu), item);
+	gtk_widget_show (item);
+
+	item = gtk_menu_item_new ();
+	gtk_menu_append (GTK_MENU (submenu), item);
+	gtk_widget_show (item);
+
+	gtk_widget_show (submenu);
+	return GTK_WIDGET (submenu);
+}
+
 void
 on_file_menu_realize (GtkWidget * widget, gpointer data)
 {
@@ -392,6 +524,23 @@ on_file_menu_realize (GtkWidget * widget, gpointer data)
 	app->widgets.menubar.file.recent_files = widget;
 }
 
+static void
+on_plugins_menu_realize (GtkWidget * widget, gpointer data)
+{
+	GtkWidget *submenu =
+		create_submenu_plugin (_("Plug Ins"), app->addIns_list,
+				GTK_SIGNAL_FUNC
+				(on_plugins_menu_item_activate));
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (widget), submenu);
+	app->widgets.menubar.edit.plugins = widget;
+}
+
+static void
+on_plugins_menu_item_activate (GtkMenuItem * item, AnjutaAddInPtr pPlug )
+{
+	(*pPlug->Activate)( pPlug->m_Handle, pPlug->m_UserData, app ) ;
+}
+
 void
 on_project_menu_realize (GtkWidget * widget, gpointer data)
 {
