@@ -32,6 +32,16 @@ static GnomeUIInfo recent_projects1_menu_uiinfo[] = {
 	GNOMEUIINFO_END
 };
 
+static GnomeUIInfo plugins_menu_uiinfo[] = {
+	{
+	 GNOME_APP_UI_ITEM, N_("plugins"),
+	 NULL,
+	 on_nonimplemented_activate, NULL, NULL,
+	 GNOME_APP_PIXMAP_NONE, NULL,
+	 0, 0, NULL},
+	GNOMEUIINFO_END
+};
+
 static GnomeUIInfo file1_menu_uiinfo[] = {
 	{
 	 GNOME_APP_UI_ITEM, N_("_New"),
@@ -69,6 +79,12 @@ static GnomeUIInfo file1_menu_uiinfo[] = {
 	 on_close_file1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE,
 	 GDK_D, GDK_MOD1_MASK, NULL},
+	{
+	 GNOME_APP_UI_ITEM, N_("Close All Files"),
+	 N_("Close all files"),
+	 on_close_all_file1_activate, NULL, NULL,
+	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE,
+	 GDK_D, GDK_MOD1_MASK, NULL},
 	{
 	 GNOME_APP_UI_ITEM, N_("Reload F_ile"),
 	 N_("Reload current file"),
@@ -182,6 +198,29 @@ static GnomeUIInfo select1_submenu_uiinfo[] = {
 	GNOMEUIINFO_END
 };
 
+static GnomeUIInfo inserttext1_submenu_uiinfo[] = {
+       {
+        GNOME_APP_UI_ITEM, N_("/* GPL Notice */"),
+        N_("Insert GPL notice with C style comments"),
+        on_insert_c_gpl_notice, NULL, NULL,
+        GNOME_APP_PIXMAP_NONE, NULL,
+        0, 0, NULL},
+       {
+        GNOME_APP_UI_ITEM, N_("// GPL Notice"),
+        N_("Insert GPL notice with C++ style comments"),
+        on_insert_cpp_gpl_notice, NULL, NULL,
+        GNOME_APP_PIXMAP_NONE, NULL,
+        0, 0, NULL},
+       { 
+        GNOME_APP_UI_ITEM, N_("Current Date & Time"),
+        N_("Insert current date & time"),
+        on_insert_date_time, NULL, NULL,
+        GNOME_APP_PIXMAP_NONE, NULL,
+        0, 0, NULL},
+       GNOMEUIINFO_END
+};
+
+
 static GnomeUIInfo goto1_submenu_uiinfo[] = {
 	{
 	 GNOME_APP_UI_ITEM, N_("_Goto Line no ..."),
@@ -222,124 +261,156 @@ static GnomeUIInfo goto1_submenu_uiinfo[] = {
 	GNOMEUIINFO_END
 };
 
-static GnomeUIInfo edit1_menu_uiinfo[] = {
-	{
+
+#define	NUM_EDIT_MENUES	(25)
+
+static GnomeUIInfo edit1_menu_uiinfo[NUM_EDIT_MENUES+1] = {
+	{/*0*/
 	 GNOME_APP_UI_ITEM, N_("U_ndo"),
 	 N_("Undo the last action"),
 	 on_undo1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO,
 	 GDK_Z, GDK_CONTROL_MASK, NULL},
 
-	{
+	{/*1*/
 	 GNOME_APP_UI_ITEM, N_("_Redo"),
 	 N_("Redo the last undone action"),
 	 on_redo1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REDO,
 	 GDK_R, GDK_CONTROL_MASK, NULL},
 
-	 GNOMEUIINFO_SEPARATOR,
+	 GNOMEUIINFO_SEPARATOR,/*2*/
 	
-	{
+	{/*3*/
 	 GNOME_APP_UI_ITEM, N_("C_ut"),
 	 N_("Cut the selected text from the editor to the clipboard"),
 	 on_cut1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CUT,
 	 GDK_X, GDK_CONTROL_MASK, NULL},
 	
-	{
+	{/*4*/
 	 GNOME_APP_UI_ITEM, N_("_Copy"),
 	 N_("Copy the selected text to the clipboard"),
 	 on_copy1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY,
 	 GDK_C, GDK_CONTROL_MASK, NULL},
 	
-	{
+	{/*5*/
 	 GNOME_APP_UI_ITEM, N_("_Paste"),
 	 N_("Paste the content of clipboard at the current position"),
 	 on_paste1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE,
 	 GDK_V, GDK_CONTROL_MASK, NULL},
 	
-	{
+	{/*6*/
 	 GNOME_APP_UI_ITEM, N_("C_lear"),
 	 N_("Delete the selected text from the editor"),
 	 on_clear1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_Delete, 0, NULL},
 	
-	GNOMEUIINFO_SEPARATOR,
+	GNOMEUIINFO_SEPARATOR,/*7*/
 	
-	{
+	{/*8*/
 	 GNOME_APP_UI_SUBTREE, N_("_Transform"),
 	 NULL,
 	 transform1_submenu_uiinfo, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 0, 0, NULL},
 	
-	{
+	{/*9*/
 	 GNOME_APP_UI_SUBTREE, N_("_Select"),
 	 NULL,
 	 select1_submenu_uiinfo, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 0, 0, NULL},
 	
-	GNOMEUIINFO_SEPARATOR,
+	{/*10*/
+	 GNOME_APP_UI_SUBTREE, N_("_Insert text"),
+	 NULL,
+	 inserttext1_submenu_uiinfo, NULL, NULL,
+	 GNOME_APP_PIXMAP_NONE, NULL,
+	 0, 0, NULL},
 	
-	{
+	GNOMEUIINFO_SEPARATOR,/*11*/
+	
+	{/*12*/
 	 GNOME_APP_UI_ITEM, N_("_AutoComplete"),
 	 N_("AutoComplete the current word"),
 	 on_autocomplete1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_Return, GDK_CONTROL_MASK, NULL},
 	
-	{
+	{/*13*/
 	 GNOME_APP_UI_ITEM, N_("S_how calltip"),
 	 N_("Show calltip for the function"),
 	 on_calltip1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 0, 0, NULL},
 
-	 GNOMEUIINFO_SEPARATOR,
+	 GNOMEUIINFO_SEPARATOR,/*14*/
 	
-	{
+	{/*15*/
 	 GNOME_APP_UI_ITEM, N_("_Find ..."),
 	 N_("Search for a string or regexp in the editor"),
 	 on_find1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
 	 GDK_F, GDK_CONTROL_MASK, NULL},
 	
-	{
+	{/*16*/
 	 GNOME_APP_UI_ITEM, N_("Fin_d in files ..."),
 	 N_("Search for a string in multiple files or directories"),
 	 on_find_in_files1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 0, 0, NULL},
 	
-	{
+	{/*17*/
 	GNOME_APP_UI_ITEM, N_("Find and R_eplace ..."),
 	N_("Search for and replace a string or regexp with another string"),
 	on_find_and_replace1_activate, NULL, NULL,
 	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SRCHRPL,
 	GDK_R, GDK_CONTROL_MASK, NULL},
 	
-	GNOMEUIINFO_SEPARATOR,
-	
-	{
+	{/*18*/
+	 GNOME_APP_UI_ITEM, N_("Find _Next"),
+	 N_("Repeat the Last Find command"),
+	 on_findnext1_activate, NULL, NULL,
+	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
+	 GDK_P, GDK_CONTROL_MASK, NULL},
+	 
+	{/*19*/
 	 GNOME_APP_UI_SUBTREE, N_("G_o to"),
 	 NULL,
 	 goto1_submenu_uiinfo, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 0, 0, NULL},
 	
-	GNOMEUIINFO_SEPARATOR,
-	
-	{
+	GNOMEUIINFO_SEPARATOR,/*20*/
+
+	 {/*21*/
 	 GNOME_APP_UI_ITEM, N_("Ed_it Application GUI ..."),
 	 N_("Edit application GUI with the Glade GUI editor"),
 	 on_edit_app_gui1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_E, GDK_MOD1_MASK, NULL},
 	
+	GNOMEUIINFO_SEPARATOR,/*22*/
+	
+	{/*23*/
+	 GNOME_APP_UI_ITEM, N_("Save Build Messages"),
+	 N_("Save build messages to file"),
+	 on_save_build_messages_activate, NULL, NULL,
+	 GNOME_APP_PIXMAP_NONE, NULL,
+	 0, 0, NULL},
+	 
+	{/*24*/
+	 GNOME_APP_UI_SUBTREE, N_("_Plug Ins"),
+	 NULL,
+	 plugins_menu_uiinfo, NULL, NULL,
+	 GNOME_APP_PIXMAP_NONE, NULL,
+	 0, 0, NULL},
+
+	 
 	GNOMEUIINFO_END
 };
 
@@ -498,98 +569,105 @@ static GnomeUIInfo zoom_text1_submenu_uiinfo[] = {
 	GNOMEUIINFO_END
 };
 
-static GnomeUIInfo view1_menu_uiinfo[] = {
-	{
+#define	NVIEWMENUS	(21)
+static GnomeUIInfo view1_menu_uiinfo[NVIEWMENUS+1] = {
+	{/*0*/
 	 GNOME_APP_UI_ITEM, N_("_Messages"),
 	 N_("Show/Hide the Message window"),
 	 on_messages1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ALIGN_LEFT,
 	 GDK_F1, GDK_CONTROL_MASK, NULL},
-	{
+	{/*1*/
 	 GNOME_APP_UI_ITEM, N_("_Project Listing"),
 	 N_("Show/Hide the Project window"),
 	 on_project_listing1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ALIGN_LEFT,
 	 GDK_F2, GDK_CONTROL_MASK, NULL},
-	{
+	{/*2*/
 	 GNOME_APP_UI_ITEM, N_("B_ookmarks"),
 	 N_("Show the Bookmark window"),
 	 on_bookmarks1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_F3, GDK_CONTROL_MASK, NULL},
-	GNOMEUIINFO_SEPARATOR,
-	{
+	GNOMEUIINFO_SEPARATOR,/*3*/
+	{/*4*/
 	 GNOME_APP_UI_SUBTREE, N_("_Toolbars"),
 	 NULL,
 	 toolbar1_submenu_uiinfo, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 0, 0, NULL},
-	GNOMEUIINFO_SEPARATOR,
-	{
+	GNOMEUIINFO_SEPARATOR,/*5*/
+	{/*6*/
 	 GNOME_APP_UI_SUBTREE, N_("_Editor"),
 	 NULL,
 	 editor1_submenu_uiinfo, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 0, 0, NULL},
-	GNOMEUIINFO_SEPARATOR,
-	{
+	GNOMEUIINFO_SEPARATOR,/*7*/
+	{/*8*/
 	 GNOME_APP_UI_SUBTREE, N_("_Zoom text"),
 	 NULL,
 	 zoom_text1_submenu_uiinfo, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 0, 0, NULL},
-	GNOMEUIINFO_SEPARATOR,
-	{
+	GNOMEUIINFO_SEPARATOR,/*9*/
+	{/*10*/
 	 GNOME_APP_UI_ITEM, N_("_Breakpoints"),
 	 N_("Show breakpoints editor window"),
 	 on_breakpoints1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_F4, GDK_CONTROL_MASK, NULL},
-	GNOMEUIINFO_SEPARATOR,
-	{
+	GNOMEUIINFO_SEPARATOR,/*11*/
+	{/*12*/
 	 GNOME_APP_UI_ITEM, N_("_Watch Window"),
 	 N_("Show expression watch window"),
 	 on_watch_window1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_F5, GDK_CONTROL_MASK, NULL},
-	{
+	{/*13*/
 	 GNOME_APP_UI_ITEM, N_("_Registers"),
 	 N_("Show CPU registers and their contents"),
 	 on_registers1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_F6, GDK_CONTROL_MASK, NULL},
-	{
+	{/*14*/
 	 GNOME_APP_UI_ITEM, N_("Program _Stack"),
 	 N_("Show stack trace of the program"),
 	 on_program_stack1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_F7, GDK_CONTROL_MASK, NULL},
-	{
+	{/*15*/
 	 GNOME_APP_UI_ITEM, N_("Shared _Libraries"),
 	 N_("Show shared libraries loaded by the program"),
 	 on_shared_lib1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_F8, GDK_CONTROL_MASK, NULL},
-	{
+	{/*16*/
 	 GNOME_APP_UI_ITEM, N_("_Kernel Signals"),
 	 N_("Show the kernel signals editor window"),
 	 on_kernal_signals1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_F9, GDK_CONTROL_MASK, NULL},
-	{
+	{/*17*/
 	 GNOME_APP_UI_ITEM, N_("_Dump Window"),
 	 N_("Show memory dump window"),
 	 on_dump_window1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 GDK_F10, GDK_CONTROL_MASK, NULL},
-	GNOMEUIINFO_SEPARATOR,
-	{
+	GNOMEUIINFO_SEPARATOR,/*18*/
+	{/*19*/
 	 GNOME_APP_UI_ITEM, N_("_Console"),
 	 N_("Show the console where the program runs"),
 	 on_console1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_MIDI,
 	 GDK_F11, GDK_CONTROL_MASK, NULL},
-	GNOMEUIINFO_END
+	{/*20*/
+	 GNOME_APP_UI_TOGGLEITEM, N_("Show _Locals"),
+	 N_("Show/Hide Local Variables in MessagePane"),
+	 on_showhide_locals, NULL, NULL,
+	 GNOME_APP_PIXMAP_NONE, NULL,
+	 0, 0, NULL},
+	GNOMEUIINFO_END/*21*/
 };
 
 static GnomeUIInfo import_file1_menu_uiinfo[] = {
@@ -1346,7 +1424,7 @@ static GnomeUIInfo settings1_menu_uiinfo[] = {
 	GNOMEUIINFO_SEPARATOR,
 	{
 	 GNOME_APP_UI_ITEM, N_("_Preferences ..."),
-	 N_("Do your prefer coffee to tea? Check it out."),
+	 N_("Do you prefer coffee to tea? Check it out."),
 	 on_set_preferences1_activate, NULL, NULL,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
 	 0, 0, NULL},
@@ -1404,7 +1482,7 @@ static GnomeUIInfo help1_menu_uiinfo[] = {
 	{
 	 GNOME_APP_UI_ITEM, N_("Libraries API references"),
 	 N_("Online reference library for GDK, GLib, GNOME etc.."),
-	 on_url_activate, "http://www.lidn.org", NULL,
+	 on_url_activate, "http://lidn.sourceforge.net", NULL,
 	 GNOME_APP_PIXMAP_NONE, NULL,
 	 0, 0, NULL},
 	{
