@@ -25,6 +25,7 @@
 #include <signal.h>
 #include <string.h>
 #include <sched.h>
+#include <time.h>
 #include <sys/wait.h>
 #include <errno.h>
 
@@ -54,6 +55,8 @@
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 
+void on_toolbar_find_clicked (GtkButton * button, gpointer user_data);
+
 gboolean closing_state;		/* Do not temper with this variable  */
 
 void
@@ -146,6 +149,29 @@ on_close_file1_activate (GtkMenuItem * menuitem, gpointer user_data)
 		anjuta_remove_current_text_editor ();
 }
 
+void
+on_close_all_file1_activate (GtkMenuItem * menuitem, gpointer user_data)
+{
+	GList *node;
+	
+	/* Close all 'saved' files */
+	node = app->text_editor_list;
+	while (node)
+	{
+		TextEditor* te;
+		GList* next;
+		te = node->data;
+		next = node->next; // Save it now, as we may change it.
+		if(te)
+		{
+			if (text_editor_is_saved (te) && te->full_filename)
+			{
+				anjuta_remove_text_editor(te);
+			}
+		}
+		node = next;
+	}
+}
 
 void
 on_reload_file1_activate (GtkMenuItem * menuitem, gpointer user_data)
@@ -281,6 +307,12 @@ on_project1_activate (GtkMenuItem * menuitem, gpointer user_data)
 	anjuta_not_implemented (__FILE__, __LINE__);
 }
 
+void
+on_nonimplemented_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+	anjuta_not_implemented (__FILE__, __LINE__);
+}
+
 
 void
 on_exit1_activate (GtkMenuItem * menuitem, gpointer user_data)
@@ -383,6 +415,78 @@ on_transform_eolchars1_activate (GtkMenuItem * menuitem, gpointer user_data)
 	aneditor_command (te->editor_id, ANE_EOL_CONVERT, 0, 0);
 }
 
+
+void
+on_insert_c_gpl_notice(GtkMenuItem * menuitem, gpointer user_data)
+{
+       TextEditor *te;
+       char *GPLNotice =
+			"/*\n"
+			" *  This program is free software; you can redistribute it and/or modify\n"
+			" *  it under the terms of the GNU General Public License as published by\n"
+			" *  the Free Software Foundation; either version 2 of the License, or\n"
+			" *  (at your option) any later version.\n"
+			" *\n"
+			" *  This program is distributed in the hope that it will be useful,\n"
+			" *  but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+			" *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+			" *  GNU Library General Public License for more details.\n"
+			" *\n"
+			" *  You should have received a copy of the GNU General Public License\n"
+			" *  along with this program; if not, write to the Free Software\n"
+			" *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.\n"
+			" */\n"
+			" \n";
+
+       te = anjuta_get_current_text_editor ();
+       if (te == NULL)
+               return;
+       aneditor_command (te->editor_id, ANE_INSERTTEXT, -1, (long)GPLNotice);
+}
+
+void
+on_insert_cpp_gpl_notice(GtkMenuItem * menuitem, gpointer user_data)
+{
+       TextEditor *te;
+       char *GPLNotice =
+			"// This program is free software; you can redistribute it and/or modify\n"
+			"// it under the terms of the GNU General Public License as published by\n"
+			"// the Free Software Foundation; either version 2 of the License, or\n"
+			"// (at your option) any later version.\n"
+			"//\n"
+			"// This program is distributed in the hope that it will be useful,\n"
+			"// but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+			"// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+			"// GNU Library General Public License for more details.\n"
+			"//\n"
+			"// You should have received a copy of the GNU General Public License\n"
+			"// along with this program; if not, write to the Free Software\n"
+			"// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.\n"
+			"\n";
+
+       te = anjuta_get_current_text_editor ();
+       if (te == NULL)
+               return;
+       aneditor_command (te->editor_id, ANE_INSERTTEXT, -1, (long)GPLNotice);
+}
+
+
+void
+on_insert_date_time(GtkMenuItem * menuitem, gpointer user_data)
+{
+       TextEditor *te;
+       time_t cur_time = time(NULL);
+       char DateTime[256] = {0};
+       
+       sprintf(DateTime,ctime(&cur_time));
+       DateTime[strlen(DateTime)-1] = '\0';    // strip the \n that ctime appends
+               
+       te = anjuta_get_current_text_editor ();
+       if (te == NULL)
+               return;
+       aneditor_command (te->editor_id, ANE_INSERTTEXT, -1, (long)DateTime);
+}
+
 void
 on_autocomplete1_activate (GtkMenuItem * menuitem, gpointer user_data)
 {
@@ -453,7 +557,7 @@ void
 on_goto_line_no1_activate (GtkMenuItem * menuitem, gpointer user_data)
 {
 	GtkWidget *gt;
-	gt = create_goto_line_dialog ();
+	gt = gotoline_new ();
 	gtk_widget_show (gt);
 }
 
@@ -505,6 +609,19 @@ on_edit_app_gui1_activate (GtkMenuItem * menuitem, gpointer user_data)
 	project_dbase_summon_glade (app->project_dbase);
 }
 
+
+void
+on_save_build_messages_activate (GtkMenuItem * menuitem, gpointer user_data)
+{
+	if (app->messages->data[MESSAGE_BUILD])
+		gtk_widget_show (app->save_as_build_msg_sel);
+	else
+		anjuta_error("There are no build messages.");
+
+	return;
+}
+
+
 /***********************************************************************/
 
 void
@@ -588,6 +705,12 @@ on_console1_activate (GtkMenuItem * menuitem, gpointer user_data)
 	anjuta_not_implemented (__FILE__, __LINE__);
 }
 
+void
+on_showhide_locals(GtkMenuItem * menuitem, gpointer user_data)
+{
+	project_dbase_set_show_locals( app->project_dbase, GTK_CHECK_MENU_ITEM (menuitem)->active ) ;
+}
+
 /************************************************************************/
 
 void
@@ -1427,7 +1550,7 @@ on_debugger_start_activate (GtkMenuItem * menuitem, gpointer user_data)
 		prog = project_dbase_get_source_target (app->project_dbase);
 		if (file_is_executable (prog) == FALSE)
 		{
-			anjuta_warning(_("Executable does not exsits this project."));
+			anjuta_warning(_("Executable does not exist for this project."));
 			g_free (prog);
 			prog = NULL;
 		}
@@ -1827,3 +1950,8 @@ on_reload_yes_clicked (GtkButton * button, gpointer te_data)
 	text_editor_load_file ((TextEditor *) te_data);
 	anjuta_update_title ();
 }
+void
+on_findnext1_activate (GtkMenuItem * menuitem, gpointer user_data)
+{
+	on_toolbar_find_clicked ( NULL, NULL );
+}
