@@ -89,6 +89,9 @@ on_file2_activate                      (GtkMenuItem     *menuitem,
 void
 on_project1_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+void
+on_nonimplemented_activate             (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
 
 void
 on_exit1_activate                      (GtkMenuItem     *menuitem,
@@ -212,7 +215,9 @@ on_dump_window1_activate               (GtkMenuItem     *menuitem,
 void
 on_console1_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
-
+void
+on_showhide_locals           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
 /***********************************************************/
 void
 on_main_toolbar1_activate           (GtkMenuItem     *menuitem,
@@ -646,5 +651,21 @@ void
 on_reload_yes_clicked                  (GtkButton       *button,
                                         gpointer         data);
                                         
-                                        
+void
+on_insert_c_gpl_notice(GtkMenuItem * menuitem, gpointer user_data);
+
+
+void
+on_insert_cpp_gpl_notice(GtkMenuItem * menuitem, gpointer user_data);
+
+void
+on_insert_date_time(GtkMenuItem * menuitem, gpointer user_data);
+
+void
+on_save_build_messages_activate (GtkMenuItem * menuitem, gpointer user_data);
+
+void
+on_findnext1_activate                   (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
 #endif
