@@ -98,6 +98,7 @@ struct _PreferencesWidgets
 	GtkWidget *auto_indent_check;
 	GtkWidget *use_tabs_check;
 	GtkWidget *braces_check_check;
+	GtkWidget *dos_eol_check;
 
 	GtkWidget *tab_size_spin;
 	GtkWidget *autosave_timer_spin;
@@ -130,6 +131,8 @@ struct _PreferencesWidgets
 	GtkWidget *tag_pos_radio[4];
 	GtkWidget *no_tag_check;
 	GtkWidget *tags_update_check;
+	/* Page Comps */
+	GtkWidget *use_components;
 };
 
 struct _Preferences
@@ -246,6 +249,7 @@ void preferences_set_build_options(Preferences* p);
 #define INDENT_AUTOMATIC "indent.automatic"
 #define USE_TABS "use.tabs"
 #define BRACES_CHECK "braces.check"
+#define DOS_EOL_CHECK "editor.doseol"
 #define TAB_SIZE "tabsize"
 #define INDENT_SIZE "indent.size"
 #define AUTOSAVE_TIMER "autosave.timer"
@@ -270,6 +274,8 @@ void preferences_set_build_options(Preferences* p);
 #define AUTOMATIC_TAGS_UPDATE "automatic.tags.update"
 
 #define COMMAND_PRINT "command.print"
+#define USE_COMPONENTS "components.enable"
+
 
 void
 ColorFromString (const gchar * val, guint8 * r, guint8 * g, guint8 * b);
