@@ -43,6 +43,11 @@
 #include "compatibility_0.h"
 #include "defaults.h"
 #include "ccview.h"
+#include "glades.h"
+#include "CORBA-Server.h"
+#include "debugger.h"
+#include "find_replace.h"
+#include "executer.h"
 
 /* Including small pixmaps at compile time */
 /* So that these at least work when gnome pixmaps are not found */
@@ -57,6 +62,22 @@
 #include "../pixmaps/file_html.xpm"
 #include "../pixmaps/file_i18n.xpm"
 
+// To debug it
+#define	SHOW_LOCALS_DEFAULT 	TRUE
+
+static void
+project_reload_session_files(ProjectDBase * p);
+static void
+project_dbase_save_session (ProjectDBase * p);
+static void
+project_dbase_reload_session (ProjectDBase * p);
+static void
+project_dbase_save_session_files (ProjectDBase * p);
+static void
+project_dbase_save_markers( ProjectDBase * p, TextEditor *te, const gint nItem );
+
+static const gchar szShowLocalsItem[] = { "ShowLocals" };
+
 static GdkPixmap *opened_folder_pix,
 	*closed_folder_pix,
 	*file_h_pix, *file_c_pix, *file_cpp_pix, *file_pix_pix,
@@ -298,7 +319,7 @@ project_dbase_new (PropsID pr_props)
 
 	p = g_malloc (sizeof (ProjectDBase));
 	if (!p) return NULL;
-
+	p->size = sizeof(ProjectDBase);
 	/* Cannot assign p to static fsd.data, so doing it now */
 	fsd1.data = p;
 	fsd2.data = p;
@@ -308,6 +329,7 @@ project_dbase_new (PropsID pr_props)
 	p->fileselection_open = create_fileselection_gui (&fsd1);
 	p->fileselection_add_file = create_fileselection_gui (&fsd2);
 	p->project_config = project_config_new ();
+	p->tm_project = NULL;
 
 	p->widgets.root_node = NULL;
 	p->widgets.current_node = NULL;
@@ -315,6 +337,7 @@ project_dbase_new (PropsID pr_props)
 	{
 		p->widgets.module_node[i] = NULL;
 	}
+	p->m_prj_ShowLocal = SHOW_LOCALS_DEFAULT ;
 	p->project_is_open = FALSE;
 	p->is_showing = TRUE;
 	p->is_docked = TRUE;
@@ -434,6 +457,7 @@ project_dbase_clear_ctree (ProjectDBase * p)
 	gtk_ctree_remove_node (GTK_CTREE (p->widgets.ctree),
 			       p->widgets.root_node);
 	p->widgets.root_node = NULL;
+	session_sync();
 }
 
 void
@@ -449,6 +473,7 @@ project_dbase_clear (ProjectDBase * p)
 			      _("Project: None"));
 	p->project_is_open = FALSE;
 	p->is_saved = TRUE;
+	p->m_prj_ShowLocal	= SHOW_LOCALS_DEFAULT ;
 	extended_toolbar_update ();
 }
 
@@ -605,6 +630,90 @@ load_from_buffer (ProjectDBase* p, gchar* prj_buff, gint loc)
 	return TRUE;
 }
 
+#if 0
+static void
+project_reload_session_files_v(ProjectDBase * p)
+{
+	gint	nItems, i ;
+	gchar** argvp ;
+	
+	g_return_if_fail( p != NULL );
+
+	session_get_strings( p, SECSTR(SECTION_FILELIST), &nItems, &argvp );
+	for( i = 0 ; i < nItems ; i ++ )
+	{
+		if( argvp[i] )
+		{
+			anjuta_goto_file_line (argvp[i], -1);
+		}
+		g_free( argvp[i] );
+	}
+}
+#endif
+
+static void
+project_reload_session_files(ProjectDBase * p)
+{
+	gpointer	config_iterator;
+	g_return_if_fail( p != NULL );
+
+	config_iterator = session_get_iterator( p, SECSTR(SECTION_FILELIST) );
+	if ( config_iterator !=  NULL )
+	{
+		gchar * szItem, *szFile;
+		while ((config_iterator = gnome_config_iterator_next( config_iterator,
+									&szItem, &szFile )))
+		{
+			// shouldn't happen, but I'm paranoid
+			if( ( NULL != szFile ) && ( NULL != szItem ) )
+			{
+				gint nItem = atoi(szItem);
+				/* Read the line number */
+				glong	gline = session_get_long_n( p, SECSTR(SECTION_FILENUMBER), nItem, -1 );
+				gchar	*szMarkers = session_get_string_n( p, SECSTR(SECTION_FILEMARKERS), nItem, "" );
+				TextEditor * te = anjuta_goto_file_line ( szFile, gline );
+				/*printf( "%d:%s %d id:%ld (%s)\n", nItem, szFile, (int)gline, (long)te->editor_id, szMarkers );*/
+				if( ( NULL != szMarkers ) && ( NULL != te ) )
+				{
+					/* a simple data parsing */
+					gchar	*szData = szMarkers ;
+					gchar	*szEnd ;
+					while( (szEnd = strchr( szData, ',' ) ) != NULL )
+					{
+						gint	nLine = atoi( szData );
+						if( nLine >= 0 )
+						{
+							aneditor_command (te->editor_id, ANE_BOOKMARK_TOGGLE_LINE, nLine, 0);
+							/*printf( "To Editor %ld %d\n", (long)te->editor_id, nLine );*/
+						}
+						szData = szEnd + 1;
+					}
+				}
+				g_free( szMarkers );
+			}
+			g_free( szItem );
+			g_free( szFile );
+		}
+	}
+}
+
+static void
+project_dbase_reload_session (ProjectDBase * p)
+{
+	g_return_if_fail( NULL != p );
+	debugger_reload_session_breakpoints(p);	
+	project_reload_session_files(p);
+
+	find_replace_load_session( app->find_replace, p );
+	executer_load_session( app->executer, p );
+	find_in_files_load_session( app->find_in_files, p );
+	p->m_prj_ShowLocal = session_get_bool( p, SECSTR(SECTION_PROJECTDBASE), szShowLocalsItem, SHOW_LOCALS_DEFAULT );
+	
+	/* Updates the menu */
+	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM (app->widgets.menubar.view.show_hide_locals), p->m_prj_ShowLocal );
+}
+
+
 gboolean
 project_dbase_load_project (ProjectDBase * p, gboolean show_project)
 {
@@ -726,6 +835,9 @@ done:
 	anjuta_set_active ();
 	if (show_project)
 		project_dbase_show (p);
+	project_dbase_reload_session(p);
+	if( IsGladen() )
+		project_dbase_summon_glade ( p );
 	return TRUE;
 
 go_error:
@@ -964,6 +1076,8 @@ project_dbase_save_project (ProjectDBase * p)
 	if (src_paths_save (app->src_paths, fp) == FALSE)
 		goto error_show;
 	tags_manager_save (app->tags_manager);
+	if (p->tm_project)
+		tm_project_save(TM_PROJECT(p->tm_project));
 	ccview_project_save(CCVIEW_PROJECT(app->project_dbase->widgets.ccview));
 	p->is_saved = TRUE;
 	fclose (fp);
@@ -1035,7 +1149,17 @@ project_dbase_update_tags_image(ProjectDBase* p)
 	gchar* src_dir;
 	if (p->project_is_open == FALSE)
 		return;
-	
+	if (p->tm_project)
+	{
+		if (((NULL == TM_PROJECT(p->tm_project)->file_list) ||
+			(0 == TM_PROJECT(p->tm_project)->file_list->len))
+			&& (p->top_proj_dir))
+			tm_project_autoscan(TM_PROJECT(p->tm_project), p->top_proj_dir);
+		else
+			tm_project_update(p->tm_project, FALSE, TRUE, TRUE);
+	}
+	else if (p->top_proj_dir)
+		p->tm_project = tm_project_new(p->top_proj_dir, NULL);
 	src_dir = project_dbase_get_module_dir (p, MODULE_SOURCE);
 	if (src_dir)
 	{
@@ -1051,6 +1175,79 @@ project_dbase_update_tags_image(ProjectDBase* p)
 	}
 }
 
+static void
+project_dbase_save_session_files (ProjectDBase * p)
+{
+	GList	*node;
+	gint	nIndex = 0;
+
+	g_return_if_fail (p != NULL);
+	g_return_if_fail (p->project_is_open == TRUE);
+	
+	/* Save session.... */
+	session_clear_section( p, SECSTR(SECTION_FILELIST) );
+	node = app->text_editor_list;
+	while (node)
+	{
+		TextEditor* te;
+		te = node->data;
+		if(te)
+		{
+			if ( te->full_filename )
+			{
+				session_save_string( p, SECSTR(SECTION_FILELIST), nIndex, te->full_filename );
+				session_save_long_n( p, SECSTR(SECTION_FILENUMBER), nIndex, te->current_line );
+				project_dbase_save_markers( p, te, nIndex );
+				nIndex++;
+			}
+		}
+		node = node->next;
+	}
+}
+
+
+static void
+project_dbase_save_markers( ProjectDBase * p, TextEditor *te, const gint nItem )
+{
+	gint	nLineNo = -1 ;
+	gint	nIndex = 0 ;
+	gint	nMarks;
+	gchar	*szSaveStr;
+
+	g_return_if_fail (p != NULL);
+	g_return_if_fail (te != NULL);
+	nMarks = text_editor_get_num_bookmarks(te);
+	szSaveStr = g_malloc( nMarks*20+2);
+	if( ( NULL != szSaveStr ) && nMarks )
+	{
+		gchar	*sz = szSaveStr ;
+		strcpy( sz, "" );
+		while( ( nLineNo = text_editor_get_bookmark_line( te, nLineNo ) ) >= 0 )
+		{
+			sz += sprintf( sz, "%d,", nLineNo );
+			nIndex ++ ;
+			if( nIndex > nMarks )
+				break;
+		}
+		session_save_string( p, SECSTR(SECTION_FILEMARKERS), nItem, szSaveStr );
+	} else
+		session_save_string( p, SECSTR(SECTION_FILEMARKERS), nItem, "" );
+	g_free( szSaveStr );
+}
+
+static void
+project_dbase_save_session (ProjectDBase * p)
+{
+	debugger_save_session_breakpoints( p );
+	project_dbase_save_session_files ( p );
+	find_replace_save_session( app->find_replace, p );
+	executer_save_session( app->executer, p );
+	find_in_files_save_session( app->find_in_files, p );
+	session_save_bool( p, SECSTR(SECTION_PROJECTDBASE), szShowLocalsItem, p->m_prj_ShowLocal );
+	session_sync();
+}
+
+
 void
 project_dbase_close_project (ProjectDBase * p)
 {
@@ -1078,6 +1275,8 @@ project_dbase_close_project (ProjectDBase * p)
 		}
 	}
 
+	/* Save session.... */
+	project_dbase_save_session(p);
 	/* Close all files that are part of the project */
 	node = app->text_editor_list;
 	while (node)
@@ -1092,17 +1291,23 @@ project_dbase_close_project (ProjectDBase * p)
 			{
 				if (strncmp(te->full_filename, p->top_proj_dir, strlen(p->top_proj_dir)) ==0)
 				{
-					g_print("Closing file %s\n", te->filename);
+					/*g_print("Closing file %s\n", te->filename);*/
 					anjuta_remove_text_editor(te);
 				}
 			}
 		}
 		node = next;
 	}
+	if (p->tm_project)
+	{
+		tm_project_free(p->tm_project);
+		p->tm_project = NULL;
+	}
 	project_dbase_hide (p);
-	project_dbase_update_menu (p);
+	project_dbase_update_menu (p);	
 	project_dbase_clean_left (p);
 	ccview_project_clear(CCVIEW_PROJECT(p->widgets.ccview));
+	p->project_is_open = FALSE;
 }
 
 void
@@ -1375,16 +1580,18 @@ project_dbase_get_version (ProjectDBase * p)
 }
 
 /* project type. */
-gint
+Project_Type*
 project_dbase_get_project_type (ProjectDBase* p)
 {
 	gchar *str;
 	gint i;
 	
-	g_return_val_if_fail (p != NULL, PROJECT_TYPE_END_MARK;);
+	if (p == NULL)
+		return NULL;
+	
 	str = prop_get (p->props, "project.type");
 	if (!str)
-		return PROJECT_TYPE_END_MARK;
+		return NULL;
 
 	for (i=0; i<PROJECT_TYPE_END_MARK; i++)
 	{
@@ -1393,10 +1600,10 @@ project_dbase_get_project_type (ProjectDBase* p)
 		if (ret == 0)
 		{
 			g_free (str);
-			return i;
+			return load_project_type(i);
 		}
 	}
-	return PROJECT_TYPE_END_MARK;
+	return NULL;
 }
 
 /* project language. */
@@ -1406,7 +1613,8 @@ project_dbase_get_language (ProjectDBase* p)
 	gchar *str;
 	gint i;
 	
-	g_return_val_if_fail (p != NULL, PROJECT_PROGRAMMING_LANGUAGE_END_MARK;);
+	g_return_val_if_fail (p != NULL, PROJECT_PROGRAMMING_LANGUAGE_END_MARK);
+	
 	str = prop_get (p->props, "project.programming.language");
 	if (!str)
 		return PROJECT_PROGRAMMING_LANGUAGE_END_MARK;
@@ -1828,6 +2036,8 @@ project_dbase_add_file_to_module (ProjectDBase * p, PrjModule module,
 	g_free (new_file_list);
 	g_free (file_list);
 	g_free (mod_files);
+	if ((MODULE_INCLUDE == module) || (MODULE_SOURCE == module))
+		tm_project_add_file(TM_PROJECT(p->tm_project), filename, TRUE);
 	project_dbase_update_tree (p);
 	p->is_saved = FALSE;
 }
@@ -1837,6 +2047,7 @@ project_dbase_remove_file (ProjectDBase * p)
 {
 	gchar *key, *fn, *files, *pos;
 	gint i;
+	TMWorkObject *source_file;
 	PrjModule module;
 	
 	module = p->current_file_data->module;
@@ -1847,6 +2058,9 @@ project_dbase_remove_file (ProjectDBase * p)
 		g_free (key);
 		return;
 	}
+	source_file = tm_project_find_file(p->tm_project, p->current_file_data->filename);
+	if (source_file)
+		tm_project_remove_object(TM_PROJECT(p->tm_project), source_file);
 	fn = extract_filename (p->current_file_data->filename);
 	
 	pos = strstr (files, fn);
@@ -1877,3 +2091,27 @@ project_dbase_remove_file (ProjectDBase * p)
 	g_free (key);
 	g_free (files);
 }
+
+gchar*
+project_dbase_get_dir (ProjectDBase * p)
+{
+	g_return_val_if_fail( (NULL != p), NULL );
+	return p->top_proj_dir ;
+}
+
+gchar*
+project_dbase_get_name (ProjectDBase * p)
+{
+	g_return_val_if_fail( (NULL != p), NULL );
+	return prop_get (p->props, "project.name");
+}
+
+void
+project_dbase_set_show_locals( ProjectDBase * p,  const gboolean bActive )
+{
+	/* Null can be a valid entry */
+	if( NULL != p )
+	{
+		p->m_prj_ShowLocal = bActive ;
+	}
+}
