@@ -22,6 +22,10 @@
 #include "tags_manager.h"
 #include "properties.h"
 #include "project_config.h"
+#include "project_type.h"
+#include "project_type.h"
+
+#include "tm_tagmanager.h"
 
 #define COMPATIBILITY_LEVEL    1
 
@@ -109,6 +113,7 @@ struct _ProjectDBaseGui
 
 struct _ProjectDBase
 {
+	glong	size;	/* sizeof() used as version # for components */
 	ProjectDBaseGui widgets;
 	GtkWidget *fileselection_open;
 	GtkWidget *fileselection_add_file;
@@ -119,6 +124,7 @@ struct _ProjectDBase
 	guint props;
 
 	gchar *top_proj_dir;
+	TMWorkObject *tm_project;
 	gchar *proj_filename;
 	gboolean is_saved;
 	gboolean is_showing;
@@ -131,6 +137,7 @@ struct _ProjectDBase
 	/* Private */
 	gdouble progress_state;
 	PrjModule sel_module;
+	gboolean	m_prj_ShowLocal;	/* Cfg to show local variables */
 };
 
 extern gchar* module_map[];
@@ -181,6 +188,8 @@ gboolean project_dbase_save_yourself (ProjectDBase * p, FILE * stream);
 
 gboolean project_dbase_load_yourself (ProjectDBase * p, PropsID props);
 
+void project_dbase_set_show_locals( ProjectDBase * p,  const gboolean bActive );
+
 void
 project_dbase_dock (ProjectDBase * p);
 
@@ -209,7 +218,7 @@ gchar *
 project_dbase_get_proj_name (ProjectDBase * p);
 
 /* project type. */
-gint
+Project_Type*
 project_dbase_get_project_type (ProjectDBase* p);
 
 /* Target type*/
@@ -295,6 +304,13 @@ project_dbase_update_tree (ProjectDBase * p);
 void
 project_dbase_update_controls (ProjectDBase * pd);
 
+/* reference */
+gchar*
+project_dbase_get_dir (ProjectDBase * p);
+
+/* copy of string */
+gchar*
+project_dbase_get_name (ProjectDBase * p);
 
 /* Callback signals */
 
