@@ -66,6 +66,21 @@ static glong marker_prop[] =
 	MARKER_PROP_END,	/* end */
 };
 
+static void check_tm_file(TextEditor *te)
+{
+	if (NULL == te->tm_file)
+	{
+		te->tm_file = tm_workspace_find_object(
+		  TM_WORK_OBJECT(app->tm_workspace), te->full_filename);
+		if (NULL == te->tm_file)
+		{
+			te->tm_file = tm_source_file_new(te->full_filename, TRUE);
+			if (NULL != te->tm_file)
+				tm_workspace_add_object(te->tm_file);
+		}
+	}
+}
+
 static void initialize_markers (TextEditor* te)
 {
 	gint i, marker;
@@ -96,8 +111,10 @@ text_editor_new (gchar * filename, TextEditor * parent, Preferences * eo)
 	te = (TextEditor *) g_malloc (sizeof (TextEditor));
 	if (te == NULL)
 		return NULL;
+	te->size	= sizeof(TextEditor);
 	te->filename = g_strdup_printf ("Newfile#%d", ++new_file_count);
 	te->full_filename = NULL;
+	te->tm_file = NULL;
 	te->modified_time = time (NULL);
 	te->preferences = eo;
 	te->force_hilite = TE_LEXER_AUTOMATIC;
@@ -234,6 +251,9 @@ text_editor_destroy (TextEditor * te)
 			g_free (te->filename);
 		if (te->full_filename)
 			g_free (te->full_filename);
+		if (te->tm_file)
+			if (te->tm_file->parent == TM_WORK_OBJECT(app->tm_workspace))
+				tm_workspace_remove_object(te->tm_file, TRUE);
 		if (te->menu)
 			text_editor_menu_destroy (te->menu);
 		if (te->editor_id)
@@ -435,12 +455,174 @@ text_editor_set_line_marker (TextEditor *te, glong line)
 	text_editor_set_marker (te, line, TEXT_EDITOR_LINEMARKER);
 }
 
+/* Support for DOS-Files
+ *
+ * On load, Anjuta will detect DOS-files by finding <CR><LF>. 
+ * Anjuta will translate some chars >= 128 to native charset.
+ * On save the DOS_EOL_CHECK(preferences->editor) will be checked
+ * and chars >=128 will be replaced by DOS-codes, if any translation 
+ * match(see struct tr_dos in this file) and <CR><LF> will used 
+ * instead of <LF>.
+ * The DOS_EOL_CHECK-checkbox will be set on loading a DOS-file.
+ *
+ *  23.Sep.2001  Denis Boehme<boehme at syncio dot de>
+ */
+
+/*
+ * this is a translation table from unix->dos 
+ * this table will be used by filter_chars and save filtered.
+ */
+static struct {
+	unsigned char c; /* unix-char */
+	unsigned char b; /* dos-char */
+} tr_dos[]= {
+	{ '', 0x84 },
+	{ '', 0x8e },
+	{ '', 0xe1 },
+	{ '', 0x81 },
+	{ '', 0x9a },
+	{ '', 0x94 },
+	{ '', 0x99 },
+	{ '', 0x82 },
+	{ '', 0x90 },
+	{ '', 0x9a },
+	{ '', 0xd4 },
+	{ '', 0x88 },
+	{ '', 0xd2 },
+	{ '', 0xa0 },
+	{ '', 0xb5 },
+	{ '', 0x85 },
+	{ '', 0xb7 },
+	{ '', 0x83 },
+	{ '', 0xb6 },
+	{ '', 0xa3 },
+	{ '', 0xe9 },
+	{ '', 0x97 },
+	{ '', 0xeb },
+	{ '', 0x96 },
+	{ '', 0xea }
+};
+
+/*
+ * filter chars in buffer, by using tr_dos-table. 
+ *
+ */
+static size_t
+filter_chars( gchar *data_, size_t size )
+{
+	int k;
+	size_t i;
+	unsigned char * data = data_;
+	unsigned char * tr_map;
+
+	tr_map = (unsigned char *)malloc( 256 );
+	memset( tr_map, 0, 256 );
+	for ( k = 0; k < sizeof(tr_dos )/2 ; k++ )
+		tr_map[tr_dos[k].b] = tr_dos[k].c;
+
+	for ( i = 0; i < size; i++ )
+	{
+      		if ( (data[i] >= 128) && ( tr_map[data[i]] != 0) )
+			data[i] = tr_map[data[i]];;
+	}
+
+	if ( tr_map )
+		free( tr_map );
+
+	return size;
+}
+
+/*
+ * save buffer. filter chars and set dos-like CR/LF if dos_text is set.
+ */
+static size_t
+save_filtered( FILE * f, gchar *data_, size_t size, gint dos_text )
+{
+	size_t i, j;
+	gchar dos_eol_data[] = { 0x0d, 0x0a };
+	unsigned char *data;
+	unsigned char *tr_map;
+	int k;
+
+	/* build the translation table */
+	tr_map = malloc( 256 );
+	memset( tr_map, 0, 256 );
+	for ( k = 0; k < sizeof(tr_dos)/2; k++)
+	  tr_map[tr_dos[k].c] = tr_dos[k].b;
+
+	data = data_;
+	i = 0; j = 0;
+	while ( i < size )
+	{
+		if ( data[i] == 0x0d )
+		{
+			/* ignore CR */
+			i++;
+			continue;
+		}
+		if ( data[i]==0x0a )
+		{
+			/* create CR+LF or LF */
+			if ( dos_text )
+				j += fwrite( dos_eol_data, 1, 2, f );
+			else
+				j += fwrite( &data[i], 1, 1, f );
+			i++;
+		}
+		else if ( data[i]>=128 && dos_text )
+		  {
+		    /* convert dos-text */
+		    if ( tr_map[data[i]] != 0 )
+		      j += fwrite( &tr_map[data[i]], 1, 1, f );
+		    else
+		      /* char not found, skip transform */
+		      j += fwrite( &data[i], 1, 1, f );
+
+		    i++;
+		  }
+		else
+		  {
+			/* write normal chars */
+			j += fwrite( &data[i], 1, 1, f );
+			i++;
+		  }
+		
+	}
+
+	if ( tr_map )
+		free(tr_map);
+
+//	printf( "size: %d, written: %d\n", size, j );
+	return size;
+}
+
+/*
+ * Check end-of-line type. Returns true, if it is DOS <cr><lf> type. 
+ */
+static gboolean
+is_dos_eol ( const gchar * buffer, gint size )
+{
+	int i;
+	int cr = 0;
+	
+	for ( i = 0; i < size ; i++ )
+	{
+		if ( buffer[i] == 0x0a ) // LF
+			break;
+		if ( buffer[i] == 0x0d ) // CR
+			cr = TRUE;
+	}
+	
+	return cr;
+}
+
 static gboolean
 load_from_file (TextEditor * te, gchar * fn)
 {
 	FILE *fp;
 	gchar *buffer;
 	gint nchars;
+	gint dos_text;
 	struct stat st;
 	size_t size;
 
@@ -467,6 +649,14 @@ load_from_file (TextEditor * te, gchar * fn)
 	}
 	/* Crude way of loading, but faster */
 	nchars = fread (buffer, 1, size, fp);
+	/* check EOL type */
+	dos_text = is_dos_eol(buffer,nchars);
+	preferences_set_int ( te->preferences, DOS_EOL_CHECK, 
+			      dos_text );
+	preferences_sync( te->preferences );
+	if ( dos_text )
+	  nchars = filter_chars( buffer, nchars );
+       
 	if (size != nchars)
 		g_warning ("File size and loaded size not matching");
 	scintilla_send_message (SCINTILLA (te->widgets.editor), SCI_ADDTEXT,
@@ -553,12 +743,15 @@ save_to_file (TextEditor * te, gchar * fn)
 	if (data)
 	{
 		size_t size;
-
+		gint dos_text;
+		
 		size = strlen (data);
+		dos_text = preferences_get_int( te->preferences, DOS_EOL_CHECK );
 		if (size != nchars)
 			g_warning
 				("Text length and no. of bytes saved is not equal");
-		size = fwrite (data, size, 1, fp);
+//		size = fwrite (data, size, 1, fp);
+		size = save_filtered( fp, data, size, dos_text );
 		g_free (data);
 	}
 	if (ferror (fp))
@@ -602,6 +795,7 @@ text_editor_load_file (TextEditor * te)
 	{
 		tags_manager_update (app->tags_manager, te->full_filename);
 	}
+	check_tm_file(te);
 	anjuta_set_active ();
 	text_editor_thaw (te);
 	scintilla_send_message (SCINTILLA (te->widgets.editor),
@@ -652,6 +846,9 @@ text_editor_save_file (TextEditor * te)
 			    && tags_update)
 			tags_manager_update (app->tags_manager,
 					     te->full_filename);
+		check_tm_file(te);
+		if (te->tm_file)
+			tm_source_file_update(TM_WORK_OBJECT(te->tm_file), FALSE, FALSE, TRUE);
 		/* we need to update UI with the call to scintilla */
 		text_editor_thaw (te);
 		scintilla_send_message (SCINTILLA (te->widgets.editor),
@@ -723,7 +920,7 @@ text_editor_update_preferences (TextEditor * te)
 }
 
 gboolean
-text_editor_check_disk_status (TextEditor * te)
+text_editor_check_disk_status (TextEditor * te, const gboolean bForce )
 {
 	struct stat status;
 	time_t t;
@@ -745,19 +942,25 @@ text_editor_check_disk_status (TextEditor * te)
 	}
 	if (te->modified_time < status.st_mtime)
 	{
-		buff =
-			g_strdup_printf (_
-					 ("WARNING: The file \"%s\" in the disk is more recent "
-					  "than\nthe current buffer.\nDo you want to reload it."),
-			te->filename);
-		messagebox2 (GNOME_MESSAGE_BOX_WARNING, buff,
-			     GNOME_STOCK_BUTTON_YES,
-			     GNOME_STOCK_BUTTON_NO,
-			     GTK_SIGNAL_FUNC
-			     (on_text_editor_check_yes_clicked),
-			     GTK_SIGNAL_FUNC
-			     (on_text_editor_check_no_clicked), te);
-		return FALSE;
+		if( bForce )
+		{
+			text_editor_load_file (te);
+		} else
+		{
+			buff =
+				g_strdup_printf (_
+						 ("WARNING: The file \"%s\" in the disk is more recent "
+						  "than\nthe current buffer.\nDo you want to reload it."),
+				te->filename);
+			messagebox2 (GNOME_MESSAGE_BOX_WARNING, buff,
+					 GNOME_STOCK_BUTTON_YES,
+					 GNOME_STOCK_BUTTON_NO,
+					 GTK_SIGNAL_FUNC
+					 (on_text_editor_check_yes_clicked),
+					 GTK_SIGNAL_FUNC
+					 (on_text_editor_check_no_clicked), te);
+			return FALSE;
+		}
 	}
 	return TRUE;
 }
@@ -961,3 +1164,24 @@ text_editor_line_from_handle (TextEditor* te, gint marker_handle)
 	return linenum_scintilla_to_text_editor (line);
 }
 
+gint text_editor_get_bookmark_line( TextEditor* te, const gint nLineStart )
+{
+	return aneditor_command (te->editor_id, ANE_GETBOOKMARK_POS, nLineStart, 0 );
+}
+
+
+gint text_editor_get_num_bookmarks(TextEditor* te)
+{
+	gint	nLineNo = -1 ;
+	gint	nMarkers = 0 ;
+	
+	g_return_val_if_fail (te != NULL, 0 );
+
+	while( ( nLineNo = text_editor_get_bookmark_line( te, nLineNo ) ) >= 0 )
+	{
+		//printf( "Line %d\n", nLineNo );
+		nMarkers++;
+	}
+	//printf( "out Line %d\n", nLineNo );
+	return nMarkers ;
+}
