@@ -207,7 +207,7 @@ void
 on_toolbar_help_clicked (GtkButton * button, gpointer user_data)
 {
 	gtk_signal_emit_by_name (GTK_OBJECT
-				 (app->widgets.menubar.help.contents),
+				 (app->widgets.menubar.help.context_help),
 				 "activate");
 }
 
@@ -528,7 +528,7 @@ on_member_combo_list_select_child (GtkList * list,
 }
 
 void
-on_browser_widzard_clicked (GtkButton * button, gpointer user_data)
+on_browser_wizard_clicked (GtkButton * button, gpointer user_data)
 {
 
 }
