@@ -197,7 +197,12 @@ GtkWidget *gnome_filelist_new_with_path(gchar *path)
    gtk_widget_show(scrolled_window);
 
    file_list->file_list = gtk_ctree_new(1, 0);
-   gtk_clist_set_selection_mode(GTK_CLIST(file_list->file_list), GTK_SELECTION_SINGLE);
+	//gnome_filelist_set_selection_mode(file_list, GTK_SELECTION_EXTENDED);
+//	gnome_filelist_set_selection_mode(file_list, GTK_SELECTION_MULTIPLE);
+	gnome_filelist_set_selection_mode(file_list, GTK_SELECTION_SINGLE);
+	file_list->multiple_selection = TRUE;
+//   gtk_clist_set_selection_mode(GTK_CLIST(file_list->file_list), GTK_SELECTION_SINGLE);
+//   gtk_clist_set_selection_mode(GTK_CLIST(file_list->file_list), GTK_SELECTION_MULTIPLE);
    gtk_clist_set_shadow_type(GTK_CLIST(file_list->file_list), GTK_SHADOW_ETCHED_IN);
    gtk_clist_set_column_justification(GTK_CLIST(file_list->file_list), 0, GTK_JUSTIFY_LEFT);
    gtk_clist_set_row_height(GTK_CLIST(file_list->file_list), 18);
@@ -206,6 +211,9 @@ GtkWidget *gnome_filelist_new_with_path(gchar *path)
    gtk_ctree_set_line_style (GTK_CTREE(file_list->file_list), GTK_CTREE_LINES_NONE);
    gtk_clist_column_titles_passive(GTK_CLIST(file_list->file_list));
    gtk_container_add(GTK_CONTAINER(scrolled_window), file_list->file_list);
+ gtk_widget_set_events(file_list->file_list, 
+                                        GDK_KEY_PRESS_MASK);
+
    gtk_signal_connect(GTK_OBJECT(file_list->file_list), "key_press_event", GTK_SIGNAL_FUNC(gnome_filelist_key_press), 0);
    gtk_signal_connect(GTK_OBJECT(file_list->file_list), "select_row", GTK_SIGNAL_FUNC(file_select_event), file_list);
    gtk_signal_connect(GTK_OBJECT(file_list->file_list), "unselect_row", GTK_SIGNAL_FUNC(file_unselect_event), file_list);
@@ -335,7 +343,6 @@ static void file_select_event(GtkCTree *tree, gint row, gint col, GdkEvent *even
       file_list->history_position = -1;
 }
 
-
 static void file_unselect_event(GtkCTree *tree, gint row, gint col, GdkEvent *event, GnomeFileList *file_list)
 {
    file_list->selected_row = -1;   
@@ -496,6 +503,79 @@ gchar *gnome_filelist_get_filename(GnomeFileList *file_list)
    return(full);
 }
 
+GList * gnome_filelist_get_filelist(GnomeFileList * file_list)
+{
+	gchar * path = NULL;
+	gchar * full = NULL;
+	GList * list = NULL;
+	int i;
+	int num_elements;
+	GList * temp = GTK_CLIST(file_list->file_list)->selection;
+	num_elements = g_list_length(temp);
+	
+	i=0;
+	while(num_elements!=0)
+	{
+		gchar *text;
+		GtkCTreeNode * node = g_list_nth_data(temp,i);
+	
+		if(gtk_ctree_node_get_pixtext(GTK_CTREE(file_list->file_list), node, 0, &text, NULL, NULL, NULL))
+		{
+			path = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(file_list->history_combo)->entry));
+			full = g_strconcat(path,text,NULL);
+			list = g_list_append(list, full);
+		}
+//		gtk_ctree_unselect(GTK_CTREE(file_list->file_list), node);
+		temp = GTK_CLIST(file_list->file_list)->selection;
+		num_elements--;
+		i++;
+	}
+	return list;
+}
+
+GList * gnome_filelist_get_nodelist(GnomeFileList * file_list)
+{
+	GList * list = NULL;
+	int num_elements;
+	GList * temp = GTK_CLIST(file_list->file_list)->selection;
+	num_elements = g_list_length(temp);
+	
+	while(num_elements!=0)
+	{
+		GtkCTreeNode * node = g_list_nth_data(temp,0);
+		list = g_list_append(list, node);
+		gtk_ctree_unselect(GTK_CTREE(file_list->file_list), node);
+		temp = GTK_CLIST(file_list->file_list)->selection;
+		num_elements--;
+	}
+	return list;
+}
+
+gchar * gnome_filelist_get_lastfilename(GnomeFileList * file_list, GList * list)
+{
+	GtkCTreeNode * node = g_list_nth_data(list, 0);
+	gchar * filename = NULL;
+	gchar * text = NULL;
+	gchar * full = NULL;
+	gchar * path;
+	
+	if(!gtk_ctree_node_get_pixtext(GTK_CTREE(file_list->file_list), node, 0, &text, NULL, NULL, NULL))
+		return NULL;
+	filename = g_strdup(text);
+	gtk_ctree_select(GTK_CTREE(file_list->file_list), node);
+	path = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(file_list->history_combo)->entry));
+	if(!path)
+		return NULL;
+	g_free(file_list->selected);
+	file_list->selected = g_new(char, strlen(text)+1);
+	strcpy(file_list->selected, text);
+	set_file_selection(file_list);
+	
+	full = g_strconcat(path, filename, NULL);
+	g_free(filename);
+	return full;
+}
+
 gchar *gnome_filelist_get_path(GnomeFileList *file_list)
 {
    g_return_val_if_fail(file_list != NULL, NULL);
@@ -587,6 +667,16 @@ void gnome_filelist_set_title(GnomeFileList *file_list, gchar *title)
    gtk_window_set_title(GTK_WINDOW(file_list), title);
 }
 
+void gnome_filelist_set_selection_mode(GnomeFileList *file_list, GtkSelectionMode mode)
+{
+	g_return_if_fail(file_list != NULL);
+	g_return_if_fail(GNOME_IS_FILELIST(file_list));
+	g_return_if_fail(file_list->file_list != NULL);
+	g_return_if_fail(GTK_IS_CTREE(file_list->file_list));
+	gtk_clist_set_selection_mode(GTK_CLIST(file_list->file_list), mode);
+	/* printf("I am here\n"); */
+}
+
 static gboolean gnome_filelist_check_dir_exists(gchar *path)
 {
    DIR *dir_file;
@@ -694,6 +784,12 @@ static gint gnome_filelist_key_press(GtkWidget *widget, GdkEventKey *event)
       gtk_widget_grab_focus(GTK_WIDGET(clist));
       del = TRUE;
    }
+	else if(event->keyval == GDK_Control_L || event->keyval == GDK_Control_R)
+	{
+		// printf("Control pressed\n");
+		gnome_filelist_set_selection_mode(file_list, GTK_SELECTION_MULTIPLE);
+	}
+	
    if(row >= 0 && row < clist->rows)
    {
       gtk_clist_select_row(clist, row, 0);
